!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for acc related arrays
!
!  Module junctions also contains the following subroutine(s)
!
!     allok8_acc
!
!  ===============================================================================================================================
!
      module accumulators
!
      use kind_specs 
! 
      integer(kind=ikd), save :: & 
          num_acc,  &  !  number of acc
         siz_racc,  &  !  size of the array allocated for the real acc pointer arrays
         siz_iacc,  &  !  size of the array allocated for the integer acc pointer arrays
         siz_lacc      !  size of the array allocated for the logical acc pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_acc(:)
!
!  All of the following real, integer and logical pointer arrays have bounds (1:num_acc)
!  Space is allocated and pointers associations are made in subroutine allok8_acc
!
      real(kind=rkd), pointer, save :: &
       acclev(:), &  !  initial accumulator level                            
       excoef(:), &  !  polytropic gas constant                      
          pvg(:), &  !  constant = po*v**excoeff                     
         tair(:)     !  noncondensable temperature                   
!
      integer(kind=ikd), target, allocatable, save :: i_acc(:)
      integer(kind=ikd), pointer, save :: &
       idvacc(:), &  !  volume index for accumulator volumes         
       gamopt(:)     !  optional flag for gamma calc
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_acc  
!
!  Subroutine to allocate space and set pointers for acc related arrays
!     Note: num_acc must be defined before allok8_acc is called
!
!     allok8_acc is called from subroutine inacc
! 
      use kind_specs 
      use restart_block_list 
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr     
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status , l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_acc .le. 0) then
         write (6,110) num_acc
         call fail
         return
      endif
!      
      if (allocated (r_acc)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_acc, i2, acclev, r_acc)
         call r_pntr (allocatd, num_acc, i2, excoef, r_acc)
         call r_pntr (allocatd, num_acc, i2,    pvg, r_acc)
         call r_pntr (allocatd, num_acc, i2,   tair, r_acc)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_racc   = i2
!
            allocate (r_acc(siz_racc), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_racc, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_acc
            block  = 'r_acc'
            call build_restart_block_list (block, rdum, idum, ldum, siz_racc, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_acc      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_acc, i2, idvacc, i_acc)
         call i_pntr (allocatd, num_acc, i2, gamopt, i_acc)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_iacc   = i2
!
            allocate (i_acc(siz_iacc), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iacc, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_acc
            block  = 'i_acc'
            call build_restart_block_list (block, rdum, idum, ldum, siz_iacc, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_acc      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' accumulator arrays (allok8_acc).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for accumulator arrays (allok8_acc). '/ &
          10x,'Variable num_acc = ',i3, ', must be > 0.')
          
      end subroutine allok8_acc
!
      end module accumulators
