!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for the auxiliary dnb model related arrays
!
!  Module junctions also contains the following subroutine(s)
!
!     allok8_dnb_data
!     allok8_dnb_arrays
!     allok8_dnb_volumes
!     allok8_dnb_energy
!     allok8_dnb_calc
!
!  ===============================================================================================================================
!
      module auxiliary_dnb
!
      use kind_specs 
      use restart_block_list 
! 
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for dnb model dimension and desription related arrays
!
      integer(kind=ikd), save :: & 
         num_dnbr,  &  !  always = 1
         siz_rdnbr,  &  !  size of the array allocated for the real dnbr pointer arrays
         siz_idnbr      !  size of the array allocated for the integer dnbr pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_dnbr(:)
!
!  All of the following real and integer pointer arrays are Rank (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_dnbr
!
      real(kind=rkd), pointer, save :: &
       corfl1(:), &  !  core inlet flow rate                             
       corfl2(:), &  !  core outlet flow rate                            
       corhp1(:), &  !  core inlet enthalpy                              
       corhp2(:), &  !  core outlet enthalpy                             
        cotht(:), &  !  total core height                                
       dnbrco(:), &  !  dnb ratio of average core                        
       dnbrht(:), &  !  dnb ratio at hot spot                            
        dnbrm(:), &  !  min dnbr value                                   
       dnbrmz(:), &  !  dnb ratio at zmin                                
        eqdia(:), &  !  equivalent dia. based on wetted perimeter(ft)    
        fdelh(:), &  !  channel enthalpy adjustment factor               
       fdhcor(:), &  !  adjustment factor on hot bundle mass flux        
        fqeng(:), &  !  eng. heat flux factor                            
        fqunc(:), &  !  heat flux uncertainty factor                     
          frn(:), &  !  nuc. radial heat flux factor                     
        frpow(:), &  !  fraction of power generated in fuel rod          
        heatl(:), &  !  heated channel length                            
        hgrco(:), &  !  linear heat generation rate in core              
       hgrhot(:), &  !  linear heat generation rate at hot spot          
        hgrmd(:)     !  linear heat generation rate at zmdnb             
!
      real(kind=rkd), pointer, save :: &
       hgrmnz(:), &  !  linear heat generation rate at zmin              
        hidia(:), &  !  hydraulic dia based on heated perimeter          
        pitch(:), &  !  rod-rod pitch(ft)                                
         qbar(:), &  !  average heat flux in core                        
       qtzbar(:), &  !  average heat flux in core at time zero           
        rgrid(:), &  !  r grid factor                                    
        rodia(:), &  !  rod diameter(ft)                                 
          tdc(:), &  !  thermal diffusivity parameter                    
       timdnb(:), &  !  time of dnb calculation                          
         tota(:), &  !  total heat transfer area                         
         zhot(:), &  !  height of hot spot                               
         zmax(:), &  !  max. height of fuel for dnb calc.(ft)            
        zmdnb(:), &  !  height of min dnbr                               
         zmin(:)     !  min. height of fuel for dnb calc.(ft)            
!        
      integer(kind=ikd), target, allocatable, save :: i_dnbr(:)
      integer(kind=ikd), pointer, save :: &
         ichf(:), &  !  flag for chf correlation                         
          icw(:), &  !  flag for cold wall                               
        ispgr(:), &  !  grid spacer indicator                            
       jlevel(:), &  !  nodal point numbered from bottom of core         
          lwr(:), &  !  reactor type                                     
           n1(:), &  !  number of core volumes                           
        ndelt(:), &  !  no. of dnb calc. between retran time step        
         ndnb(:), &  !  no. of nodal points for dnb calc.                
          neg(:), &  !  no. of nodal points for energy calc. in core     
          noa(:), &  !  number of axial power profile data sets          
          nuh(:)     !  flag for non-uniform heat flux                   
!          
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for dnb model related arrays (axial power, dnbr, znode)
!
      integer(kind=ikd), save :: & 
          siz_rdnbm,  &  !  size of the array allocated for the real dnbm pointer arrays
          siz_idnbm      !  size of the array allocated for the integer dnbm pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_dnbm(:)
!
!  The following real pointer arrays are Rank (1:noa)
!  Space is allocated and pointers associations are made in subroutine allok8_dnb_arrays
!
      real(kind=rkd), pointer, save :: &
        axial(:), &  !  normalized axial power values       
         ypos(:)     !  relative position of normalized axial power values

!
!  The following real pointer arrays are Rank (1:ndnb)
!  Space is allocated and pointers associations are made in subroutine allok8_dnb_arrays
!
      real(kind=rkd), pointer, save :: &
         dnbr(:), &  !  calculated dnb ratios for the nodal points      
        znode(:)     !  relative position of the nodal points for dnb calculations 
!
!   -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for dnb model average volumes related arrays  
!
      integer(kind=ikd), save :: & 
        siz_idnbv, & !  size of the array allocated for the integer dnb volume pointer arrays
        siz_rdnbv    !  size of the array allocated for the real dnb volume pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_dnbv(:)
!
!  The following real and integer pointer arrays are Rank (1:n1)
!  Space is allocated and pointers associations are made in subroutine allok8_dnb_volumes
!
      real(kind=rkd), pointer, save :: &
        cohtf(:), &  !  heat flux for conductor in core                  
       conode(:), &  !  nodal position in core volumes                   
       corepr(:), &  !  core inlet volume pressure                       
       coresd(:), &  !  core inlet specific volume                       
        corex(:), &  !  inlet core volume quality                        
       cotemp(:), &  !  core fluid temperature                           
       covoht(:), &  !  inlet core volume height                         
          hta(:), &  !  heat transfer area for a conductor in core       
        qcrit(:), &  !  dnb flux for avg. core                           
          sol(:)     !  coefficients for heat flux as func. of position
!
      integer(kind=ikd), target, allocatable, save :: i_dnbv(:)
!      
      integer(kind=ikd), pointer, save :: &
       nvldnb(:)     !  core volume number       
!       
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for dnb energy calculation related arrays
!
      integer(kind=ikd), save :: &
        siz_idnbe, & !  size of the array allocated for the integer dnb energy calculation pointer arrays
        siz_rdnbe    !  size of the array allocated for the real dnb energy calculation pointer arrays      
!  
      real(kind=rkd), target, allocatable, save :: r_dnbe(:)
!      
!  The following real pointer arrays are Rank (1:neg)
!  Space is allocated and pointers associations are made in subroutine allok8_dnb_energy
!
      real(kind=rkd), pointer, save :: &
          fzn(:), &  !  axial hot channel factor                         
         gdnb(:), &  !  local mass flux corrected for energy calc.       
         gflo(:), &  !  mass flux at nodal point for energy calc.        
         heng(:), &  !  enthalpy at nodal point for energy calc.         
           hf(:), &  !  enthalpy of saturated liquid                     
          hfg(:), &  !  latent heat                                      
        prsre(:), &  !  pressure at nodal point for energy calc.         
        qqeng(:), &  !  heat flux at nodal point for energy calc.        
        qtzfz(:), &  !  heat flux as function of height at time zero     
          rho(:), &  !  density at nodal point for energy calc.          
          spa(:), &  !  specific volume at nodal point                   
         tten(:), &  !  temperature at nodal point                       
        xqual(:), &  !  quality at nodal point for energy calc.          
         xten(:), &  !  quality at nodal point                           
         zeng(:)     !  nodal point height for energy calc.                      
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for dnb calculation related arrays
!
      integer(kind=ikd), save :: &
        siz_idnbc, & !  size of the array allocated for the integer dnb calculation pointer arrays
        siz_rdnbc    !  size of the array allocated for the real dnb calculation pointer arrays      
!        
      real(kind=rkd), target, allocatable, save :: r_dnbc(:)
!      
!  The following real pointer arrays are Rank (1:jlevel)
!  Space is allocated and pointers associations are made in subroutine allok8_dnb_calc
!
      real(kind=rkd), pointer, save :: &
           bb(:), &  !  heat flux calculated for non-uniform factor calc.
         dens(:), &  !  density of fluid at local point                  
           gt(:), &  !  mass flux at local point                         
         hdnb(:), &  !  enthalpy at local point                          
        hfdnb(:), &  !  enthalpy of saturated liquid at local point      
       hfgdnb(:), &  !  latent heat for dnb calc.                        
          prs(:), &  !  local pressure                                   
           qq(:), &  !  heat flux for dnb calc.                          
           xm(:), &  !  heat flux / length   for non-uniform factor calc.
           xq(:), &  !  quality at local point                           
         zdnb(:)     !  nodal point for dnb calc.                        
!         
      contains
!
!  ===============================================================================================================================
      subroutine allok8_dnb_data
!
!  Subroutine to allocate space and set pointers for dnb model dimension related arrays
!     allok8_dnb_data is called from subroutine indnb
! 
      use kind_specs
      use restart_block_list 
! 
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_dnbr)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_dnbr = 1
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_dnbr, i2, corfl1, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, corfl2, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, corhp1, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, corhp2, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  cotht, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, dnbrco, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, dnbrht, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  dnbrm, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, dnbrmz, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  eqdia, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  fdelh, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, fdhcor, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  fqeng, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  fqunc, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,    frn, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  frpow, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  heatl, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  hgrco, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, hgrhot, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  hgrmd, r_dnbr)
!         
         call r_pntr (allocatd, num_dnbr, i2, hgrmnz, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  hidia, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  pitch, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,   qbar, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, qtzbar, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  rgrid, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  rodia, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,    tdc, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2, timdnb, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,   tota, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,   zhot, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,   zmax, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,  zmdnb, r_dnbr)
         call r_pntr (allocatd, num_dnbr, i2,   zmin, r_dnbr)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rdnbr   = i2
!
            allocate (r_dnbr(siz_rdnbr), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rdnbr, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_dnbr
            block  = 'r_dnbr'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rdnbr, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_dnbr      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_dnbr, i2,   ichf, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,    icw, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,  ispgr, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2, jlevel, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,    lwr, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,     n1, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,  ndelt, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,   ndnb, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,    neg, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,    noa, i_dnbr)
         call i_pntr (allocatd, num_dnbr, i2,    nuh, i_dnbr)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_idnbr   = i2
!
            allocate (i_dnbr(siz_idnbr), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_idnbr, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_dnbr
            block  = 'i_dnbr'
            call build_restart_block_list (block, rdum, idum, ldum, siz_idnbr, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_dnbr      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' auxiliary dnb model arrays (allok8_dnb_data).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_dnb_data
!
!  ===============================================================================================================================
      subroutine allok8_dnb_arrays
!
!  Subroutine to allocate space and set pointers for dnb model related arrays
!     allok8_dnb_arrays is called from subroutine indnb
! 
      use kind_specs
      use restart_block_list 
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (.not.allocated (r_dnbr)) then
         write (6,110) 
         call fail
         return
      endif
!      
      if (allocated (r_dnbm)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd,    noa(1), i2,  axial, r_dnbm)
         call r_pntr (allocatd,    noa(1), i2,   ypos, r_dnbm)
!
         call r_pntr (allocatd,   ndnb(1), i2,   dnbr, r_dnbm)
         call r_pntr (allocatd,   ndnb(1), i2,  znode, r_dnbm)        
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rdnbm   = i2
!
            allocate (r_dnbm(siz_rdnbm), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rdnbm, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_dnbm
            block  = 'r_dnbm'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rdnbm, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_dnbm      = 0.0d0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' dnb model related arrays (allok8_dnb_arrays).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for dnb model related arrays(allok8_dnb_arrays).'/ &
          10x,'Subroutine allok8_dnb_arrays called before subroutine allok8_dnb - programming error') 
      end subroutine allok8_dnb_arrays
!
!  ===============================================================================================================================
!
      subroutine allok8_dnb_volumes
!
!  Subroutine to allocate space and set pointers for dnb model dimension related arrays
!     allok8_dnb_data is called from subroutine indnb
! 
      use kind_specs
      use restart_block_list 
! 
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (.not.allocated (r_dnbr)) then
         write (6,110) 
         call fail
         return
      endif
!      
      if (allocated (r_dnbv)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd,    n1(1), i2,  cohtf, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2, conode, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2, corepr, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2, coresd, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2,  corex, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2, cotemp, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2, covoht, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2,    hta, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2,  qcrit, r_dnbv)
         call r_pntr (allocatd,    n1(1), i2,    sol, r_dnbv)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rdnbv   = i2
!
            allocate (r_dnbv(siz_rdnbv), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rdnbv, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_dnbv
            block  = 'r_dnbv'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rdnbv, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_dnbv      = 0.0d0
         end if
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd,    n1(1), i2, nvldnb, i_dnbv) 
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_idnbv   = i2
!
            allocate (i_dnbv(siz_idnbv), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_idnbv, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_dnbv
            block  = 'i_dnbv'
            call build_restart_block_list (block, rdum, idum, ldum, siz_idnbv, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_dnbv      = 0
         end if
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' dnb volume arrays (allok8_dnb_volume).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for dnb volume arrays(allok8_dnb_volume).'/ &
          10x,'Subroutine allok8_dnb_volume called before subroutine allok8_dnb - programming error') 
!
     end subroutine allok8_dnb_volumes
!     
!  ===============================================================================================================================
!
      subroutine allok8_dnb_energy
!
!  Subroutine to allocate space and set pointers for dnb model dimension related arrays
!     allok8_dnb_data is called from subroutine indnb
! 
      use kind_specs
      use restart_block_list 
! 
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (.not.allocated (r_dnbr)) then
         write (6,110) 
         call fail
         return
      end if
!      
      if (allocated (r_dnbe)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd,    neg(1), i2,    fzn, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   gdnb, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   gflo, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   heng, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,     hf, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,    hfg, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,  prsre, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,  qqeng, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,  qtzfz, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,    rho, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,    spa, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   tten, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,  xqual, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   xten, r_dnbe)
         call r_pntr (allocatd,    neg(1), i2,   zeng, r_dnbe)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rdnbe   = i2
!
            allocate (r_dnbe(siz_rdnbe), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rdnbe, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_dnbe
            block  = 'r_dnbe'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rdnbe, 1, 2)
!
         else
            r_dnbe      = 0.0d0
         end if
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' dnb energy arrays (allok8_dnb_energy).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for dnb energy arrays(allok8_dnb_energy).'/ &
          10x,'Subroutine allok8_dnb_energy called before subroutine allok8_dnb - programming error') 
!
     end subroutine allok8_dnb_energy
!     
!  ===============================================================================================================================
!
      subroutine allok8_dnb_calc
!
!  Subroutine to allocate space and set pointers for dnb model dimension related arrays
!     allok8_dnb_data is called from subroutine indnb
! 
      use kind_specs
      use restart_block_list 
! 
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (.not.allocated (r_dnbr)) then
         write (6,110) 
         call fail
         return
      end if
!      
      if (allocated (r_dnbc)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, jlevel(1), i2,     bb, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,   dens, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,     gt, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,   hdnb, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,  hfdnb, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2, hfgdnb, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,    prs, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,     qq, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,     xm, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,     xq, r_dnbc)
         call r_pntr (allocatd, jlevel(1), i2,   zdnb, r_dnbc)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rdnbc   = i2
!
            allocate (r_dnbc(siz_rdnbc), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rdnbc, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_dnbc
            block  = 'r_dnbc'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rdnbc, 1, 2)         
!
!     initialize real arrays to 0.0d0
!
         else
            r_dnbc      = 0.0d0
         end if
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' dnb calculation arrays (allok8_dnb_calc).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for dnb calculation arrays(allok8_dnb_calc).'/ &
          10x,'Subroutine allok8_dnb_calc called before subroutine allok8_dnb - programming error') 
!
     end subroutine allok8_dnb_calc
!     
!  ===============================================================================================================================
!
      end module auxiliary_dnb

