!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications for auxiliary file format arrays
!
!  This module also includes the following subroutine
!
!     allok8_aux_file_format
!
!  ===============================================================================================================================
!
      module auxiliary_file
!
!  The following information is defined in subroutine inedit
!
      use kind_specs
!
      integer(kind=ikd), save  :: &
          num_ufmt, &  ! The number of 8 character words that comprise the user supplied format string
              i60hdr   ! Flag indicating if a header is to be written to the auxiliary file - 1 =  header - 0 = no header
      character(kind=ckd,len=8), save, target, allocatable :: &
           ufmt(:)     ! The user supplied format string used to write the auxiliary file
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_aux_file_format
!
!  Subroutine to allocate space and set pointer for auxiliary file format array
!     Note: num_ufmt must be defined before allok8_aux_file_format is called
!
!     allok8_aux_file_format is called from subroutine inedit
!
      use kind_specs
! 
      implicit none
      integer(kind=ikd) l, len, i2, status, allocatd
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space 
!  -------------------------------------------------------------------------------------------------------------------------------
!
!
      if (allocated (ufmt)) then
         return
      else
         allocate (ufmt(num_ufmt), stat=status)
!
!     Check status of allocate - write error if allocate failed
!
         if (status .ne. 0) then
            write (6,100) num_ufmt, status
            call fail
         endif
!
!     Initialize real arrays to blank
!
         ufmt      = '            '
      endif
!
      return
!
  100 format (' ******** Error allocating space for auxiliary file user format array (allok8_aux_file_format).'/ &
           10x,'Length requested = ',i8/ &
           10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_aux_file_format
!
      end module auxiliary_file
