!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for chan related arrays
!
!  This module also includes the following subroutines
!
!      allok8_chan
!      allok8_ch_axial
!      allok8_ch_para
!      allok8_ch_crossflow
!      allok8_ch_vol
!      allok8_ch_jun
!      allok8_ch_cond
!      allok8_ch_reg
!      allok8_3dgeom
!      allok8_3dsub
!
!  ===============================================================================================================================
!
      module channels
!
      use kind_specs 
      use restart_block_list 
! 
      integer(kind=ikd), save :: & 
          num_chan,  &  !  number of chan
         siz_rchan,  &  !  size of the array allocated for the real chan pointer arrays
         siz_ichan,  &  !  size of the array allocated for the integer chan pointer arrays
         siz_lchan      !  size of the array allocated for the logical chan pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_chan(:)
!
!  All of the following real, integer and logical pointer arrays have bounds (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_chan
!
      real(kind=rkd), pointer, save :: &
       chexpn(:), &  !  expansion factor for conductor geometries         
       elevch(:), &  !  elevation (bottom of core)                        
         ewch(:), &  !  wall roughness                                    
       qbmdch(:), &  !  bypass direct moderator heating frac              
       qdmdch(:), &  !  direct moderator heating decay frac               
       qpmdch(:), &  !  direct moderator heating prompt frac              
       velrch(:), &  !  moc  velocity ratio                               
       wcrech(:)     !  total core flow                                   
!
      integer(kind=ikd), target, allocatable, save :: i_chan(:)
      integer(kind=ikd), pointer, save :: &
        ieqch(:), &  !  5-equations flag                                  
        ihqch(:), &  !  enthalpy transport indicator                      
       imcrch(:), &  !  heat transfer correlation indicator               
        impch(:), &  !  5-equations volume orientation flag               
       jmpjch(:), &  !  2phase friction flag                              
        mocch(:), &  !  moc indicator/ min part resolution                
       nbypss(:), &  !  number of bypass flow sets                        
       nbypsv(:), &  !  number of bypass channels in cdi file             
       nchanr(:), &  !  number of channels                                
       ncross(:), &  !  number of cross flow sets                         
       ngapch(:), &  !  number of gap conductance models                  
       ngeoch(:), &  !  number of new geometries                          
        ninpn(:), &  !  number of unheated inlet planes                   
       njunch(:), &  !  number of junction geometry types                 
       nlckch(:), &  !  number of fibwr lateral leakage types             
       nmatch(:), &  !  number of new materials                           
       noutpn(:), &  !  number of unheated outlet planes                  
       nsetls(:), &  ! 
       nvolch(:), &  !  number of volume geometry types                   
       nwatch(:), &  !  number of water tube types                        
       nzonch(:)     !  number of nodes per channel                       
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for axial parameter arrays
!
      integer(kind=ikd), save :: & 
         num_axial, &   !  number of nodes per channel 
        siz_raxial      !  size of the array allocated for the real axial parameter pointer arrays
!         
!  The flch real pointer array is Rank (1:num_axial)  
!  Space is allocated and pointers associations are made in subroutine allok8_ch_axial

      real(kind=rkd), target, allocatable, save :: r_axial(:)
      real(kind=rkd), pointer, save :: &
         flch(:)     !  node height
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for channel parameter arrays
!
      integer(kind=ikd), save :: &
          num_chpara,  &  !  number of channels 
         siz_rchpara,  &  !  size of the array allocated for the real chan parameter pointer arrays
         siz_ichpara      !  size of the array allocated for the integer chan parameter pointer arrays
!
!  The following pointer array are Ranked (1:num_chpar)  
!  Space is allocated and pointers associations are made in subroutine allok8_chpara
!
      integer(kind=ikd), target, allocatable, save :: i_chpara(:)
      integer(kind=ikd), pointer, save :: &
       ibypch(:), &  ! 
       ifitch(:), &  !  profile fit model for channel                     
       ifrich(:), &  !  laminar friction set for channel                  
       igapch(:), &  !  gap conductance model for channel                 
       igeoch(:), &  !  conduction geometry number                        
       ijunch(:), &  !  junction geometry type                            
       ilckch(:), &  !  lateral leakage path type                         
        inchn(:), &  !  inlet volume to channel                           
       ioutch(:), &  !  outlet volume from channel                        
       ivolch(:), &  !  volume geometry type                              
       iwatch(:), &  !  water tube geometry type                          
       kflgch(:), &  !  unspecified loss flag              
        sumch(:)     !  count of cdi th channels               
!
      real(kind=rkd), target, allocatable, save :: r_chpara(:)
      real(kind=rkd), pointer, save :: &
       wjunch(:), &  !  channel flow (norm./real)                         
       wlckch(:), &  !  channel lateral leakage flow                      
       wwatch(:)     !  channel water tube flow                           
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for channel cross flow arrays
!
      integer(kind=ikd), save :: &
          num_chcross,  &  !  number of cross flow sets 
         siz_rchcross,  &  !  size of the array allocated for the real chan cross flow pointer arrays
         siz_ichcross      !  size of the array allocated for the integer chan cross flow pointer arrays
!
!  The following pointer arrays are Ranked (1:ncross)  
!  Space is allocated and pointers associations are made in subroutine allok8_chcross
!
      integer(kind=ikd), target, allocatable, save :: i_chcross(:)
      integer(kind=ikd), pointer, save :: &      
        incrs(:), &  !  inlet channel number                              
       ioutcr(:)     !  outlet channel number                             

      real(kind=rkd), target, allocatable, save :: r_chcross(:)     
      real(kind=rkd), pointer, save :: &      
       cntcrs(:), &  !  cross-flow centriod distance                      
       dhycrs(:), &  !  cross-flow hydraulic diameter                     
        facrs(:), &  !  cross-flow area                                      
       rkfcrs(:), &  !  cross-flow forward loss coefficient               
       rkrcrs(:), &  !  cross-flow reverse loss coefficient               
       wjncrs(:)     !  cross-flow initial flow rate                             
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for channel volume arrays
!
      integer(kind=ikd), save :: & 
          num_chvol, & !  number of volumes 
         siz_rchvol    !  size of the array allocated for the real chan volume pointer arrays
!
!  The following pointer arrays are Ranked (1:num_chvol)  
!  Space is allocated and pointers associations are made in subroutine allok8_chvol
!       
      real(kind=rkd), target, allocatable, save :: r_chvol(:)     
      real(kind=rkd), pointer, save :: &      
       chfarea(:), &  ! flow area
       chwetpe(:), &  ! wetted perimeter
       chhetpe(:), &  ! heated perimeter
       chcdvol(:), &  ! conductor volume
      chwrough(:)     ! volume surface roughness  
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for channel junction arrays       
!
      integer(kind=ikd), save :: & 
          num_chjun, &  !  number of junctions
         siz_rchjun     !  size of the array allocated for the real chan junction pointer arrays
!
!  The following pointer arrays are Ranked (1 : num_chjun )  
!  Space is allocated and pointers associations are made in subroutine allok8_chjun
!
      real(kind=rkd), target, allocatable, save :: r_chjun(:)     
      real(kind=rkd), pointer, save :: &      
      chjfarea(:), &  ! flow area
      chjwetpe(:), &  ! wetted perimeter
      chfwloss(:), &  ! foreward loss coefficient
      chbwloss(:)     ! reverse loss coefficient
!             
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for channel conductor (fuel geometry) arrays
!
      integer(kind=ikd), save :: &
          num_chcond, & !  number of fuel geometry
         siz_rchcond, & !  size of the array allocated for the real chan fuel geometry (conductor) pointer arrays
         siz_ichcond    !  size of the array allocated for the integer chan fuel geometry (conductor) pointer arrays
         
!
!  The following pointer arrays are Ranked (1:num_chcond)  
!  Space is allocated and pointers associations are made in subroutine allok8_chcond

      real(kind=rkd), target, allocatable, save :: r_chcond(:)     
      real(kind=rkd), pointer, save :: &               
         xocdi(:)       !  radial distance to the left surface
!
      integer(kind=ikd), target, allocatable, save :: i_chcond(:)
      integer(kind=ikd), pointer, save :: & 
        igpcdi(:), &    !  gap conductance index
         nrcdi(:)       !  number of material regions
!
!  Region related arrays
!
      integer(kind=ikd), save :: &
          sum_chreg, &  !  sum of regions for all geometries
         siz_rchreg, &  !  size of the array allocated for the real region pointer arrays
         siz_ichreg     !  size of the array allocated for the integer region pointer arrays              
!               
!  The following pointer arrays are Ranked (1:sum_chreg)   where sum_chreg is sum of number of material regions of all fuel geometries
!  Space is allocated and pointers associations are made in subroutine allok8_chreg
!
      real(kind=rkd), target, allocatable, save :: r_chreg(:)     
      real(kind=rkd), pointer, save :: &               
           pfcdi(:), &  !  power fraction
           xrcdi(:)     !  region width    
!
      integer(kind=ikd), target, allocatable, save :: i_chreg(:)
      integer(kind=ikd), pointer, save :: & 
         imtcdi(:), &   ! material index
         ndxcdi(:), &   ! number of space steps
         ngpcdi(:)      ! gap indicator
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for 3d geometry arrays
      integer(kind=ikd), save :: &
          num_3dgeom, & !  number of 3d geometry = 1
         siz_r3dgeom, & !  size of the array allocated for the real chan fuel geometry (conductor) pointer arrays
         siz_i3dgeom    !  size of the array allocated for the integer chan fuel geometry (conductor) pointer arrays
!
!  The following pointer arrays are Ranked (1:1)  
!  Space is allocated and pointers associations are made in subroutine allok8_3dgeom

      real(kind=rkd), target, allocatable, save :: r_3dgeom(:)     
      real(kind=rkd), pointer, save :: &               
         qbmod(:)       !  bypass direct moderator heating fraction
!
      integer(kind=ikd), target, allocatable, save :: i_3dgeom(:)
      integer(kind=ikd), pointer, save :: & 
        nbypnn(:), &  !   number of bypass channels                        
        nchann(:), &  !   number of thermal hydraulic channels             
        ngrpnn(:), &  !   number of control rod groups                     
        nlcoln(:), &  !   number of columns of assembly mesh               
        nlrown(:), &  !   number of rows of assembly mesh                  
        nplane(:), &  !   number of axial core planes                      
        npnbot(:), &  !   number of unheated planes below active fuel      
        npntop(:), &  !   number of unheated planes above active fuel      
        nreflb(:), &  !   number of reflector planes below core            
        nreflt(:)     !   number of reflector planes above core     
!               
!  Specifications for 3d geometry sub arrays
      integer(kind=ikd), save :: &
         num_asmbly, & !  # of assembly
         num_bypnod, & !  # of nodes in bypass channel
         siz_i3dsub    !  size of the array allocated for the integer chan fuel geometry (conductor) pointer arrays
!
!  Space is allocated and pointers associations are made in subroutine allok8_3dsub
!
      integer(kind=ikd), target, allocatable, save :: i_3dsub(:)
      integer(kind=ikd), pointer, save :: & 
        icchan(:), &  !   index of first channel conductor size = nchann
        ivchan(:), &  !   index of first channel volume size = nchann
        ivlbot(:), &  !   index of first channel bottom reflector volume  size = nchann
        ivlbyp(:), &  !   index of first bypass channel volume size = # of bypass channel nodes 
        ivltop(:), &  !   index of first channel top reflector volume size = nchann 
        kunfld(:)     !   channel to assembly mapping array -  size = nlcoln * nlrown    
!    
!  ===============================================================================================================================
!                 
      contains
!
!  ===============================================================================================================================
!
      subroutine allok8_chan  
!
!  Subroutine to allocate space and set pointers for chan related arrays
!     allok8_chan is called from subroutine XXXXX 
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chan)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_chan  =  1
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chan, i2, chexpn, r_chan)
         call r_pntr (allocatd, num_chan, i2, elevch, r_chan)
         call r_pntr (allocatd, num_chan, i2,   ewch, r_chan)
         call r_pntr (allocatd, num_chan, i2, qbmdch, r_chan)
         call r_pntr (allocatd, num_chan, i2, qdmdch, r_chan)
         call r_pntr (allocatd, num_chan, i2, qpmdch, r_chan)
         call r_pntr (allocatd, num_chan, i2, velrch, r_chan)
         call r_pntr (allocatd, num_chan, i2, wcrech, r_chan)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rchan   = i2
!
            allocate (r_chan(siz_rchan), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchan, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chan
            block  = 'r_chan'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchan, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chan      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_chan, i2,  ieqch, i_chan)
         call i_pntr (allocatd, num_chan, i2,  ihqch, i_chan)
         call i_pntr (allocatd, num_chan, i2, imcrch, i_chan)
         call i_pntr (allocatd, num_chan, i2,  impch, i_chan)
         call i_pntr (allocatd, num_chan, i2, jmpjch, i_chan)
         call i_pntr (allocatd, num_chan, i2,  mocch, i_chan)
         call i_pntr (allocatd, num_chan, i2, nbypss, i_chan)
         call i_pntr (allocatd, num_chan, i2, nbypsv, i_chan)
         call i_pntr (allocatd, num_chan, i2, nchanr, i_chan)
         call i_pntr (allocatd, num_chan, i2, ncross, i_chan)
         call i_pntr (allocatd, num_chan, i2, ngapch, i_chan)
         call i_pntr (allocatd, num_chan, i2, ngeoch, i_chan)
         call i_pntr (allocatd, num_chan, i2,  ninpn, i_chan)
         call i_pntr (allocatd, num_chan, i2, njunch, i_chan)
         call i_pntr (allocatd, num_chan, i2, nlckch, i_chan)
         call i_pntr (allocatd, num_chan, i2, nmatch, i_chan)
         call i_pntr (allocatd, num_chan, i2, noutpn, i_chan)
         call i_pntr (allocatd, num_chan, i2, nsetls, i_chan)
         call i_pntr (allocatd, num_chan, i2, nvolch, i_chan)
         call i_pntr (allocatd, num_chan, i2, nwatch, i_chan)
         call i_pntr (allocatd, num_chan, i2, nzonch, i_chan)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ichan   = i2
!
            allocate (i_chan(siz_ichan), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ichan, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_chan
            block  = 'i_chan'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ichan, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_chan      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' chan arrays (allok8_chan).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!          
      end subroutine allok8_chan
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_ch_axial
!
!  Subroutine to allocate space and set pointers for chan related arrays
!     allok8_axial is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_axial)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_axial, i2, flch, r_axial)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_raxial   = i2
!
            allocate (r_axial(siz_raxial), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_raxial, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_axial
            block  = 'r_axial'
            call build_restart_block_list (block, rdum, idum, ldum, siz_raxial, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_axial      = 0.0d0
         endif
      enddo
!
      return
!      
  100 format (' ******** Error allocating space for ',a8,' chan axial arrays (allok8_ch_axial).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan axial arrays (allok8_ch_axial).'/ &
          10x,'Subroutine allok8_ch_axial called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_axial
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_ch_para
!      
!  Subroutine to allocate space and set pointers for chan related arrays
!     allok8_axial is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chpara)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chpara, i2, wjunch, r_chpara)
         call r_pntr (allocatd, num_chpara, i2, wlckch, r_chpara)
         call r_pntr (allocatd, num_chpara, i2, wwatch, r_chpara)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rchpara   = i2
!
            allocate (r_chpara(siz_rchpara), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchpara, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chpara
            block  = 'r_chpara'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchpara, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chpara      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_chpara, i2, ibypch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ifitch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ifrich, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, igapch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, igeoch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ijunch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ilckch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2,  inchn, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ioutch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, ivolch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, iwatch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2, kflgch, i_chpara)
         call i_pntr (allocatd, num_chpara, i2,  sumch, i_chpara)

!     allocate space
!
         if (.not.allocatd) then
            siz_ichpara   = i2
!
            allocate (i_chpara(siz_ichpara), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ichpara, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_chpara
            block  = 'i_chpara'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ichpara, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_chpara      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' chan parameter arrays (allok8_ch_para).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan parameter arrays (allok8_ch_para).'/ &
          10x,'Subroutine allok8_ch_para called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_para
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_ch_crossflow
!      
!  Subroutine to allocate space and set pointers for chan cross flow pointer arrays
!     allok8_axial is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chcross)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chcross, i2, cntcrs, r_chcross)
         call r_pntr (allocatd, num_chcross, i2, dhycrs, r_chcross)
         call r_pntr (allocatd, num_chcross, i2,  facrs, r_chcross)
         call r_pntr (allocatd, num_chcross, i2, rkfcrs, r_chcross)
         call r_pntr (allocatd, num_chcross, i2, rkrcrs, r_chcross)                                    
         call r_pntr (allocatd, num_chcross, i2, wjncrs, r_chcross)         
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rchcross   = i2
!
            allocate (r_chcross(siz_rchcross), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchcross, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chcross
            block  = 'r_chcross'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchcross, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chcross      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_chcross, i2,  incrs, i_chcross)                                    
         call i_pntr (allocatd, num_chcross, i2, ioutcr, i_chcross)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ichcross   = i2
!
            allocate (i_chcross(siz_ichcross), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ichcross, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_chcross
            block  = 'i_chcross'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ichcross, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_chcross      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' chan cross flow arrays (allok8_ch_crossflow).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan cross flow arrays (allok8_ch_crossflow).'/ &
          10x,'Subroutine allok8_ch_crossflow called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_crossflow
!
!  -------------------------------------------------------------------------------------------------------------------------------      
!      
      subroutine allok8_ch_vol
!      
!  Subroutine to allocate space and set pointers for chan volume related arrays
!     allok8_ch_vol is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chvol)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chvol, i2,  chfarea, r_chvol)
         call r_pntr (allocatd, num_chvol, i2,  chwetpe, r_chvol)
         call r_pntr (allocatd, num_chvol, i2,  chhetpe, r_chvol)
         call r_pntr (allocatd, num_chvol, i2,  chcdvol, r_chvol)
         call r_pntr (allocatd, num_chvol, i2, chwrough, r_chvol)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rchvol   = i2
!
            allocate (r_chvol(siz_rchvol), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchvol, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chvol
            block  = 'r_chvol'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchvol, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chvol = 0.0d0
         endif
      enddo
!
      return
!      
  100 format (' ******** Error allocating space for ',a8,' chan vol arrays (allok8_ch_vol).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan vol arrays (allok8_ch_vol).'/ &
          10x,'Subroutine allok8_ch_vol called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_vol
!
!  -------------------------------------------------------------------------------------------------------------------------------      
!      
      subroutine allok8_ch_jun
!      
!  Subroutine to allocate space and set pointers for chan jun related arrays
!     allok8_ch_jun is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chjun)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chjun, i2,  chjfarea, r_chjun)
         call r_pntr (allocatd, num_chjun, i2,  chjwetpe, r_chjun)
         call r_pntr (allocatd, num_chjun, i2,  chfwloss, r_chjun)
         call r_pntr (allocatd, num_chjun, i2,  chbwloss, r_chjun)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rchjun   = i2
!
            allocate (r_chjun(siz_rchjun), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchjun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chjun
            block  = 'r_chjun'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchjun, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chjun = 0.0d0
         endif
      enddo
!
      return
!      
  100 format (' ******** Error allocating space for ',a8,' chan jun arrays (allok8_ch_jun).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan jun arrays (allok8_ch_jun).'/ &
          10x,'Subroutine allok8_ch_jun called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_jun
!
!  -------------------------------------------------------------------------------------------------------------------------------            
!
      subroutine allok8_ch_cond
!
!  Subroutine to allocate space and set pointers for chan cross flow pointer arrays
!     allok8_axial is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chcond)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_chcond, i2, xocdi, r_chcond)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rchcond   = i2
!
            allocate (r_chcond(siz_rchcond), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchcond, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chcond
            block  = 'r_chcond'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchcond, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chcond      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_chcond, i2, igpcdi, i_chcond)                                    
         call i_pntr (allocatd, num_chcond, i2,  nrcdi, i_chcond)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ichcond   = i2
!
            allocate (i_chcond(siz_ichcond), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ichcond, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_chcond
            block  = 'i_chcond'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ichcond, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_chcond      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' chan fuel geometry arrays (allok8_ch_cond).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan fuel geom arrays (allok8_ch_cond).'/ &
          10x,'Subroutine allok8_ch_cond called before subroutine allok8_chan - programming error') 
!
      end subroutine allok8_ch_cond
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_ch_reg
!
!  Subroutine to allocate space and set pointers for chan cross flow pointer arrays
!     allok8_axial is called from subroutine inchan
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_chreg)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_chan)) then
         write (6,110) 
         call fail
         return
      endif
!     
      sum_chreg  =  0
!
      do l = 1, num_chcond
         sum_chreg = sum_chreg + nrcdi(l)
      enddo
!
      if (sum_chreg .le.0) then
         write (6,120) sum_chreg
         call fail
         return
      endif
!             
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, sum_chreg, i2, pfcdi, r_chreg)
         call r_pntr (allocatd, sum_chreg, i2, xrcdi, r_chreg)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rchreg   = i2
!
            allocate (r_chreg(siz_rchreg), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rchreg, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_chreg
            block  = 'r_chreg'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rchreg, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_chreg      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, sum_chreg, i2, imtcdi, i_chreg)                                    
         call i_pntr (allocatd, sum_chreg, i2, ndxcdi, i_chreg)                                    
         call i_pntr (allocatd, sum_chreg, i2, ngpcdi, i_chreg)                                                      
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ichreg   = i2
!
            allocate (i_chreg(siz_ichreg), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ichreg, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_chreg
            block  = 'i_chreg'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ichreg, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_chreg      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' chan geometry region arrays (allok8_ch_reg).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
  110 format (' ******** Error allocating space for chan geometry region arrays (allok8_ch_reg).'/ &
          10x,'Subroutine allok8_ch_reg called before subroutine allok8_chan - programming error') 
!
  120 format (' ******** Error allocating space for chan geometry region arrays (allok8_ch_reg).'/ &
          10x,'Sum of material region of all fuel geometry = ',i6,', must be > 0 - programming error') 
!
      end subroutine allok8_ch_reg      
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_3dgeom  
!
!  Subroutine to allocate space and set pointers for 3dgeo related arrays
!     allok8_3Dgeo is called from subroutine geom3d 
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_3dgeom)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_3dgeom, i2,  qbmod, r_3dgeom)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_r3dgeom   = i2
!
            allocate (r_3dgeom(siz_r3dgeom), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_r3dgeom, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_3dgeom
            block  = 'r_3dgeom'
            call build_restart_block_list (block, rdum, idum, ldum, siz_r3dgeom, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_3dgeom      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_3dgeom, i2, nbypnn, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nchann, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, ngrpnn, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nlcoln, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nlrown, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nplane, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, npnbot, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, npntop, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nreflb, i_3dgeom)
         call i_pntr (allocatd, num_3dgeom, i2, nreflt, i_3dgeom)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_i3dgeom   = i2
!
            allocate (i_3dgeom(siz_i3dgeom), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_i3dgeom, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_3dgeom
            block  = 'i_3dgeom'
            call build_restart_block_list (block, rdum, idum, ldum, siz_i3dgeom, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_3dgeom      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' 3dgeo arrays (allok8_3dgeo).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!          
      end subroutine allok8_3dgeom
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
      subroutine allok8_3dsub  
!
!  Subroutine to allocate space and set pointers for 3dgeo related arrays
!     allok8_3Dgeo is called from subroutine geom3d 
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_3dsub)) return
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_asmbly = nlcoln(1) * nlrown(1)
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd,  nchann(1), i2, icchan, i_3dsub)
         call i_pntr (allocatd,  nchann(1), i2, ivchan, i_3dsub)
         call i_pntr (allocatd,  nchann(1), i2, ivlbot, i_3dsub)
         call i_pntr (allocatd, num_bypnod, i2, ivlbyp, i_3dsub)
         call i_pntr (allocatd,  nchann(1), i2, ivltop, i_3dsub)
         call i_pntr (allocatd, num_asmbly, i2, kunfld, i_3dsub)
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_i3dsub   = i2
!
            allocate (i_3dsub(siz_i3dsub), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_i3dsub, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_3dsub
            block  = 'i_3dsub'
            call build_restart_block_list (block, rdum, idum, ldum, siz_i3dsub, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_3dsub      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' 3dsub arrays (allok8_3dsub).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!          
      end subroutine allok8_3dsub
!
!  -------------------------------------------------------------------------------------------------------------------------------\
!
      end module channels
!
!  -------------------------------------------------------------------------------------------------------------------------------
