!$Header$
!$Copyright$
!  ===============================================================================================================================
!  Specifications and space allocations for overall control system related arrays
!
!  This module also includes the following subroutines
!
!      allok8_csub
!      allok8_ctrl
!      allok8_csuper
!      allok8_cidx
!
!  ===============================================================================================================================
!
      module control_system
!
      use kind_specs 
      use restart_block_list 
! 
      integer(kind=ikd), save :: & 
         num_csys,  &  !  number of csys = 1
        siz_rcsys,  &  !  size of the array allocated for the real csys pointer arrays
        siz_icsys,  &  !  size of the array allocated for the integer csys pointer arrays
        siz_lcsys      !  size of the array allocated for the logical csys pointer arrays
!
      real(kind=rkd), save :: &
        cnteps,  &  !  control system solution convergence criteria
        dtcmax,  &  !  max time-step size allowed for the control system
         dtcnt,  &  !  dt used by control system        
         etime      !  elapsed problem time
!
      integer(kind=ikd), save :: &
         idmax,  &  !  positive maximum value assigned as a control block identifier
         idmin,  &  !  negative minimum value assigned as a control block identifier
         iioff,  &  !  offset to the control system quantities
          init,  &  !  initialization flag
         itmax,  &  !  max number of iteration for control system solution convergence
           ncb,  &  !  number of control block specified
           nci,  &  !  number of control input specified
        ncstor,  &  !  number of words of auxiliary storage in array named box
           nts      !  number of actual time steps
!    
!
!  All of the following real, integer and logical pointer arrays have bounds (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_csys
!
!  ===============================================================================================================================
!  Specifications and space allocations for subfile idc and boxx arrays
!  ===============================================================================================================================
!
      integer(kind=ikd), save :: & 
          num_idc,  &  !
        siz_icsub      !  size of the array allocated for the integer csub pointer arrays
!    
      integer(kind=ikd), target, allocatable, save :: i_csub(:)
!      
!  boxx integer pointer arrays have bound (1:ncstor), idc integer pointer arrays have bounds (1: (idmax-idmin+1) )
!  Space is allocated and pointers associations are made in subroutine allok8_csub      
!
      integer(kind=ikd), pointer, save :: &
           idc(:)      !  card order array, determines the calculation order for control
!
!  ===============================================================================================================================
!  Specifications and space allocations for ctrl block id related arrays
!  ===============================================================================================================================
!
      integer(kind=ikd), save :: & 
          num_ctrl,  &  !  
         siz_rctrl,  &  !  size of the array allocated for the real ctrl pointer arrays
         siz_ictrl,  &  !  size of the array allocated for the integer ctrl pointer arrays
         siz_lctrl      !  size of the array allocated for the logical ctrl pointer arrays
         
!
      real(kind=rkd), target, allocatable, save :: r_ctrl(:)
!
!  All of the following real, integer and logical pointer arrays have bounds (1:(nci+ncb))
!  Space is allocated and pointers associations are made in subroutine allok8_ctrl
!
      real(kind=rkd), pointer, save :: &
        acout(:), &  !  control block output, last control iteration     
        cgain(:), &  !  control block gain                               
         cmax(:), &  !  maximum allowed value of control block output    
         cmin(:), &  !  minimum allowed value of control block output    
         cout(:), &  !  control block output                             
          cp1(:), &  !  floating point control block parameter 1         
          cp2(:), &  !  floating point control block parameter 2         
        ocout(:), &  !  control block output, old hydraulic time step    
        pcout(:)     !  control block output, old control time step      
!
      integer(kind=ikd), target, allocatable, save :: i_ctrl(:)
      integer(kind=ikd), pointer, save :: &
       idcedt(:), &  !  control input or block number of editting        
       idxbox(:), &  !  region number for control input 
                     !  pointer for boxx storage array for control block                 
         inc1(:), &  !  integer control block parameter 1, usually used  
         inc2(:), &  !  integer control block parameter 2         
        itype(:), &  !  block type                                       
         nbox(:)     !  number of regions in each super block cards
!
!  boxx arrays is used for FNG, DLY and STF. they are allocated along with super blocks
!
      real(kind=rkd), pointer, save :: &
         boxx(:)     ! boxx arrays              
!
!  ===============================================================================================================================
!  Specifications and space allocations for subfile for super blocks
!  ===============================================================================================================================
!
      type superblock
         real(kind=rkd)    :: &
            me_var    !  minor edit variable
         integer(kind=ikd) :: &
            region    !  region # 
      end type superblock
!
      integer(kind=ikd), save :: &
         num_super,  &   ! number of superblock data supplied by control system model
                         ! total number of space allocated for super blocks is 2 * num_super                         
         size_icsuper    ! size of the array allocated for the integer superblock pointer arrays
!
      type(superblock), target, allocatable, save :: csuper(:)             
!                          
!  ===============================================================================================================================
!  Specifications and space allocations for subfile for storing pointer and offset for control block/input variable
!  ===============================================================================================================================
!
      integer(kind=ikd), save :: & 
         num_cidx, &  !  number of points for index related pointers
        siz_icidx     !  size of the array allocated for the integer index pointer arrays         
!
!  All of the following integer pointer arrays have bounds (1:num_cidx)
!  icreg stores index of region number of control block/input, and icoff stores the offset
!  num_cidx is sum of total number of control inputs and total number of superblock entries
!  There is no separate index into icoff and icreg arrays, for control input index is sequantial i.e. 1 for 1st, 2 for 2nd
!  for super blocks, its index is stored into idxbox array, hence index into icoff and icreg will be nci + idxbox(i)
!  Space is allocated and pointers associations are made in subroutine allok8_cidx
         
      integer(kind=ikd), target, allocatable, save :: i_cidx(:)
      
      integer(kind=ikd), pointer, save :: &      
         icoff(:), &  !  pointer offset for control block region number index 
         icreg(:)     !  index of control block input region number              
!
      contains
!
!  ===============================================================================================================================
!
      subroutine allok8_csub
!
!  Subroutine to allocate space and set pointers for csub related arrays
!     allok8_csub is called from subroutine incnt1
! 
      use kind_specs 
      use restart_block_list 
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_csub)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns pointer arrays.
!  -------------------------------------------------------------------------------------------------------------------------------
!    
      num_idc   =  idmax - idmin
!
      if (num_idc .le. 0) then
         write (6,120) num_idc
         call fail
         return
      endif
!            
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_idc, i2,   idc, i_csub)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_icsub   = i2
!
            allocate (i_csub(siz_icsub), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_icsub, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_csub
            block  = 'i_csub'
            call build_restart_block_list (block, rdum, idum, ldum, siz_icsub, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_csub      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' control system sub arrays (allok8_csub).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for control system sub arrays (allok8_csub).'/ &
          10x,'Subroutine allok8_csub called before subroutine allok8_csys - programming error') 
  120 format (' ******** Error allocating space for control system sub arrays (allok8_csub).'/ &
          10x,'Sum of number of idc for all blocks = ',i6,', must be > 0 - programming error') 
!                
      end subroutine allok8_csub
!      
!  ===============================================================================================================================
!
      subroutine allok8_ctrl  
!
!  Subroutine to allocate space and set pointers for ctrl related arrays
!     allok8_ctrl is called from subroutine incnt1 
! 
      use kind_specs 
      use restart_block_list 
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!  
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_ctrl)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------    
!
      if (num_ctrl .le. 0) then
         write (6,120) num_ctrl
         call fail
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_ctrl, i2,  acout, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,  cgain, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,   cmax, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,   cmin, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,   cout, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,    cp1, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,    cp2, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,  ocout, r_ctrl)
         call r_pntr (allocatd, num_ctrl, i2,  pcout, r_ctrl)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rctrl   = i2
!
            allocate (r_ctrl(siz_rctrl), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rctrl, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_ctrl
            block  = 'r_ctrl'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rctrl, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_ctrl      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_ctrl, i2, idcedt, i_ctrl)
         call i_pntr (allocatd, num_ctrl, i2, idxbox, i_ctrl)
         call i_pntr (allocatd, num_ctrl, i2,   inc1, i_ctrl)
         call i_pntr (allocatd, num_ctrl, i2,   inc2, i_ctrl)
         call i_pntr (allocatd, num_ctrl, i2,  itype, i_ctrl)
         call i_pntr (allocatd, num_ctrl, i2,   nbox, i_ctrl)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ictrl   = i2
!
            allocate (i_ctrl(siz_ictrl), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ictrl, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_ctrl
            block  = 'i_ctrl'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ictrl, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_ctrl      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' ctrl arrays (allok8_ctrl).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for control block arrays (allok8_ctrl).'/ &
          10x,'Subroutine allok8_ctrl called before subroutine allok8_csys - programming error') 
  120 format (' ******** Error allocating space for control blcok arrays (allok8_ctrl).'/ &
          10x,'Sum of number of blocks = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_ctrl
!
!  ===============================================================================================================================            
!
      subroutine allok8_csuper
!     
      use kind_specs
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      if (allocated(csuper) ) return
!
      if (num_super .le. 0) then
         write (6,120) num_super
         call fail
         return
      endif
!
!     allocate superblock derived data type arrays
!
      allocate (csuper(2*num_super), stat=status)
!
      if (status .ne. 0) then
          block      = 'type(superblock)'
          write (6,100) block, num_super, status
          call fail
      endif      
!
!     allocate boxx arrays
!
      allocate (boxx(2*ncstor), stat=status)
!
      if (status .ne. 0) then
          block      = 'boxx'
          write (6,101) block, ncstor, status
          call fail
      endif      
      
      
      return
!                
  100 format (' ******** Error allocating space for ',a8,' superblock arrays (allok8_csuper).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  101 format (' ******** Error allocating space for ',a8,' boxx arrays (allok8_csuper).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)          
  120 format (' ******** Error allocating space for super control blcok arrays (allok8_csuper).'/ &
          10x,'Sum of number of blocks = ',i6,', must be > 0 - programming error') 
!      
      end subroutine allok8_csuper
!
!  ===============================================================================================================================            
!
      subroutine allok8_cboxx
!     
      use kind_specs
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      if (allocated(csuper) ) return
!
      if (ncstor .le. 1) then
         write (6,120) ncstor
         call fail
         return
      endif
!
!     allocate boxx arrays
!
      allocate (boxx(2*ncstor), stat=status)
!
      if (status .ne. 0) then
          block      = 'boxx'
          write (6,101) block, ncstor, status
          call fail
      endif      
!            
      return
!                
  101 format (' ******** Error allocating space for ',a8,' boxx arrays (allok8_cboxx).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)          
  120 format (' ******** Error allocating space for boxx control blcok arrays (allok8_cboxx).'/ &
          10x,'Number of auxiliary word = ',i6,', must be > 0 - programming error') 
!      
      end subroutine allok8_cboxx
!
!  ===============================================================================================================================            
!
      subroutine allok8_cidx
!     
!  Subroutine to allocate space and set pointers for ctrl system index related arrays
!     allok8_cindex is called from subroutine incnt2
!
      use kind_specs
      use restart_block_list       
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!  
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_cidx)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------    
!!
      if (num_cidx .le. 0) then
         write (6,120) num_cidx
         call fail
         return
      end if
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_cidx, i2,   icoff, i_cidx)         
         call i_pntr (allocatd, num_cidx, i2,   icreg, i_cidx)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_icidx   = i2
!
            allocate (i_cidx(siz_icidx), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_icidx, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_cidx
            block  = 'i_cidx'
            call build_restart_block_list (block, rdum, idum, ldum, siz_icidx, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_cidx      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' ctrl arrays (allok8_ctrl).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for control block arrays (allok8_ctrl).'/ &
          10x,'Subroutine allok8_ctrl called before subroutine allok8_csys - programming error') 
  120 format (' ******** Error allocating space for control blcok arrays (allok8_ctrl).'/ &
          10x,'Sum of number of blocks = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_cidx
!   
!  ===============================================================================================================================            
!
      end module control_system
!
