!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for fibwr model related arrays
!
!  This module also includes the following subroutines
!
!     allok8_fibwr  
!     allok8_fibwr1  
!
!  ===============================================================================================================================
!
      module fibwr_data
!
      use kind_specs 
! 
      implicit none
! 
!  Shared fibwr related data
!     
      real(kind=rkd), save :: &
        fcrod  , &  !  control rod drive flow (lbm/sec)   
        rhoref , &  !  fibwr reference density (lbm/ft**3)
        wrxtot      !  total reactor flow                 
!          
      integer(kind=ikd), save :: &
        idxlpi , &  !  index for lower plenum inlet junction                 
        idxlpv , &  !  index for lower plenum volume                         
        idxupv , &  !  index for upper plenum volume                         
        inicnd , &  !  initial condition flag  --  =1, use total core flow                                
                    !                              =2, use core pressure drop                             
        nactfp , &  !  number of active fuel planes                          
        nbyplk , &  !  plane number containing last (highest) leakage path   
        nchan  , &  !  nubmer of active channels                             
        ninpl  , &  !  number of planes below active fuel                    
        npath  , &  !  fibwr option flag (=0, no fibwr, >0 use fibwr)           
        npbnd  , &  !  number of fibwr lateral leakage paths                 
        npcom  , &  !  nubmer of fibwr axial leakage paths                   
        noutpl      !  number of planes above active fuel                    
!
      integer(kind=ikd), save :: & 
            num_fibwr,  &  !  number of fibwr model coefficient sets
           siz_rfibwr,  &  !  size of the array allocated for the real fibwr pointer arrays
           siz_ifibwr      !  size of the array allocated for the integer fibwr pointer arrays
!
!  Allocated fibwr related arrays
!
      real(kind=rkd), target, allocatable, save :: r_fibwr(:)
!
!  The real lateral equation pointer arrays have bounds (1:npbnd) and the bypass equation
!  point arrays have bounds (1:npcom).  
!  Space is allocated and pointer associations are made in subroutine allok8_fibwr
!
      real(kind=rkd), pointer, save :: &
!
            c1b(:), &  !  lateral leakage flow equation coefficient               
            c2b(:), &  !  lateral leakage flow equation coefficient               
            c3b(:), &  !  lateral leakage flow equation coefficient               
            c4b(:), &  !  lateral leakage flow equation coefficient               
          bpper(:), &  !  lateral leakage flow multiplier                         
          belev(:), &  !  lateral leakage flow path elevation (channel model only)
                                                                             
            c1c(:), &  !  bypass flow equation coefficient                        
            c2c(:), &  !  bypass flow equation coefficient                        
            c3c(:), &  !  bypass flow equation coefficient                        
            c4c(:), &  !  bypass flow equation coefficient                        
          cpper(:)     !  bypass flow multiplier 
                                           
!
!  The integer arrays idxchi through idxwrv have bounds (1:nchan).  
!  Space is allocated and pointers associations are made in subroutine allok8_fibwr
!
      integer(kind=ikd), target, allocatable, save :: i_fibwr(:)
      integer(kind=ikd), pointer, save :: &
           idxchi(:), &  !  index to channel inlet junction                          
           idxafi(:), &  !  index to junction above last leakage or water rod exit   
                         !  (flow for all jun here & above = flow in jun idxafo)     
           idxcho(:), &  !  index to channel outlet junction                         
           idxafo(:), &  !  index to junction that defines the active channel flow   
                         !  (flow def junction - chan exit or jun into wr return vol)
           idxwri(:), &  !  index of water rod inlet juncion for channel             
           idxwro(:), &  !  index of water rod outlet juncion for channel            
           idxwrv(:)     !  index of water rod volume for channel                    
!
!  The idxll and idxli integer arrays have bounds (1:9,1:nchan).  The first index is for the nth lateral leakage 
!  junction.  
!  Space is allocated and pointers associations are made in subroutine allok8_fibwr
!
      integer(kind=ikd), target, allocatable, save :: &
       idxll(:,:) , &  !  index to nth lateral leakage junction for channel
       idxli(:,:)      !  index to inlet junction for volume where nth lateral leakage junction flow exits channel 
!
!  ===================================================================================================================
!
!  Array wlevel has bounds of (1:nbyplk).   nbyplk = max (idxafi - idxchi) (may extend into the fueled region)
!  Space is allocated and pointer associations are made in subroutine allok8_fibwr1
!
      real(kind=rkd), target, allocatable, save ::  r_wlevel(:)        
!
      real(kind=rkd), pointer, save :: &
         wlevel(:)     !  lateral leakage flow into this bypass level     
!
!  Integer array idxbpj has bounds of (1:nbyplk+2).  nbyplk = max (idxafi - idxchi) (may extend into the fueled region)

!  Space is allocated and pointers associations are made in subroutine allok8_fibwr1
!
      integer(kind=ikd), target, allocatable, save :: &
         idxbpj(:)     !  index for inlet junction to bypass volume.  There are nbyplk planes (volumes).  Two 
                       !  additional elelments are used to save the index for the junction exiting plane nbyplk  
                       !  and the other is for the bypass exit junction index.   
!
!  Integer array idxvji has bounds of (1:nn,1:nchan) where nn is nbyplk = max (idxafi - idxchi) for all channels.
!  Space is allocated and pointers associations are made in subroutine allok8_fibwr1
!
!      integer(kind=ikd), target, allocatable, save :: &
      integer(kind=ikd), pointer, save :: &
      idxvji(:,:)      !  index of inlet junction for each volume in the leakage flow region for each channel
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_fibwr  
!
!  Subroutine to allocate space and set pointers for most of the fibwr model related arrays
!     Note:  npbnd, npcom and nchan must be defined before allok8_fibwr is called.
!            
!     allok8_fibwr is called from subroutine ifibwr
! 
      use kind_specs 
      use restart_block_list
! 
      use s_r_pntr
      use s_i_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_fibwr)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2               
         i2       = 0
!
         call r_pntr (allocatd, npbnd, i2,     c1b, r_fibwr)
         call r_pntr (allocatd, npbnd, i2,     c2b, r_fibwr)
         call r_pntr (allocatd, npbnd, i2,     c3b, r_fibwr)
         call r_pntr (allocatd, npbnd, i2,     c4b, r_fibwr)
         call r_pntr (allocatd, npbnd, i2,   bpper, r_fibwr)
         call r_pntr (allocatd, npbnd, i2,   belev, r_fibwr)
         call r_pntr (allocatd, npcom, i2,     c1c, r_fibwr)
         call r_pntr (allocatd, npcom, i2,     c2c, r_fibwr)
         call r_pntr (allocatd, npcom, i2,     c3c, r_fibwr)
         call r_pntr (allocatd, npcom, i2,     c4c, r_fibwr)
         call r_pntr (allocatd, npcom, i2,   cpper, r_fibwr)
!                                               
!     Allocate space                           
!                                                    
         if (.not.allocatd) then
            siz_rfibwr   = i2
!
            allocate (r_fibwr(siz_rfibwr), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rfibwr, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_fibwr
            block  = 'r_fibwr'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rfibwr, 1, 0)
!
!     initialize real arrays to 0.0d0
!
         else
            r_fibwr      = 0.0d0
         end if
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2               
         i2       = 0
!
         call i_pntr (allocatd, nchan, i2, idxchi, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxafi, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxcho, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxafo, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxwri, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxwro, i_fibwr)
         call i_pntr (allocatd, nchan, i2, idxwrv, i_fibwr)                                                      
!                                               
!     Allocate space                           
!                                                    
         if (.not.allocatd) then
            siz_ifibwr   = i2
!
            allocate (i_fibwr(siz_ifibwr), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ifibwr, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_fibwr
            block  = 'i_fibwr'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ifibwr, 2, 0)
!
!     initialize real arrays to 0.0d0
!
         else
            i_fibwr      = 0.0d0
         end if
      end do
!
!  Allocate space for idxll and idxli arrays - these arrays are not added to the restart file and
!  are not included in the block list.  They will be recreated during restart initialization.
!
      allocate (idxll(9,nchan), idxli(9,nchan), stat=status)
!
!  Check status of allocate - write error if allocate failed
!
      if (status .ne. 0) then
         block      = '2D intgr'
         len        = 2*(9*nchan)
         write (6,100) block, len, status
         call fail
      endif
!
!  Initialize integer arrays to 0
!
      idxll  = 0
      idxli  = 0
!
  100 format (' ******** Error allocating space for ',a8,' fibwr arrays (allok8_fibwr).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_fibwr
!
!  ===============================================================================================================================
      subroutine allok8_fibwr1  
!
!  Subroutine to allocate space and set pointers for remainder of fibwr model related arrays.  None of the 
!  arrays allocated are included in the restart file.  They will be recreated during restart initialization.
!     Note:  nbyplk must be defined before allok8_fibwr1 is called
!            
!     allok8_fibwr1 is called from subroutine ifibwr
! 
      use kind_specs 
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, status
!
      if (allocated (r_wlevel)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space for wlevel array - this array is not added to the restart file and
!  are not included in the block list.  They will be used for restart jobs which do not call steady state.
!  -------------------------------------------------------------------------------------------------------------------------------
!
      allocate (wlevel(nbyplk), stat=status)
!
!  Check status of allocate - write error if allocate failed
!
      if (status .ne. 0) then
         block      = 'wlevel'
         write (6,100) block, nbyplk, status
         call fail
      endif
!
!  Initialize real arrays to 0.0d0
!
      wlevel = 0.0d0

!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space for idxbpj and idxvji arrays - these arrays are not added to the restart file and
!  are not included in the block list.  They will be recreated during restart initialization.
!  -------------------------------------------------------------------------------------------------------------------------------
!
      len    = nbyplk + 2
!      
      allocate (idxbpj(len), stat=status)
!
!  Check status of allocate - write error if allocate failed
!
      if (status .ne. 0) then
         block      = 'integer'
         write (6,100) block, len, status
         call fail
      endif
!
      allocate (idxvji(nbyplk,nchan), stat=status)
!
!  Check status of allocate - write error if allocate failed
!
      if (status .ne. 0) then
         block      = '2D intgr'
         len        = nbyplk*nchan
         write (6,100) block, len, status
         call fail
      endif
!
!  Initialize integer arrays to 0
!
      idxbpj = 0
      idxvji = 0
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' fibwr arrays (allok8_fibwr1).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_fibwr1
!
      end module fibwr_data
