!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for gtrn related arrays
!
!  This module also includes the following subroutines
!
!     allok8_gtrn 
!
!  ===============================================================================================================================
!
      module general_transport
!
      use kind_specs 
      use restart_block_list 
! 
      integer(kind=ikd), save :: & 
        num_gvol,  &  !  number of general transport volumes
        num_gjun,  &  !  number of general transport junctions
       siz_rgtrn,  &  !  size of the array allocated for the real gtrn pointer arrays
       siz_igtrn      !  size of the array allocated for the integer gtrn pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_gtrn(:)
!
!  All of the following real, integer and logical pointer arrays have bounds (1:num_gvol) or (1:num_gjun)
!  Space is allocated and pointers associations are made in subroutine allok8_gtrn
!
      real(kind=rkd), pointer, save :: &
       addmso(:), &  !  old value of addmss                        
       addmss(:), &  !  mass added for negative volume mass        
        cjocv(:), &  !  volume concentration multipier kj for      
       conjun(:), &  !  junction concentration                     
        convo(:), &  !  old value on convol                        
       convol(:), &  !  volume concentration                       
       gmmssn(:), &
       gmmsso(:), &
        ttrnm(:), &  !  volume transport material mass             
       ttrnmo(:)     !  old value of ttrnm                         
!
      integer(kind=ikd), target, allocatable, save :: i_gtrn(:)
      integer(kind=ikd), pointer, save :: &
       ivtrns(:), &  !  old volume number                          
       ngsjcc(:), &  !  address of junction concentration or 0     
       ngsjce(:), &  !  address of junction efficiency or 0        
       ngsvcb(:)     !  address of source of 0 for no source       
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_gtrn  
!
!  Subroutine to allocate space and set pointers for gtrn related arrays
!     allok8_gtrn is called from subroutine ingtrn
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_gtrn)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_gvol, i2, addmso, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2, addmss, r_gtrn)
         call r_pntr (allocatd, num_gjun, i2,  cjocv, r_gtrn)
         call r_pntr (allocatd, num_gjun, i2, conjun, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2,  convo, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2, convol, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2, gmmssn, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2, gmmsso, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2,  ttrnm, r_gtrn)
         call r_pntr (allocatd, num_gvol, i2, ttrnmo, r_gtrn)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rgtrn   = i2
!
            allocate (r_gtrn(siz_rgtrn), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rgtrn, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_gtrn
            block  = 'r_gtrn'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rgtrn, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_gtrn      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_gvol, i2, ivtrns, i_gtrn)
         call i_pntr (allocatd, num_gjun, i2, ngsjcc, i_gtrn)
         call i_pntr (allocatd, num_gjun, i2, ngsjce, i_gtrn)
         call i_pntr (allocatd, num_gvol, i2, ngsvcb, i_gtrn)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_igtrn   = i2
!
            allocate (i_gtrn(siz_igtrn), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_igtrn, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_gtrn
            block  = 'i_gtrn'
            call build_restart_block_list (block, rdum, idum, ldum, siz_igtrn, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_gtrn      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' gtrn arrays (allok8_gtrn).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_gtrn
!
      end module general_transport
