!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for INP card arrays
!
!  This module also includes the following subroutines
!
!     allok8_inp_card_arrays
!     idata
!
!  ===============================================================================================================================
!
      module inp_cards
!
      use kind_specs
!
      external link  ! added for gnu f95 where link is apparently an internal function
!
      integer(kind=ikd), save :: &
         len_tbl, &  ! length of block to allocate for INP card table (inp_tbl)
         len_dta, &  ! length of block to allocat for array used to return card data (rdata)
        len_rtbl, &  ! length of block to allocate for INP card table (rst_tbl)
         siz_crd     ! size of block allocated for INP card table and data arrays
!
      real(kind=rkd), save, target, allocatable :: r_cards(:), rst_cards(:)
!
!  The following real pointer arrays are ranked as indicated, inp_tbl(inp_table:inp_table), and rcards(inp_read:inp_read)
!     Note: inp_table and inp_read are defined in subroutine set_fixed_dimensions in module problem_dimensions
!
!  Space is allocated and pointers associations are made in subroutine allok8_card_arrays
!
      real(kind=rkd), save, pointer :: &
            inp_tbl(:), &  !  array used for INP card tables (and original problem input for restart)
            rst_tbl(:), &  !  array used for INP card tables for restart input 
              rdata(:)     !  array used to return card data from INP routines (INP, etc.)
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_inp_card_arrays
!
!  Subroutine to allocate space and set pointers for INP free form input processing arrays
!
!     allok8_inp_card_arrays is called from subroutine input
! 
      use kind_specs
      use restart_block_list      
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!      
      if (allocated (r_cards)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, len_tbl, i2, inp_tbl, r_cards)
         call r_pntr (allocatd, len_dta, i2,   rdata, r_cards)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_crd   = i2
!
            allocate (r_cards(siz_crd), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_crd, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_cards
            call build_restart_block_list ('r_cards ', rdum, idum, ldum, siz_crd, 1, 1)
!
!     initialize real arrays to 0.0d0
!
         else
            r_cards      = 0.0d0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' INP free form processing arrays (allok8_inp_card_arrays).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_inp_card_arrays
!
!  ===============================================================================================================================
      function idata (i)
!
!  Funtion to return integer data read by INP free form input processing subroutines.  All data 
!  (real, integer and character) is returned in a real array.  icard is used to get the integer
!  data.  Character data will either be stored in real arrays of written into character data.
!
!     idata is called from routines using the INP free form input package
!
      implicit none
!
      real(kind=rkd)    card
      integer(kind=ikd) i, idata, icard(2,1)
      equivalence (card, icard(1,1))
!
      card   = rdata(i)
      idata  = icard(1,1)
!
      return
      end function idata
!
      end module inp_cards
