!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for junction related arrays.  Both 1D and 2D pointer arrays are
!  setup.  The 2D arrays have a fixed second dimension equal to 2.
!
!  Module junctions also contains the following subroutine(s)
!
!     allok8_junctions
!
!  ===============================================================================================================================
!
      module junctions
!
      use kind_specs
!
      integer(kind=ikd), save :: &
          num_jun, &  !  number of junctions
         siz_rjun, &  !  size of the array allocated for the real jun pointer arrays
         siz_ijun, &  !  size of the array allocated for the integer jun pointer arrays
         siz_ljun     !  size of the array allocated for the logical jun pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_jun(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_jun).  Space is 
!  allocated and pointers associations are made in subroutine allok8_junctions
!
      real(kind=rkd), pointer, save :: &
       admass(:), &  !  dynamic slip added mass coefficient              
         ajun(:), &  !  junction flow area                               
        ajunt(:), &  !  time dependent junction flow area                
        avedj(:), &  !  average junction density                         
         awgj(:), &  !  noncon mass flow rate                            
        awgjo(:), &  !  gas-to-wall interfacial area at junction         
         awlj(:), &  !  noncon mass flow rate                            
        awljo(:), &  !  liquid-to-wall interfacial area at junction      
        c0old(:), &  ! 
        chkvk(:), &  !  check valve friction factor                      
        conco(:), &  !  contraction coefficient                          
         dela(:), &  !  pressure differential - acceleration             
         dele(:), &  !  pressure differential - elevation                
         delf(:), &  !  pressure differential - friction                 
         delp(:), &  !  pressure differential - velocity head            
        delvs(:)     !  most recent change in slip velocity              
!
      real(kind=rkd), pointer, save :: &
       delvso(:), &  !  previous time step change in jun slip velocity   
        delwp(:), &  !  most recent change in jun mass flow rate         
       delwpo(:), &  !  previous time step change in jun mass flow rate  
        dhjdm(:), &  !  partial of junction enth wrt donor vol mass      
       dhjdmv(:), &  !  partial of jun enthalpy wrt from vol vapor mass  
        dhjdu(:), &  !  partial of junction enth wrt donor vol int ener  
       dhjdvs(:), &  !  partial of junction enth wrt jun slip velocity   
        dhjdw(:), &  !  partial of junction enth wrt jun mass flow rate  
        diamj(:), &  !  junction diameter                                
       dncdvs(:), &  !  partial noncon convection wrt slip velociy       
       dpmdwj(:), &  !  partial of 2-stream mix dp wrt junction flow     
       dscdvs(:)     !  partial of slip convectn (slpcnv) wrt slip vel   
!
      real(kind=rkd), pointer, save :: &
        dvsdt(:), &  !  dvs/dt = dynamic slip equation                   
       dvsdvs(:), &  !  partial of dyn slip equation wrt jun slip vel    
       dvsdwp(:), &  !  partial of dyn slip equation wrt jun mass flow   
        esubk(:), &  !  dw/dt momentum equation                          
       faninj(:), &  !  fanning friction at junction                     
        fjunf(:), &  !  specific energy loss coefficient for forward flow
        fjunr(:), &  !  specific energy loss coefficient for reverse flow
        fsubk(:), &  !  partial of momentum equation wrt jun flow        
        gsubk(:), &  !  partial of momentum equation wrt jun slip vel    
       hlprzr(:), &  !  liquid enthalpy at pressurizer                   
        holda(:), &  !  old junction enthalpy save array                 
           hp(:), &  !  junction fluid enthalpy                          
       hpflow(:), &  !  flowing enthalpy                                 
          hpg(:), &  !  vapor phase enthalpy                             
          hpl(:), &  !  liquid phase enthalpy                            
       inerta(:), &  !  junction inertia                                 
         pjun(:)     !  junction pressure                                
!
      real(kind=rkd), pointer, save :: &
        pumpp(:), &  !  pump head                                        
       rhostr(:), &  !  effective junction density                       
        slipr(:), &  !  slip ratio --- (gas vel / liq vel)               
       slpcno(:), &  !  previous time step slip energy convection        
       slpcnv(:), &  !  energy convection due to slip effects            
       slpmul(:), &  !  user specified slip multiplier (algebraic slip)  
         spvj(:), &  !  junction specific volume                         
        spvjg(:), &  !  vapor phase specific volume                      
        spvjl(:), &  !  liquid phase specific volume                     
        spvjo(:), &  !  previous time step specific volume               
        spvjv(:), &  !  junction vapor specific volume                   
        tempj(:), &  !  junction temperature                             
       thetad(:), &  !  drift flux angular orientation
        vgasj(:), &  !  gas velocity                                     
       vgasjo(:), &  !  previous time step gas velocity                  
         void(:), &  !  junction void fraction                           
       voidjo(:)     !  previous time step void fraction parameters required for slip options                             
!
      real(kind=rkd), pointer, save :: &
        vslpj(:), &  !  slip velocity -- (liq vel - vap vel)             
       vslpjo(:), &  !  previous time step slip velocity                 
          wgj(:), &  !  vapor mass flow rate                             
          wlj(:), &  !  liquid mass flow rate                            
         wold(:), &  !  previous time step flow                          
           wp(:), &  !  junction weight flow                             
       xjfccf(:), &  !  old time value of jfstar                         
           xp(:), &  !  junction quality                                 
         xpnc(:), &  !  junction noncon quality (noncon mass / mass )    
       xpnctt(:), &  !  noncon mass / mass at junc. (from volume xpnc)   
        xpold(:), &  !  previous time step quality                       
         zjun(:)     !  junction elevation                               
!
      integer(kind=ikd), target, allocatable, save :: i_jun(:)
      integer(kind=ikd), pointer, save :: &
        idfwd(:), &  !  id of cont. block defining forward loss coeff.   
        idrev(:), &  !  id of cont. block defining reverse loss coeff.   
          idw(:), &  !  donor cell index                                 
       idxmoc(:), &  !  index into method of characteristics file (id32) 
       ieqtyp(:), &  !  momentum equation option                         
       ifbrch(:), &  !  index to fibwr channel subfile for this junction 
         ifrj(:), &  !  flow regime index                                
       ifrjed(:), &  !  flow regime edit variable                        
       ihqcor(:), &  !  enthalpy transport index                         
           ij(:), &  !  mixing junction number                           
       inrtia(:), &  !  pointer to inertia if from control block         
       iprjun(:), &  !  index to pressurizer vapor region subfile when junction is connected to a pressurizer
        ipump(:), &  !  pump number -- (-) suction -- (+) discharge      
       ipumpn(:), &  !  index to pump description -- filid(15)           
          isp(:), &  !  spray index                                      
          itp(:), &  !  choking type indicator                           
       ivalve(:), &  !  valve number                                     
        jestw(:), &  !  steady state flow flag (1, input; -1, estimate)
       jcalci(:), &  !  initial condition calculation type control index 
       jchoke(:), &  !  junction choking index                           
        jflip(:), &  !  flag to flip "from/to" for mom flux calc if angljv(1 or 2) >= 90 and < 270 deg
        jtpmj(:), &  !  junction friction option                         
       jvertl(:), &  !  vertical junction control index                  
       mulslp(:), &  !  index to user specified slip multiplier          
        mvmix(:), &  !  momentum mixing index                            
       oljunn(:)     !  old junction number                              
!
      logical(kind=lkd), target, allocatable, save :: l_jun(:)
      logical(kind=lkd), pointer, save :: &
        optj1(:), &  !  momentum eqn. solve flag (solve if .false.)      
        optj2(:)     !  slip eqn. solve flag (solve if .false.)          
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_jun,1:2)
!  Space is allocated and pointers associations are made in subroutine allok8_junctions. For 
!  all of the following 2D arrays the second subscript is used to define the "from" side (=1)
!  and the "to" side (=2) for a junction variable, unless otherwise noted.
!
      integer(kind=ikd), save :: &
         siz_r2jun, &  !  size of the array allocated for the real jun pointer arrays
         siz_i2jun     !  size of the array allocated for the integer jun pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r2_jun(:,:)
      real(kind=rkd), pointer, save :: &
       alphv(:,:), &  !  void fraction of adjacent volume
       cosvj(:,:), &  !  cosine of the juncion angle relative to the adjacent volume angle
       dncdm(:,:), &  !  partial noncon convection wrt total mass of adjacent volume
       dncdu(:,:), &  !  partial of noncon convect. wrt energy of adjacent volume
       dncmg(:,:), &  !  partial noncon convection wrt vapor mass of adjacent volume
       dncnc(:,:), &  !  partial noncon convection wrt non-cn mass of adjacent volume
       dvsdp(:,:), &  !  partial of dyn slip equation wrt volume pressure of adjacent volume
       dvsdx(:,:), &  !  partial of dyn slip equation wrt volume mix quality of adjacent volume
       dwhdm(:,:), &  !  partial of energy equation convection wrt mass of adjacent volume
       dwhdu(:,:), &  !  partial of energy equation convection wrt internal energy of adjacent volume
       dwhmv(:,:), &  !  partial of energy eqn conv wrt vol vapor mass of adjacent volume
       dwhnc(:,:), &  !  partial energy convection wrt noncon mass of adjacent volume
       dwvdw(:,:), &  !  derivative of wvbar wrt the jun flow at the volume inlet (=1) or outlet (=2)
        dxdm(:,:), &  !  partial of quality wrt vol mass of adjacent volume
       dxdmn(:,:), &  !  partial of jun quality wrt noncon mass of adjacent volume
       dxdmv(:,:), &  !  partial of quality wrt vol vap mass of adjacent volume
        dxdu(:,:), &  !  partial of quality wrt vol int energy of adjacent volume
       dxndm(:,:), &  !  partial noncon mass frac wrt total mass of adjacent volume
       dxndu(:,:), &  !  partial noncon mass frac wrt total energy of adjacent volume
       dxnmn(:,:), &  !  partial noncon mass frac wrt noncon mass of adjacent volume
       dxnmv(:,:), &  !  partial noncon mass frac wrt vap mass of adjacent volume
       dxpdm(:,:), &  !  partial of vap eq conv wrt vol mass of adjacent volume
       dxpdu(:,:), &  !  partial of vap convection wrt energy of adjacent volume
       dxpmn(:,:), &  !  partial of vap convection wrt noncon mass of adjacent volume
       dxpmv(:,:), &  !  partial of vap eq conv wrt vol vap mass of adjacent volume
       headr(:,:), &  !  junction head term for adjacent volume
       pstar(:,:), &  !  velocity head of adjacent volume
       spvmf(:,:), &  !  specific volume  of adjacent volume used in the momentum flux calculation only
         xik(:,:), &  !  weighting factor momentum flux calculation      
         xil(:,:), &  !  weighting factor momentum flux calculation       
         xmf(:,:)     !  quality  of adjacent volume used in the momentum flux calculation only                   
!
      integer(kind=ikd), target, allocatable, save :: i2_jun(:,:)
      integer(kind=ikd), pointer, save :: &
          iw(:,:), &  !  juncion adjacent volume number
         iwn(:,:)     !  junction adjacent new volume number
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_junctions  
!
!  Subroutine to allocate space and set pointers for junction related arrays
!     Note: num_jun must be defined before allok8_junctions is called
!
!     allok8_jun is called from subroutine injun
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      use s_r2_pntr
      use s_i2_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, n2d, status
      logical(kind=lkd) allocatd
!
      parameter (n2d   = 2)      
!
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)  !  dummy real pointers
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)  !  dummy integer pointers
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)  !  dummy logical pointer
!
      if (num_jun .le. 0) then
         write (6,110) num_jun
         call fail
         return
      endif
!
      if (allocated (r_jun)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call r_pntr (allocatd, num_jun, i2, admass, r_jun)
         call r_pntr (allocatd, num_jun, i2,   ajun, r_jun)
         call r_pntr (allocatd, num_jun, i2,  ajunt, r_jun)
         call r_pntr (allocatd, num_jun, i2,  avedj, r_jun)
         call r_pntr (allocatd, num_jun, i2,   awgj, r_jun)
         call r_pntr (allocatd, num_jun, i2,  awgjo, r_jun)
         call r_pntr (allocatd, num_jun, i2,   awlj, r_jun)
         call r_pntr (allocatd, num_jun, i2,  awljo, r_jun)
         call r_pntr (allocatd, num_jun, i2,  c0old, r_jun)
         call r_pntr (allocatd, num_jun, i2,  chkvk, r_jun)
         call r_pntr (allocatd, num_jun, i2,  conco, r_jun)
         call r_pntr (allocatd, num_jun, i2,   dela, r_jun)
         call r_pntr (allocatd, num_jun, i2,   dele, r_jun)
         call r_pntr (allocatd, num_jun, i2,   delf, r_jun)
         call r_pntr (allocatd, num_jun, i2,   delp, r_jun)
         call r_pntr (allocatd, num_jun, i2,  delvs, r_jun)
         call r_pntr (allocatd, num_jun, i2, delvso, r_jun)
         call r_pntr (allocatd, num_jun, i2,  delwp, r_jun)
         call r_pntr (allocatd, num_jun, i2, delwpo, r_jun)
         call r_pntr (allocatd, num_jun, i2,  dhjdm, r_jun)
         call r_pntr (allocatd, num_jun, i2, dhjdmv, r_jun)
         call r_pntr (allocatd, num_jun, i2,  dhjdu, r_jun)
         call r_pntr (allocatd, num_jun, i2, dhjdvs, r_jun)
         call r_pntr (allocatd, num_jun, i2,  dhjdw, r_jun)
         call r_pntr (allocatd, num_jun, i2,  diamj, r_jun)
         call r_pntr (allocatd, num_jun, i2, dncdvs, r_jun)
         call r_pntr (allocatd, num_jun, i2, dpmdwj, r_jun)
         call r_pntr (allocatd, num_jun, i2, dscdvs, r_jun)
         call r_pntr (allocatd, num_jun, i2,  dvsdt, r_jun)
         call r_pntr (allocatd, num_jun, i2, dvsdvs, r_jun)
         call r_pntr (allocatd, num_jun, i2, dvsdwp, r_jun)
         call r_pntr (allocatd, num_jun, i2,  esubk, r_jun)
         call r_pntr (allocatd, num_jun, i2, faninj, r_jun)
         call r_pntr (allocatd, num_jun, i2,  fjunf, r_jun)
         call r_pntr (allocatd, num_jun, i2,  fjunr, r_jun)
         call r_pntr (allocatd, num_jun, i2,  fsubk, r_jun)
         call r_pntr (allocatd, num_jun, i2,  gsubk, r_jun)
         call r_pntr (allocatd, num_jun, i2, hlprzr, r_jun)
         call r_pntr (allocatd, num_jun, i2,  holda, r_jun)
         call r_pntr (allocatd, num_jun, i2,     hp, r_jun)
         call r_pntr (allocatd, num_jun, i2, hpflow, r_jun)
         call r_pntr (allocatd, num_jun, i2,    hpg, r_jun)
         call r_pntr (allocatd, num_jun, i2,    hpl, r_jun)
         call r_pntr (allocatd, num_jun, i2, inerta, r_jun)
         call r_pntr (allocatd, num_jun, i2,   pjun, r_jun)
         call r_pntr (allocatd, num_jun, i2,  pumpp, r_jun)
         call r_pntr (allocatd, num_jun, i2, rhostr, r_jun)
         call r_pntr (allocatd, num_jun, i2,  slipr, r_jun)
         call r_pntr (allocatd, num_jun, i2, slpcno, r_jun)
         call r_pntr (allocatd, num_jun, i2, slpcnv, r_jun)
         call r_pntr (allocatd, num_jun, i2, slpmul, r_jun)
         call r_pntr (allocatd, num_jun, i2,   spvj, r_jun)
         call r_pntr (allocatd, num_jun, i2,  spvjg, r_jun)
         call r_pntr (allocatd, num_jun, i2,  spvjl, r_jun)
         call r_pntr (allocatd, num_jun, i2,  spvjo, r_jun)
         call r_pntr (allocatd, num_jun, i2,  spvjv, r_jun)
         call r_pntr (allocatd, num_jun, i2,  tempj, r_jun)
         call r_pntr (allocatd, num_jun, i2, thetad, r_jun)
         call r_pntr (allocatd, num_jun, i2,  vgasj, r_jun)
         call r_pntr (allocatd, num_jun, i2, vgasjo, r_jun)
         call r_pntr (allocatd, num_jun, i2,   void, r_jun)
         call r_pntr (allocatd, num_jun, i2, voidjo, r_jun)
         call r_pntr (allocatd, num_jun, i2,  vslpj, r_jun)
         call r_pntr (allocatd, num_jun, i2, vslpjo, r_jun)
         call r_pntr (allocatd, num_jun, i2,    wgj, r_jun)
         call r_pntr (allocatd, num_jun, i2,    wlj, r_jun)
         call r_pntr (allocatd, num_jun, i2,   wold, r_jun)
         call r_pntr (allocatd, num_jun, i2,     wp, r_jun)
         call r_pntr (allocatd, num_jun, i2, xjfccf, r_jun)
         call r_pntr (allocatd, num_jun, i2,     xp, r_jun)
         call r_pntr (allocatd, num_jun, i2,   xpnc, r_jun)
         call r_pntr (allocatd, num_jun, i2, xpnctt, r_jun)
         call r_pntr (allocatd, num_jun, i2,  xpold, r_jun)
         call r_pntr (allocatd, num_jun, i2,   zjun, r_jun)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_rjun   = i2
!
            allocate (r_jun(siz_rjun), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rjun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_jun
            block  = 'r_jun'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rjun, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_jun      = 0.0d0
         endif
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         i2       = 0
!
         call r2_pntr (allocatd, num_jun, n2d, i2, alphv, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, cosvj, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dncdm, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dncdu, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dncmg, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dncnc, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dvsdp, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dvsdx, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dwhdm, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dwhdu, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dwhmv, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dwhnc, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dwvdw, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2,  dxdm, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxdmn, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxdmv, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2,  dxdu, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxndm, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxndu, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxnmn, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxnmv, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxpdm, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxpdu, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxpmn, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, dxpmv, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, headr, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, pstar, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2, spvmf, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2,   xik, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2,   xil, r2_jun)
         call r2_pntr (allocatd, num_jun, n2d, i2,   xmf, r2_jun)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_r2jun   = i2
!
            allocate (r2_jun(siz_r2jun,1:n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_r2jun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            r2dum  => r2_jun
            block  = 'r2_jun'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_r2jun, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r2_jun      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call i_pntr (allocatd, num_jun, i2,  idfwd, i_jun)
         call i_pntr (allocatd, num_jun, i2,  idrev, i_jun)
         call i_pntr (allocatd, num_jun, i2,    idw, i_jun)
         call i_pntr (allocatd, num_jun, i2, idxmoc, i_jun)
         call i_pntr (allocatd, num_jun, i2, ieqtyp, i_jun)
         call i_pntr (allocatd, num_jun, i2, ifbrch, i_jun)
         call i_pntr (allocatd, num_jun, i2,   ifrj, i_jun)
         call i_pntr (allocatd, num_jun, i2, ifrjed, i_jun)
         call i_pntr (allocatd, num_jun, i2, ihqcor, i_jun)
         call i_pntr (allocatd, num_jun, i2,     ij, i_jun)
         call i_pntr (allocatd, num_jun, i2, inrtia, i_jun)
         call i_pntr (allocatd, num_jun, i2, iprjun, i_jun)
         call i_pntr (allocatd, num_jun, i2,  ipump, i_jun)
         call i_pntr (allocatd, num_jun, i2, ipumpn, i_jun)
         call i_pntr (allocatd, num_jun, i2,    isp, i_jun)
         call i_pntr (allocatd, num_jun, i2,    itp, i_jun)
         call i_pntr (allocatd, num_jun, i2, ivalve, i_jun)
         call i_pntr (allocatd, num_jun, i2,  jestw, i_jun)         
         call i_pntr (allocatd, num_jun, i2, jcalci, i_jun)
         call i_pntr (allocatd, num_jun, i2, jchoke, i_jun)
         call i_pntr (allocatd, num_jun, i2,  jflip, i_jun)
         call i_pntr (allocatd, num_jun, i2,  jtpmj, i_jun)
         call i_pntr (allocatd, num_jun, i2, jvertl, i_jun)
         call i_pntr (allocatd, num_jun, i2, mulslp, i_jun)
         call i_pntr (allocatd, num_jun, i2,  mvmix, i_jun)
         call i_pntr (allocatd, num_jun, i2, oljunn, i_jun)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ijun   = i2
!
            allocate (i_jun(siz_ijun), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ijun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_jun
            block  = 'i_jun'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ijun, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_jun      = 0
         end if
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         i2       = 0
!
         call i2_pntr (allocatd, num_jun, n2d, i2,    iw, i2_jun)
         call i2_pntr (allocatd, num_jun, n2d, i2,   iwn, i2_jun)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_i2jun   = i2
!
            allocate (i2_jun(siz_i2jun,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_i2jun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            i2dum  => i2_jun
            block  = 'i2_jun'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_i2jun, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i2_jun      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_jun, i2,  optj1, l_jun)
         call l_pntr (allocatd, num_jun, i2,  optj2, l_jun)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ljun   = i2
!
            allocate (l_jun(siz_ljun), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_ljun, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_jun
            block  = 'l_jun'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ljun, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_jun      = .false.
         endif
      enddo
!
  100 format (' ******** Error allocating space for ',a8,' junction arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for junction arrays. '/ &
          10x,'Variable num_jun = ',i3, ', must be > 0.')
      end subroutine allok8_junctions
!
      end module junctions
