!$Header$
!$Copyright$
!  ===============================================================================================================================
!  Specifications and space allocations for minor edit summary 
!  Specifications and procedures used to create minor edit search list and procedures for locating minor edit requests and 
!  returning values
!
!  This module also includes the following subroutines
!
!     allok8_me_sum
!     me_sum_stor 
!     me_sum_edit
!
!  ===============================================================================================================================
!
      module minor_edit_summary
!      
      use kind_specs
      use time_data
!
      implicit none      
!
      real(rkd), save :: memax, memin
      real(rkd), allocatable,  dimension(:),  save :: tmax,  tmin,  vfinal,  vinitl,  vmax,  vmin  
!
      data  memax / 1.0D20 / 
!      
      contains
!      
!  ===============================================================================================================================
!
     subroutine allok8_me_sum (ne)
!
!  Subroutine to allocate memory to store summary information for each minor edit variable.
!
!     ne     = number of minor edit varialbes for this problem           (input)
!
!  allok8_me_sum is called from subroutine edit
!      
      use kind_specs
!
! Dummy arguments
!
      integer(ikd) :: ne  !  number of minor edit variables
      intent (in)     ne
!
! Local variables
!
      integer(ikd) ::   i_err
!
!                                    minor edit summary values as indicated below
!                                    ============================================
      allocate ( vinitl(1:ne), &  !  initial value (t=0)
                 vfinal(1:ne), &  !  final value
                   vmax(1:ne), &  !  maximum value
                   tmax(1:ne), &  !  time maximum value occurs
                   vmin(1:ne), &  !  minimum value            
                   tmin(1:ne), &  !  time minimum value occurs
                   stat = i_err)        
!
      if ( i_err/=0 ) then
         write (6,100) ne, i_err
         call fail
      end if
!
      vmax   = -memax
      tmax   =  0.0d0
      vmin   =  memax
      tmin   =  0.0d0
      vinitl =  memax
      vfinal =  0.0d0
!
  100 format (' ******** Error allocating space for minor edit summary arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_me_sum
!
!  ===============================================================================================================================
!
      subroutine me_sum_stor 
!
!  Subroutine me_sum_stor stores the minor edit variables in an array used 
!  to display the initial, final and minimum and maximum values and associated
!  times for each minor edit variable
!
!  me_sum_stor is called by subroutine edit
!
      use kind_specs
      use minor_edits
      use minor_edit_search
      use system
      use units
!
!  Local variables
!
      integer(kind=ikd) ::  i, idx, ioff, meunits
      real(kind=rkd)    ::  vtemp 
!
      do i = 1, num_var
         idx    = var_list(i) % me_index 
         ioff   = var_list(i) % reg_offset
         vtemp  = me_value (idx, ioff)
!                                       
         if ( siunit ) then
            meunits = me_list(idx) % me_units
            if ( meunits==1 ) then
               vtemp  = cftok (vtemp)    !  temperature conversion to Kelvin
            end if
         end if
!         
!     initial value
!
         if ( vinitl(i)==memax ) vinitl(i) = vtemp
!
!     final value
!
         vfinal(i) = vtemp
!
!     maximum value
!
         if ( vtemp>vmax(i) ) then
            vmax(i) = vtemp
            tmax(i) = timex(1)
         end if
!
!     minimum value
!
         if ( vtemp<vmin(i) ) then
            vmin(i) = vtemp
            tmin(i) = timex(1)
         end if
      end do
!
      end subroutine me_sum_stor
!
!  ===============================================================================================================================
!
      subroutine me_sum_edit
!
!  Subroutine me_sum_stor edits the minor edit variable  with initial, final, max value, time at max, min value, and
!  time at min value at the end of run
!
!  me_sum_edit is called by subroutine retran and finish
!
      use kind_specs
      use minor_edits
      use units
!
      implicit none
!
!  Local variables
!
      integer(kind=ikd) ::  i, idxflg, dummy
      character(8) :: dummy_char
!
      if ( num_var>0 ) then
!      
         call subpg (dummy)
!         
         write (6,9010)
         write (6,9020)
         write (6,9030)
         write (6,9040)
         write (6,9020)
!
         idxflg = var_list(1)%me_index   !  minor edit search list index for timex  -  don't write it out
!
         do i = 1, num_var
!         
            if ( idxflg/=var_list(i)%me_index )  then
!            
!        convert blk_title and region into one string
!
               if (var_list(i)%me_region == 0) then
                  write(dummy_char,9070) var_list(i)%blk_title
               else
                  write(dummy_char,9080) var_list(i)%blk_title,var_list(i)%me_region
               end if
!               
               write (6,9050) dummy_char, var_list(i)%var_desc, var_list(i)%units,  &
                              vinitl(i), vfinal(i), vmax(i), tmax(i), vmin(i), tmin(i)
            end if
         end do
!         
         write (6,9020)
         write (6,9060)
      end if
!
 9010 format (52x,'**********************',/52x,'* MINOR EDIT SUMMARY *'/52x,'**********************')
 9020 format (1x,125('='))
 9030 format ('  MINOR                                 INITIAL        ',   &
              'FINAL          ------ MAXIMUM ------         ------ MINIMUM ------')
 9040 format ('  EDIT      DESCRIPTION    UNITS        VALUE          ',  &
              'VALUE          VALUE         TIME (SEC)      VALUE          TIME (SEC)')
 9050 format (2x,a8,2x,a12,2x,a8,6(2x,1p,e13.5))
 9060 format (//,55x,'* END SUMMARY *'/1x,  &
     &'****************************************************************************************************************************&
     &******* ')
 9070 format (A8)     
 9080 format (A4,i4) 
!
      end subroutine me_sum_edit
!      
      end module minor_edit_summary                                     
