!$Header$
!$Copyright$
!
!  ==============================================================================================================================
!  Derived type specification used to define variable list for requested minor edits
!
!  This module also includes the following subroutine
!
!     allok8_minor_edits
!
!  ==============================================================================================================================
!
      module minor_edits
!
      use kind_specs
!
      implicit none      
!
!  Derived type specification used to save minor edit information
!
      type me_save_list
         integer(kind=ikd)             :: me_index,   &  ! index in minor edit list for given minor edit request
                                          reg_offset, &  ! region offset
                                          me_region      ! minor edit reion number (entered with minor edit flag as pair)
         character(kind=ckd,len=4)     :: me_flag        ! minor edit request flag 
         character(kind=ckd,len=8)     :: me_flg_reg, &  ! minor edit flag and region number (combined string) 
                                          blk_title,  &  ! block title string -- e.g. " VOL 999", " SYSTEM ", etc
                                          units          ! units string for minor edit variable, e.g. " btu/lbm"
         character(kind=ckd,len=12)    :: var_desc       ! variable description string
         real(kind=rkd), dimension(50) :: var_value      ! value of minor edit variable for 50 time points
      end type me_save_list
!
!  Minor edit variable save list
!
      type (me_save_list), target, allocatable, save :: var_list(:)
      integer(kind=ikd), save :: num_var, &  !  the number of minor edit variables in me_save_list
                                 val_num     !  last index of var_value saved
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_minor_edits
!
!  Subroutine to allocate space to save minor edit requests and up to 50 values for each minor edit
!     Note: num_var must be defined before allok8_minor_edits is called
!
!     allok8_minor_edits is called from subroutine inedit
!
      use kind_specs
! 
      implicit none
!      
      integer(kind=ikd) l, len, i2, status, allocatd
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space 
!  -------------------------------------------------------------------------------------------------------------------------------
!
!
      if (allocated (var_list)) then
         return
      else
         allocate (var_list(num_var), stat=status)
!
!     Check status of allocate - write error if allocate failed
!
         if (status .ne. 0) then
            write (6,100) num_var, status
            call fail
         endif
      endif
!
      val_num = -1  ! Will be re-initialized to 0 in subroutine edit during first call to subroutine edit
!
  100 format (' ******** Error allocating space for minor edit request list (allok8_minor_edits).'/ &
              '          Length requested                   = ',i8/ &
              '          Status flag returned from allocate = ',i3)
      end subroutine allok8_minor_edits
!
      end module minor_edits
