!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for point kinetics related arrays
!
!  This module also includes the following subroutine
!
!     allok8_point_kinetics
!     allok8_ptkin_coeff
!     allok8_ptkin_fdbk
!     allok8_ptkin_fdbk_tables
!
!  ===============================================================================================================================
!
      module point_kinetics
!
      use kind_specs
!
      integer(kind=ikd), save :: &
          num_ptkin, &  !  number of point kinetics models (always = 1)
         siz_rptkin, &  !  size of the array allocated for the real ptkin pointer arrays
         siz_iptkin     !  size of the array allocated for the integer ptkin pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_ptkin(:)
!
!  All of the following real and integer pointer arrays are Rank (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_point_kinetics
!
      real(kind=rkd), pointer, save :: &
       ajovra(:), &  !  delayed neutron yield fractions (6)             
         bovl(:), &  !  beta/prompt lifetime                            
       dlamda(:), &  !  delayed neutron decay constants (6)             
         dpow(:), &  !  decay heat power                                
           dt(:), &  !  new time steps                                   
       period(:), &  !  reactor period                                  
         ppow(:), &  !  prompt power                                    
        preac(:), &  !  reactivity                                      
       preaco(:), &  !  previous time step reactivity.                  
       prompt(:), &  !  fraction of fission power released as prompt    
        rhoin(:), &  !  initial reactivity                              
        slope(:), &  !  rate of reactivity change                       
           ss(:), &  !  not used                                        
          sum(:), &  !  accumulator for summing delayed effects         
            t(:), &  !  kinetics sub-time step sze                      
       unp239(:), &  !  not used                                        
            w(:), &  !  average inverse period                          
           w1(:), &  !  inverse period                                  
        w1old(:), &  !  previous (two steps back) time step             
           w3(:)     !  instantaneous inverse period                    
!
      real(kind=rkd), pointer, save :: &
        w3old(:), &  !  previous time step instantaneous period.        
        woldo(:)     !  previous time step inverse period               
!        
      integer(kind=ikd), target, allocatable, save :: i_ptkin(:)
!
      integer(kind=ikd), pointer, save :: &
           kk(:), &  !  runge kutta integration step                    
        kodel(:)     !  internal kinetics calc flag                     
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for point kinetics core section related quantities (reactivity coefficients)
!  -------------------------------------------------------------------------------------------------------------------------------
!
      integer(kind=ikd), save :: &
           num_rcof, &  !  number of core sections
          siz_rrcof, &  !  size of the array allocated for the real rcof pointer arrays
          siz_ircof     !  size of the array allocated for the integer rcof pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_rcof(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_rcof)
!  Space is allocated and pointers associations are made in subroutine allok8_point_kinetics
!
      real(kind=rkd), pointer, save :: &
       alphtm(:), &  !  fuel temperature coefficient                     
       alphtw(:), &  !  water temperature coefficient                    
        dopwt(:), &  !  fuel temperature weighting factor                
           v1(:), &  !  initial  coolant specific volume                 
       voidwt(:), &  !  density weighting factor                         
           vw(:)     !  subcooled vapor volume fraction                             
!
      integer(kind=ikd), target, allocatable, save :: i_rcof(:)
      integer(kind=ikd), pointer, save :: &
         idop(:), &  !  current position in dopro table                  
        ivoid(:)     !  current position in voidro table                 
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for point kinetics feedback 
!  -------------------------------------------------------------------------------------------------------------------------------
!
      integer(kind=ikd), save :: &
          num_fdbk, &  !  number of feedback sets (always = 1)
         siz_rfdbk, &  !  size of the array allocated for the real fdbk pointer arrays
         siz_ifdbk     !  size of the array allocated for the integer fdbk pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_fdbk(:)
!
!
!  All of the following real and integer pointer arrays are Rank (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_ptkin_fdbk
!  
      real(kind=rkd), pointer, save :: &
           rd(:), &  !  doppler reactivity                               
        rdcal(:), &  !  calculated doppler reactivity at start           
           rf(:), &  !  fuel reactivity                                  
        rfcal(:), &  !  calculated fuel reactivity at start              
       rhocal(:), &  !  input reactivity                                 
           rv(:), &  !  void reactivity                                  
        rvcal(:), &  !  calculated void reactivity at start              
           rw(:), &  !  coolant reactivity                               
        rwcal(:)     !  calculated coolant reactivity at start           
!
      integer(kind=ikd), target, allocatable, save :: i_fdbk(:)
      integer(kind=ikd), pointer, save :: &
       dopunt(:), &  !  doppler units flag                               
       nmodht(:), &  !  number of mod heating wt factor point
       ntbltf(:), &  !  number of regional density fuel temp tables      
        ntblv(:)     !  number of regional density reactivity tables     
!        
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for point kinetics feedback tables
!  -------------------------------------------------------------------------------------------------------------------------------
!
      integer(kind=ikd), save :: &
         n_ifdbk_tbls, &  !  number of integer feedback table pointer arrays assigned
         n_rfdbk_tbls, &  !  number of real feedback table arrays assigned
       siz_rfdbk_tbls, &  !  size of the array allocated for the real feedback table pointer arrays         
       siz_ifdbk_tbls, &  !  size of the array allocated for the integer feedback table pointer arrays
            sum_denro, &  !  total number of points in all density reactivity tables
            sum_dopro     !  total number of points in all doppler reactivity tables       
!
      integer(kind=ikd), target, allocatable, save :: i_fdbk_tbls(:)
!
!  The integer arrays nvoid and idxdnt are Rank (1:ntblv) and integer arrays ndop and idxdpt are Rank (1:ntlbtf)
!  Space is allocated and pointers associations are made in subroutine allok8_ptkin_tbls
!
      integer(kind=ikd), pointer, save :: &
       idxdnt(:), &  !  index of file containing voidro table            
       idxdpt(:), &  !  index of file containing dopro table             
         ndop(:), &  !  number of doppler reactivity points  
        nvoid(:)     !  number of density reactivity points  
!        
      real(kind=rkd), target, allocatable, save :: r_fdbk_tbls(:)
!
!  The following real pointer arrays are Rank (1:numpts) where numpts is the 2*(sum of number of points 
!       for each table of a given table type)
!  Space is allocated and pointers associations are made in subroutine allok8_ptkin_tbls
!
      real(kind=rkd), pointer, save :: &
       denrho(:), &  !  moderator density reactivity table
       doprho(:), &  !  fuel temperature reactivity table
       weight(:)     !  direct moderator heating density weighting table       
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_point_kinetics
!
!  Subroutine to allocate space and set pointers for general point kinetics related arrays and 
!  core section related kinetics quantities.
!
!     allok8_point_kinetics is called from subroutine inrken
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_ptkin)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_ptkin = 1
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_ptkin, i2, ajovra, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,   bovl, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2, dlamda, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,   dpow, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,     dt, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2, period, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,   ppow, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,  preac, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2, preaco, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2, prompt, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,  rhoin, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,  slope, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,     ss, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,    sum, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,      t, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2, unp239, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,      w, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,     w1, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,  w1old, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,     w3, r_ptkin)
!         
         call r_pntr (allocatd, num_ptkin, i2,  w3old, r_ptkin)
         call r_pntr (allocatd, num_ptkin, i2,  woldo, r_ptkin)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rptkin   = i2
!
            allocate (r_ptkin(siz_rptkin), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rptkin, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_ptkin
            block  = 'r_ptkin'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rptkin, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_ptkin      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_ptkin, i2,     kk, i_ptkin)
         call i_pntr (allocatd, num_ptkin, i2,  kodel, i_ptkin)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_iptkin   = i2
!
            allocate (i_ptkin(siz_iptkin), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iptkin, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_ptkin
            block  = 'i_ptkin'
            call build_restart_block_list (block, rdum, idum, ldum, siz_iptkin, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_ptkin      = 0
         endif
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' point kinetics arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!          
      end subroutine allok8_point_kinetics
!
!  ===============================================================================================================================     
!
      subroutine allok8_ptkin_coeff
!
!  Subroutine to allocate space and set pointers for point kinetics reactivity coefficient related arrays
!     allok8_point_kinetics is called from subroutine inreac
!  Note: num_rcof must be defined as the number of core sections prior to calling allok8_ptkin_coeff
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_rcof)) return
!
      if (num_rcof .le. 0) then
         write (6,110) 
         call fail 
         return
      end if
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_rcof, i2,  dopwt, r_rcof)
         call r_pntr (allocatd, num_rcof, i2, alphtm, r_rcof)
         call r_pntr (allocatd, num_rcof, i2, alphtw, r_rcof)
         call r_pntr (allocatd, num_rcof, i2,     v1, r_rcof)         
         call r_pntr (allocatd, num_rcof, i2, voidwt, r_rcof)
         call r_pntr (allocatd, num_rcof, i2,     vw, r_rcof)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rrcof   = i2
!
            allocate (r_rcof(siz_rrcof), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rrcof, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_rcof
            block  = 'r_rcof'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rrcof, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_rcof      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_rcof, i2,   idop, i_rcof)
         call i_pntr (allocatd, num_rcof, i2,  ivoid, i_rcof)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ircof   = i2
!
            allocate (i_rcof(siz_ircof), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ircof, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_rcof
            block  = 'i_rcof'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ircof, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_rcof      = 0
         endif
      enddo
!      
  100 format (' ******** Error allocating space for ',a8,' point kinetics reactivity coefficient arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for point kinetics reactivity coefficient arrays. ',&
              ' The number of core sections (num_rcof)'/ &
          10x,'was not initialized prior to calling subroutine allok8_ptkin_coeff.')
!
      end subroutine allok8_ptkin_coeff
!      
!  ===============================================================================================================================     
!
      subroutine allok8_ptkin_fdbk  
!
!  Subroutine to allocate space and set pointers for fdbk related arrays
!     allok8_ptkin_fdbk is called from subroutine inreac
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_fdbk)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_fdbk = 1
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_fdbk, i2,     rd, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,  rdcal, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,     rf, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,  rfcal, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2, rhocal, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,     rv, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,  rvcal, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,     rw, r_fdbk)
         call r_pntr (allocatd, num_fdbk, i2,  rwcal, r_fdbk)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rfdbk   = i2
!
            allocate (r_fdbk(siz_rfdbk), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rfdbk, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_fdbk
            block  = 'r_fdbk'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rfdbk, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_fdbk      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_fdbk, i2, dopunt, i_fdbk)
         call i_pntr (allocatd, num_fdbk, i2, nmodht, i_fdbk)
         call i_pntr (allocatd, num_fdbk, i2, ntbltf, i_fdbk)
         call i_pntr (allocatd, num_fdbk, i2,  ntblv, i_fdbk)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ifdbk   = i2
!
            allocate (i_fdbk(siz_ifdbk), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ifdbk, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_fdbk
            block  = 'i_fdbk'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ifdbk, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_fdbk      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' point kinetics feedback arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_ptkin_fdbk
!
!  ===============================================================================================================================
      subroutine allok8_ptkin_fdbk_tables
!
!  Subroutine to allocate space point kinetics feedback table arrays
!     Note: Subroutine allok8_ptkin_fdbk must be called before allok8_ptkin_fdbk_tables is called.
!           Subroutine allok8_ptkin_tbls is called twice.  The first call allocates the integer 
!           pointer index and count arrays used to locate the table space.  Prior to the second
!           call to allok8_ptkin_fdbk_tbls nmodht must be defined in the feedback array and nvoid 
!           and ndop must be defined for each region.
!
!  allok8_ptkin_fdbk_tables is called from subroutine inreac 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, l, i1, i2, status, point_sum1, point_sum2
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_fdbk_tbls)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all feedback regions to detrmine space required for all point kinetics feedback table arrays.  Note: All point
!  kinetics feedback table arrays and decay heat density weighting tables reside in a single array r_fdbk_tbls.  The beginning 
!  index for the denrho table for region 1 is idxdpt(1) = 1.  For the region 2 table idxdpt(2) = (idxdpt(1) + 2*nvoid(1)). 
!  This pattern continues for all other regions. The indices are similarly computed for the doprho tables.  The decay heat 
!  density weighting table index is 1.
!
      if (.not.allocated (i_fdbk)) then
         write (6,110) 
         call fail
         return
      endif
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd,  ntblv(1), i2, idxdnt, i_fdbk_tbls)
         call i_pntr (allocatd, ntbltf(1), i2, idxdpt, i_fdbk_tbls)
         call i_pntr (allocatd, ntbltf(1), i2,   ndop, i_fdbk_tbls)         
         call i_pntr (allocatd,  ntblv(1), i2,  nvoid, i_fdbk_tbls)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ifdbk_tbls = i2
            n_ifdbk_tbls   = siz_ifdbk/num_fdbk
!
            allocate (i_fdbk_tbls(siz_ifdbk_tbls), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ifdbk_tbls, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_fdbk_tbls
            block  = 'i_fdbk_tbls'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ifdbk_tbls, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_fdbk_tbls = 0
         endif
      enddo
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, 2*sum_denro, i2, denrho, r_fdbk_tbls)
         call r_pntr (allocatd, 2*sum_dopro, i2, doprho, r_fdbk_tbls)
         call r_pntr (allocatd,  2*nmodht(1), i2, weight, r_fdbk_tbls)         
!
!      allocate space
!
         if (.not.allocatd) then
!
            siz_rfdbk_tbls = i2
!
            allocate (r_fdbk_tbls(siz_rfdbk_tbls), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rfdbk_tbls, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_fdbk_tbls
            block  = 'r_fdbk_tbls'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rfdbk_tbls, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_fdbk_tbls      = 0.0d0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' point kinetics feedback table arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for point kinetics feedback table arrays.'/ &
          10x,'Subroutine allok8_ptkin_fdbk_tables called before subroutine allok8_ptkin_fdbk - programming error') 
  120 format (' ******** Error allocating space for point kinetics feedback table arrays.'/ &
          10x,'Sum of number of points for all point kinetics feedback tables = ',i6,', must be > 0 - programming error') 
!
      end subroutine allok8_ptkin_fdbk_tables
!
      end module point_kinetics
