!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for problem dimension and option related arrays
!
!  This module also includes the following subroutine
!
!     allok8_problem_dimensions
!
!  ===============================================================================================================================
!
      module problem_dimensions
!
      use kind_specs
!
      integer(kind=ikd), save :: &
           num_dim, &  !  number of dim (always = 1)
          siz_idim     !  size of the array allocated for the integer dim pointer arrays
!
!  All of the following integer pointer arrays are Rank (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_problem_dimensions
!
      integer(kind=ikd), target, allocatable, save :: i_dim(:)
      integer(kind=ikd), pointer, save :: &
       ibstak(:), &  !  number of bubble rise stacks                     
          icf(:), &  !  control system option flag                       
        idnbc(:), &  !  auxiliary dnb option flag                        
        igntr(:), &  !  general transport option flag                    
       ihtmap(:), &  !  heat transfer map option flag                    
        iprzr(:), &  !  non-equilibrium pressurizer option flag          
       isflag(:), &  !  slip option flag                                 
       istgen(:), &  !  flag to request coupled steam gen initialization 
       ithslv(:), &  !  iterative th matrix soln optn flag (3d core)     
        itrns(:), &  !  transport delay option flag                      
       itvoid(:), &  !  neutron void reactivity option flag              
        jflow(:), &  !  junction flow component option flag              
         jsst(:), &  !  steady state inititalization option flag         
         ldmp(:), &  !  restart, plot tape control flag                  
         mtdv(:), &  !  number of time dependent volume boundary conditions to be retrieved from a data tape
       naccum(:), &  !  number of accumulator volumes                    
         nbub(:), &  !  number of bubble parameter sets supplied         
       ncflow(:), &  !  flag for noncondensable flow                     
       nchki2(:), &  ! 
         ncht(:)     !  number of specified heat transfer coefficient description cards
!
      integer(kind=ikd), pointer, save :: &
         nckv(:), &  !  number of valves described                       
         ncor(:), &  !  number of core sections                          
        ndgrp(:), &  !  number of delayed neutron groups                 
         nedi(:), &  !  number of minor edit parameters                  
       neweqs(:), &  !  equation set selection                           
         nfit(:), &  !  profile fit coefficient index                    
         nfll(:), &  !  number of fill sets                              
         ngap(:), &  !  number of dynamic gap models                     
         ngom(:), &  !  number of heat conductor geometries              
         nhtx(:), &  !  number of heat exchanger sets                    
         njun(:), &  !  number of junctions                              
          nlk(:), &  !  number of generalized data tables                
         nmat(:), &  !  number of heat conductor materials               
        nodel(:), &  !  power calculation type                           
        nofcs(:), &  !  number of neutronic control states               
        nofdg(:), &  !  number of delayed neutron families               
         nofg(:), &  !  number of neutron energy groups                  
         nofp(:), &  !  number of neutronic mesh points                  
         nofr(:), &  !  number of neutronic regions                      
        npmpc(:)     !  number of pumps                                  
!
      integer(kind=ikd), pointer, save :: &
        nsepr(:), &  !  separator efficiency model option flag           
         nslb(:), &  !  number of heat conductors                        
         nstk(:), &  !  local condition heat transfer option flag        
          ntc(:), &  !  number of time step sets supplied                
         ntdv(:), &  !  number of time dependent volumes                 
         ntrp(:), &  !  number of trip control cards supplied            
         ntsh(:), &  !  number of turbine shafts                         
       numrcs(:), &  !  numerical solution option                        
         nvol(:)     !  number of volumes                                
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_problem_dimensions
!
!  Subroutine to allocate space and set pointers for dim related arrays
!     Note: num_dim is always defined as 1
!
!     allok8_problem_dimensions is called from subroutine ret_input
!
      use kind_specs 
      use restart_block_list
!
!  Interface modules
!
      use s_i_pntr      
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_dim)) return
!
      num_dim = 1
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_dim, i2, ibstak, i_dim)
         call i_pntr (allocatd, num_dim, i2,    icf, i_dim)
         call i_pntr (allocatd, num_dim, i2,  idnbc, i_dim)
         call i_pntr (allocatd, num_dim, i2,  igntr, i_dim)
         call i_pntr (allocatd, num_dim, i2, ihtmap, i_dim)
         call i_pntr (allocatd, num_dim, i2,  iprzr, i_dim)
         call i_pntr (allocatd, num_dim, i2, isflag, i_dim)
         call i_pntr (allocatd, num_dim, i2, istgen, i_dim)
         call i_pntr (allocatd, num_dim, i2, ithslv, i_dim)
         call i_pntr (allocatd, num_dim, i2,  itrns, i_dim)
         call i_pntr (allocatd, num_dim, i2, itvoid, i_dim)
         call i_pntr (allocatd, num_dim, i2,  jflow, i_dim)
         call i_pntr (allocatd, num_dim, i2,   jsst, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ldmp, i_dim)
         call i_pntr (allocatd, num_dim, i2,   mtdv, i_dim)
         call i_pntr (allocatd, num_dim, i2, naccum, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nbub, i_dim)
         call i_pntr (allocatd, num_dim, i2, ncflow, i_dim)
         call i_pntr (allocatd, num_dim, i2, nchki2, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ncht, i_dim)
!         
         call i_pntr (allocatd, num_dim, i2,   nckv, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ncor, i_dim)
         call i_pntr (allocatd, num_dim, i2,  ndgrp, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nedi, i_dim)
         call i_pntr (allocatd, num_dim, i2, neweqs, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nfit, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nfll, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ngap, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ngom, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nhtx, i_dim)
         call i_pntr (allocatd, num_dim, i2,   njun, i_dim)
         call i_pntr (allocatd, num_dim, i2,    nlk, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nmat, i_dim)
         call i_pntr (allocatd, num_dim, i2,  nodel, i_dim)
         call i_pntr (allocatd, num_dim, i2,  nofcs, i_dim)
         call i_pntr (allocatd, num_dim, i2,  nofdg, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nofg, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nofp, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nofr, i_dim)
         call i_pntr (allocatd, num_dim, i2,  npmpc, i_dim)
!         
         call i_pntr (allocatd, num_dim, i2,  nsepr, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nslb, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nstk, i_dim)
         call i_pntr (allocatd, num_dim, i2,    ntc, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ntdv, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ntrp, i_dim)
         call i_pntr (allocatd, num_dim, i2,   ntsh, i_dim)
         call i_pntr (allocatd, num_dim, i2, numrcs, i_dim)
         call i_pntr (allocatd, num_dim, i2,   nvol, i_dim)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_idim   = i2
!
            allocate (i_dim(siz_idim), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_idim, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_dim
            block  = 'i_dim'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_idim, 2, 1)
!
!     initialize integer arrays to 0
!
         else
            i_dim      = 0
         endif
      enddo
!
  100 format (' ******** Error allocating space for ',a8,' problem dimension arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_problem_dimensions
!
      end module problem_dimensions
