!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications for RETRAN configuration constants.  Typically these variables would be hardwired in the code.  
!  They are included in the RETRAN configuration file so they can be changed without recompiling the code.
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     set_configuration
!
!  ===============================================================================================================================
!
      module retran_configuration
!
      use kind_specs
!
      integer(kind=ikd), save :: &
              inp_data, &  !  length of the vector used by the inp routines (e.g. inp2) to return data as it is read
                           !  (used in subroutine ret_input)
              len_rblk, &  !  length of vector used by restart routines, used during restart run           
            rstrt_blks     !  number of blocks saved in restart block list (used in subroutine ret_input)
!
      character(kind=ckd,len=32), save :: &
           organization    !  name of licensed organization (used in subroutine ret_input)
!
      contains
!
!  ===============================================================================================================================
      subroutine set_configuration
!
!  Subroutine to set RETRAN-3D configuration constants.  Values defined in this subroutine are the default
!  values.  They can be changed by supplying a RETRAN configuration file (retran.cfg) with new definitions for
!  the variables.  
!
!  set_configuration is called by main program RMAIN
!
      use kind_specs
!
      logical(kind=lkd) exists
      integer(kind=ikd) i, j, isav, jsav
      integer(kind=ikd), parameter :: nc=132
      integer(kind=ikd), parameter :: iunit=30
      character(kind=ckd,len=nc)  a
      character(kind=ckd,len=12)  key
      character(kind=ckd,len=10), parameter :: cfg_file='retran.cfg'
!
!  Set default values
!
      inp_data     = 500
      len_rblk     = 50
      rstrt_blks   = 200
      organization = ' ** Unlicensed Trial Version ** '
!
!  Read RETRAN configuration file if it exists - then process content
!
      inquire (file=cfg_file, exist=exists, err=100)
      if (.not.exists) return
!
      open (unit=iunit, file=cfg_file, status='old', access='sequential')
!
!  Read next record from configuration file
!
   10 continue
      read(iunit, 200, end=100) a
!
      do i = 1, nc 
         if (a(i:i) .eq. '[') then
            do j = i+1, nc
               if (a(j:j) .eq. ']') then
                  key    = a(i+1:j-1)
                  jsav   = j + 1
                  go to 20
               end if
            end do
         end if
      end do
!
!  Key not included as record prefix - skip rest of record
!
      go to 10
!
!  Key found -- find location of key value
!
   20 continue
      do i = jsav, nc 
         if (a(i:i) .eq. '[') then
            do j = i+1, nc
               if (a(j:j) .eq. ']') then
                  isav = i + 1
                  jsav = j - 1
                  go to 30
               end if
            end do
         end if
      end do
!
!  No key value or string -- skip rest of record
!
      go to 10
!
!  Save key value - take precidence over default value
!
   30 continue
      if (key .eq. 'inp_data') then
         read (a(isav:jsav),*) inp_data
!       
      else if (key .eq. 'len_rblk') then
         read (a(isav:jsav),*) len_rblk
!         
      else if (key .eq. 'rstrt_blks') then
         read (a(isav:jsav),*) rstrt_blks
!  
      else if (key .eq. 'organization') then
         organization = a(isav:jsav)
      endif
!
      go to 10
!
!  End of file encountered
!
  100 continue
      close (iunit)
!
      return
!
  200 format (a132)
      end subroutine set_configuration
!
      end module retran_configuration
