!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for scram related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_scram
!     allok8_react
!
!  ===============================================================================================================================
!
      module scram
!
      use kind_specs 
      use restart_block_list 
! 
      integer(kind=ikd), save :: & 
       siz_rscram,  &  !  size of the array allocated for the real scram pointer arrays
       siz_iscram,  &  !  size of the array allocated for the integer scram pointer arrays
              nsc      !  number of scram  curves                 
!
!  All of the following real, integer and logical pointer arrays have bounds (1:nsc)
!  Space is allocated and pointers associations are made in subroutine allok8_scram
!          
      real(kind=rkd), target, allocatable, save :: r_scram(:)
      real(kind=rkd), pointer, save :: &
               rc(:),  &  !  control reactivity                               
            rccal(:)      !  calculated control reactivity at the start                                                  
!
      integer(kind=ikd), target, allocatable, save :: i_scram(:)
      integer(kind=ikd), pointer, save :: &
       idxtbl(:), &  !  index of file containing table nsc           
         iscr(:), &  !  current position in table nsc                
       itscrm(:), &  !  trip number for scram nsc                    
         ktbl(:), &  !  flag for units of scram curve                
         nscr(:)     !  number of scram data points from table nsc   
!
!  ===============================================================================================================================
!
! Specifications for time versus reactivity table related array
!
      integer(kind=ikd), save :: &
         nscr_sum , &  !  sum of number of scram data points in all tables
         siz_rtscr     !  size of real array allocated for time versus reactivity table
!
      real(kind=rkd), target, allocatable, save :: r_tscr(:)
      real(kind=rkd), pointer, save :: &
           tscr(:)  !  reactivity table containing time and reactivity info       
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_scram  
!
!  Subroutine to allocate space and set pointers for scram related arrays
!     allok8_scram is called from subroutine inscrm 
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (nsc .le. 0) then
         write (6,110) nsc
         call fail
         return
      endif
!
      if (allocated (i_scram)) return
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, 1, i2,    rc, r_scram)
         call r_pntr (allocatd, 1, i2, rccal, r_scram)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_rscram   = i2
!
            allocate (r_scram(siz_rscram), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rscram, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_scram
            block  = 'r_scram'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rscram, 1, 2)
!
!     initialize integer arrays to 0
!
         else
            r_scram      = 0
         endif
      enddo
!            
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, nsc, i2, idxtbl, i_scram)
         call i_pntr (allocatd, nsc, i2,   iscr, i_scram)
         call i_pntr (allocatd, nsc, i2, itscrm, i_scram)
         call i_pntr (allocatd, nsc, i2,   ktbl, i_scram)
         call i_pntr (allocatd, nsc, i2,   nscr, i_scram)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_iscram   = i2
!
            allocate (i_scram(siz_iscram), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iscram, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_scram
            block  = 'i_scram'
            call build_restart_block_list (block, rdum, idum, ldum, siz_iscram, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_scram      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' scram arrays (allok8_scram).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for scram arrays (allok8_scram). '/ &
           10x,'Variable nsc = ',i3, ', must be > 0.')          
      end subroutine allok8_scram
!
!  ===============================================================================================================================
!
      subroutine allok8_react
!      
!  Subroutine to allocate space and set pointers for scram reactivity related arrays
!     allok8_scram is called from subroutine inscrm
!
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, i, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_tscr)) return
!      
      if (.not.allocated (i_scram)) then
         write (6,110) 
         call fail
         return
      endif
!
      if (nscr_sum .le.0) then
         write (6,120) nscr_sum
         call fail
         return
      end if
!            
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, nscr_sum * 2, i2, tscr, r_tscr)
! 
!      allocate space
!
         if (.not.allocatd) then
            siz_rtscr   = i2
!
            allocate (r_tscr(siz_rtscr), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtscr, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_tscr
            block  = 'r_tscr'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtscr, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_tscr      = 0.0d0
         endif
      enddo
! 
!  initialize idxtp for all conductors
!
      i1     = 1
      do i = 1, nsc
         idxtbl(i)  = i1
         i1        = i1 + nscr(i)
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' reactivity table arrays (allok8_react).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for reactivity table arrays (allok8_react).'/ &
          10x,'Subroutine allok8_react called before subroutine allok8_scram - programming error') 
  120 format (' ******** Error allocating space for reactivity table arrays (allok8_react).'/ &
          10x,'Sum of number of scram data points for all tables = ',i6,', must be > 0 - programming error') 
!
      end subroutine allok8_react  
!      
      end module scram
