!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for sep related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_sep
!
!  ===============================================================================================================================
!
      module separators
!
      use kind_specs 
! 
      integer(kind=ikd), save :: & 
          num_sep,  &  !  number of sep
         siz_rsep,  &  !  size of the array allocated for the real sep pointer arrays
         siz_isep      !  size of the array allocated for the integer sep pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_sep(:)
!
!  All of the following real, integer and logical pointer arrays have bounds (1:num_sep)
!  Space is allocated and pointers associations are made in subroutine allok8_sep
!
      real(kind=rkd), pointer, save :: &
         covd(:), &  !   design value of carry over.                       
         covi(:), &  !                                                     
         cund(:), &  !   initial (s.s.) value of carry under.              
         hdfs(:), &  !   height of the separator barrel (inches) user      
        rfsep(:), &  !   radius to the liquid film.                        
         rhub(:), &  !   separator hub radius (inches). user supplied      
         rrng(:), &  !   separator pick off ring radius (inches) user      
       somega(:), &  !   liquid angular velocity.                          
       thetas(:), &  !   angle of separator vanes relative to              
        volfs(:), &  !   volume of separator first stage                   
         xind(:), &  !   initial (s.s.) value of flowing quality at inlet. 
        zbias(:), &  !   elevation of bottom of separator vanes relative   
          zmd(:)     !   design value of "normalized" mixture level.       
!
      integer(kind=ikd), target, allocatable, save :: i_sep(:)
      integer(kind=ikd), pointer, save :: &
         ico1(:), &  !   either the number of a data table containing the  
         ico2(:), &  !   either the number of a data table containing the  
         icu1(:), &  !   either the number of a data table containing the  
         icu2(:), &  !   either the number of a data table containing the  
        isepn(:), &  !   total number of separators. user supplied         
         isev(:), &  !   retran volume which represents the separator.     
         iudc(:), &  !   upper downcomer volume number for mixture         
          jin(:), &  !   retran junction representing the separator inlet. 
         jrsc(:), &  !   retran junction representing the separator        
         jstm(:)     !   retran junction representing the sep. steam line  
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_sep  
!
!  Subroutine to allocate space and set pointers for sep related arrays
!     allok8_sep is called from subroutine insep
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) len, i1, i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_sep)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_sep, i2,   covd, r_sep)
         call r_pntr (allocatd, num_sep, i2,   covi, r_sep)
         call r_pntr (allocatd, num_sep, i2,   cund, r_sep)
         call r_pntr (allocatd, num_sep, i2,   hdfs, r_sep)
         call r_pntr (allocatd, num_sep, i2,  rfsep, r_sep)
         call r_pntr (allocatd, num_sep, i2,   rhub, r_sep)
         call r_pntr (allocatd, num_sep, i2,   rrng, r_sep)
         call r_pntr (allocatd, num_sep, i2, somega, r_sep)
         call r_pntr (allocatd, num_sep, i2, thetas, r_sep)
         call r_pntr (allocatd, num_sep, i2,  volfs, r_sep)
         call r_pntr (allocatd, num_sep, i2,   xind, r_sep)
         call r_pntr (allocatd, num_sep, i2,  zbias, r_sep)
         call r_pntr (allocatd, num_sep, i2,    zmd, r_sep)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rsep   = i2
!
            allocate (r_sep(siz_rsep), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rsep, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_sep
            block  = 'r_sep'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rsep, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_sep      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_sep, i2,   ico1, i_sep)
         call i_pntr (allocatd, num_sep, i2,   ico2, i_sep)
         call i_pntr (allocatd, num_sep, i2,   icu1, i_sep)
         call i_pntr (allocatd, num_sep, i2,   icu2, i_sep)
         call i_pntr (allocatd, num_sep, i2,  isepn, i_sep)
         call i_pntr (allocatd, num_sep, i2,   isev, i_sep)
         call i_pntr (allocatd, num_sep, i2,   iudc, i_sep)
         call i_pntr (allocatd, num_sep, i2,    jin, i_sep)
         call i_pntr (allocatd, num_sep, i2,   jrsc, i_sep)
         call i_pntr (allocatd, num_sep, i2,   jstm, i_sep)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_isep   = i2
!
            allocate (i_sep(siz_isep), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_isep, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_sep
            block  = 'i_sep'
            call build_restart_block_list (block, rdum, idum, ldum, siz_isep, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_sep      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' sep arrays (allok8_sep).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_sep
!
      end module separators
