!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for system related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_system
!     allok8_jdact
!
!  ===============================================================================================================================
!
      module system
!
      use kind_specs
!
      logical(kind=lkd), save :: failed  !  local error flag to avoid using nogo before it is allocated
!
      integer(kind=ikd), save :: &
          num_sys, &  !  number of systems (always = 1)
         siz_rsys, &  !  size of the array allocated for the real sys pointer arrays
         siz_isys, &  !  size of the array allocated for the integer sys pointer arrays
         siz_lsys     !  size of the array allocated for the logical sys pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_sys(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:1)
!  Space is allocated and pointers associations are made in subroutine allok8_system
!
      real(kind=rkd), pointer, save :: &
       acepsi(:), &  !  acceleration psi convergence criteria            
           ae(:), &  !  energy added                                     
        amass(:), &  !  mass added                                       
        amptm(:), &  !  cpu time spent calculating amplitude function    
       bmassa(:), &  !  mass balance of air                              
       bmassw(:), &  !  mass balance of water                            
       cnstnt(:), &  !  constant 1.0                                     
       cpusec(:), &  !  accumulated cpu time                             
       delayp(:), &  !  delayed power fraction                           
       dmassa(:), &  !  mass of air leaked                               
       dmassw(:), &  !  mass of water leaked                             
       dmepsi(:), &  !  mass convergence criteria                        
        dtold(:), &  !  old time step value
           eb(:), &  !  energy balance                                   
        epsim(:), &  !  mass balance convergence criteria                
           fe(:), &  !  energy in fuel                                   
        hdelt(:), &  !  min enth chg between vol and jun to deactivate   
           he(:), &  !  energy extracted by heat exchanger               
        hepsi(:), &  !  volume energy balance convergence criteria       
       opnorm(:)     !  old time step normalized reactor power
!
      real(kind=rkd), pointer, save :: &
       pmppow(:), &  !  total heat addition to the system from the pumps 
        pnorm(:), &  !  normalized reactor power                         
        pnucl(:), &  !  initial nuclear power level                      
        power(:), &  !  system power                                     
        powri(:), &  !  initial power level                              
       prompp(:), &  !  prompt power fraction                            
       ptherm(:), &  !  initial thermal power level                      
        qloss(:), &  !  total rate of heat removal                       
       reshtm(:), &  !  cpu time spent in shape function solution        
        spatm(:), &  !  cpu time spent in 1-d kinetics solution          
       steptm(:), &  !  cpu time spent in shape-amplitude iteration      
        timex(:), &  !  problem time                                     
       vsepsi(:), &  !  slip velocity convergence criteria               
        xectm(:)     !  cpu time spent in cross section formulation      
!
      integer(kind=ikd), target, allocatable, save :: i_sys(:)
      integer(kind=ikd), pointer, save :: &
       iccflo(:), &  !  enth tran countercurrent flow deactivation flag  
       iceqst(:), &  !  flow network number to which the core sections belong                                           
       identr(:), &  !  trip id to deactivate enth tran (everywhere)     
       ientrn(:), &  !  activation flag for improved error handling      
       iflwrv(:), &  !  enth tran flow reversal deactivation flag        
       iselct(:), &  !  index for entran sel. deact. subfile setdfl      
       jssdbg(:), &  !  reserved for future use                          
       lcount(:), &  !  maximum number of steady state iterations        
       maxnod(:)     !  max. number of nodes specified for any conductor 
!
      logical(kind=lkd), target, allocatable, save :: l_sys(:)
      logical(kind=lkd), pointer, save :: &
         nogo(:)     !  error flag                                       
!
!  ===============================================================================================================================
!  Specifications and space allocations for selective enthalpy deactivation data
!  ===============================================================================================================================
!  All of the following integer pointers are rank (1: netset )
!  netjun(:) is ranked (1: numjdact - total # of junctions in all sets )
!  Space is allocated and pointers associations are made in subroutine allok8_jdact
!
      integer(kind=ikd), save :: &
           netset, &  !  # of selective deactivation entran sets
        num_jdact, &  !  total number of junctions for all sets
        siz_ijdact    !  size of the array allocated for the integer enthalpy deactivation pointer arrays
!
      integer(kind=ikd), target, allocatable, save :: i_jdact(:)
      integer(kind=ikd), pointer, save :: &
         idactj(:), &  !  index of first junction 
          ietid(:), &  !  trip ids for data sets          
         netjun(:), &  !  index of junctions for each data set
         netsiz(:)     !  size of each deactivation data set
!
!  ===============================================================================================================================
!
      contains
!
!  ===============================================================================================================================
!
      subroutine allok8_system
!
!  Subroutine to allocate space and set pointers for sys related arrays
!     Note: num_sys is always defined as 1
!
!     allok8_sys is called from subroutine input
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_sys)) return
!
      num_sys = 1
      failed  = .false.
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_sys, i2, acepsi, r_sys)
         call r_pntr (allocatd, num_sys, i2,     ae, r_sys)
         call r_pntr (allocatd, num_sys, i2,  amass, r_sys)
         call r_pntr (allocatd, num_sys, i2,  amptm, r_sys)
         call r_pntr (allocatd, num_sys, i2, bmassa, r_sys)
         call r_pntr (allocatd, num_sys, i2, bmassw, r_sys)
         call r_pntr (allocatd, num_sys, i2, cnstnt, r_sys)
         call r_pntr (allocatd, num_sys, i2, cpusec, r_sys)
         call r_pntr (allocatd, num_sys, i2, delayp, r_sys)
         call r_pntr (allocatd, num_sys, i2, dmassa, r_sys)
         call r_pntr (allocatd, num_sys, i2, dmassw, r_sys)
         call r_pntr (allocatd, num_sys, i2, dmepsi, r_sys)
         call r_pntr (allocatd, num_sys, i2,  dtold, r_sys)
         call r_pntr (allocatd, num_sys, i2,     eb, r_sys)
         call r_pntr (allocatd, num_sys, i2,  epsim, r_sys)
         call r_pntr (allocatd, num_sys, i2,     fe, r_sys)
         call r_pntr (allocatd, num_sys, i2,  hdelt, r_sys)
         call r_pntr (allocatd, num_sys, i2,     he, r_sys)
         call r_pntr (allocatd, num_sys, i2,  hepsi, r_sys)
         call r_pntr (allocatd, num_sys, i2, opnorm, r_sys)
         call r_pntr (allocatd, num_sys, i2, pmppow, r_sys)
         call r_pntr (allocatd, num_sys, i2,  pnorm, r_sys)
         call r_pntr (allocatd, num_sys, i2,  pnucl, r_sys)
         call r_pntr (allocatd, num_sys, i2,  power, r_sys)
         call r_pntr (allocatd, num_sys, i2,  powri, r_sys)
         call r_pntr (allocatd, num_sys, i2, prompp, r_sys)
         call r_pntr (allocatd, num_sys, i2, ptherm, r_sys)
         call r_pntr (allocatd, num_sys, i2,  qloss, r_sys)
         call r_pntr (allocatd, num_sys, i2, reshtm, r_sys)
         call r_pntr (allocatd, num_sys, i2,  spatm, r_sys)
         call r_pntr (allocatd, num_sys, i2, steptm, r_sys)
         call r_pntr (allocatd, num_sys, i2,  timex, r_sys)
         call r_pntr (allocatd, num_sys, i2, vsepsi, r_sys)
         call r_pntr (allocatd, num_sys, i2,  xectm, r_sys)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rsys   = i2
!
            allocate (r_sys(siz_rsys), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rsys, status
               failed = .true.
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_sys
            block  = 'r_sys'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rsys, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_sys      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_sys, i2, iccflo, i_sys)
         call i_pntr (allocatd, num_sys, i2, iceqst, i_sys)
         call i_pntr (allocatd, num_sys, i2, identr, i_sys)
         call i_pntr (allocatd, num_sys, i2, ientrn, i_sys)
         call i_pntr (allocatd, num_sys, i2, iflwrv, i_sys)
         call i_pntr (allocatd, num_sys, i2, iselct, i_sys)
         call i_pntr (allocatd, num_sys, i2, jssdbg, i_sys)
         call i_pntr (allocatd, num_sys, i2, lcount, i_sys)
         call i_pntr (allocatd, num_sys, i2, maxnod, i_sys)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_isys   = i2
!
            allocate (i_sys(siz_isys), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_isys, status
               failed = .true.
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_sys
            block  = 'i_sys'
            call build_restart_block_list (block, rdum, idum, ldum, siz_isys, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_sys      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_sys, i2,   nogo, l_sys)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_lsys   = i2
!
            allocate (l_sys(siz_lsys), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_lsys, status
               failed = .true.
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_sys
            block  = 'l_sys'
            call build_restart_block_list (block, rdum, idum, ldum, siz_lsys, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_sys      = .false.
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' system arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_system
!
!  ===============================================================================================================================
!
      subroutine allok8_jdact
!
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (netset .le. 0 .or. num_jdact .le. 0) then
         write (6,110) netset, num_jdact
         call fail
         return
      endif
!      
      if (allocated (i_jdact)) return
!            
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, netset, i2, idactj, i_jdact)
         call i_pntr (allocatd, netset, i2,  ietid, i_jdact)         
         call i_pntr (allocatd, netset, i2, netsiz, i_jdact)
         call i_pntr (allocatd, num_jdact, i2, netjun, i_jdact)         
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_ijdact  = i2
!
            allocate (i_jdact(siz_ijdact), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ijdact, status
               call fail
            endif
!
!     initialize integer arrays to 0
!
         else
            i_jdact      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' entran deactivation arrays (allok8_jdact).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for entran deactivation arrays (allok8_jdact). '/ &
          10x,'Variable net_set = ',i3, ' and num_jdact = ',i3,' , must be > 0.')
!      
      end subroutine allok8_jdact
!
      end module system
