!$Header$
!$Copyright$
      module time_data
!
      use kind_specs
      use problem_dimensions
      use system
!      
!  ===============================================================================================================================
!  Specifications and space allocations for timing routines
!  ===============================================================================================================================
! parameter definitions
!
      integer(kind=ikd) name1, name2
      parameter (name1 = 29     )
      parameter (name2 = 2*name1)   
!
      real(kind=rkd),   dimension(2, name1), save ::  array    
      character(len=8), dimension(name1),    save :: carray    
      real(kind=rkd), save    ::    cptim                         
      real(kind=rkd), save    ::    cpu, cpusav, del0, delt_t, elpsd, fact, percnt 
      integer(kind=ikd), save :: igon, ipage, kgo, knum, number 
      logical, save :: isubfl
      integer(kind=ikd), dimension(name1), save ::  ncals   
      character(len=8), save :: waltim
      character(10), dimension(3), save :: cval
      integer(kind=ikd), dimension(8), save :: ival      
!      
      data isubfl/.false./
      data  array /  name2*0.d0 /
      data carray / 'advflo  ', 'contrl  ', 'dirsol  ', 'edit    ', 'energy  ', 'genmat  ', 'junprp  ', &
                    'pltape  ', 'powrt   ', 'presur  ', 'przr    ', 'pumps   ', 'slabht  ', 'slip    ', &
                    'solve3  ', 'solve4  ', 'statew  ', 'statum  ', 'turbot  ', 'ampltd  ', 'reshap  ', &
                    'spactm  ', 'xsectn  ', 'drive2  ', 'ststat  ', 'drive3  ', 'tran    ', 'reacty  ', &
                    'gencof  ' /
!
      data ncals  / name1*0 /
      data cpusav / 1.D20 /, ipage / 0 /
!
      contains   
!  ===============================================================================================================================
!
      subroutine timint (number)
!
!  Subroutine timint is an internal timing routine used to time various factions of retran computations.
!
!  timint is called by subroutines advflo, contrl, oxgen, dnbm, edinit, editre, edit, energy, finish, inacc, inarea, inbubl,
!  inckv, incnt1, incore, indata, indnb, inedte, infill, infit, ingap, ingtrn, inheat, inhtxq, injun, inmodh, inneq, inplot,
!  inpm, inpump, inqx1, inrken, inrstr, inrtrn, insep, inslab, instgn, intrip, intstp, inturb, intv, invol, junprp, jvedit,
!  ploter, plotr, pltape, powrt, presur, prnply, przr, pumps, redtap, reedit, reshap, rkenp, slabht, slip, solve3, solve4,
!  spactm, statew, statum, ststat, tran, trpsum and tubbot and drive2, drive3, reacty and gencof
!
!  number   = table number of routine to be timed      (input)
!
      use kind_specs
      use edit_headers
      use problem_dimensions
!      
      implicit none
!
      integer(kind=ikd) :: number
      intent (in) number
!      intent (out) cptim
!
      if ( number<=0 ) then
         write (6,9010)
         call fabend
      end if
!      
      call timer(delt_t)
!     
      cpu    = delt_t
      del0   = delt_t
      elpsd  = delt_t
      return
!
 9010 format ('0******** Error, argument in call to timint must be greater than zero')
!
      end subroutine timint
!
!  ===============================================================================================================================
!
      subroutine time_edit
!
      use edit_headers
      use problem_dimensions
      use units
!      
      implicit none
!            
      integer(kind=ikd) :: i
      call timer(delt_t)
!      
      elpsd  = elpsd + delt_t
      cpu    = cpu + delt_t
      isubfl  = (allocated(l_sys)) .and. (ldmp(1)>=-1)
      if ( isubfl ) then
         cpusec(1) = cpusec(1) + delt_t + del0
         del0   = 0.0D0
      else
         del0   = del0 + delt_t
      end if
      if ( kgo<0 ) then
!
         if ( cpu<cpusav ) ipage = 0
         ipage  = ipage + 1
         cpusav = cpu
!         
         call time8( )
!                                                                               
         write (6,9040) version_header, run_date, ipage
         write (6,9050) licensed_org, waltim
         write (6,9060) problem_title, cpu
         if ( isubfl ) then
            if ( nogo(1) ) write (6,9070)
         end if
         write (6,9080)
         if ( igon==0 ) go to 300
!
         write (6,9090) elpsd
         write (6,9100)
         if ( elpsd>zero ) fact = 1.0D2/elpsd
         do i = 1, name1
            if ( ncals(i)>0 ) then
               percnt = array(1,i)*fact
               write (6,9110) carray(i), array(1,i), percnt, ncals(i)
            end if
         end do
         return
      else if ( kgo==0 ) then
!
!  start timing of subroutine
!
         if ( number<0 ) then
!
!  stop timing of subroutine
!
            knum   = abs(number)
            if ( array(2,knum)==zero ) write (6,9030) carray(knum)
            ncals(knum)   = ncals(knum) + 1
            array(1,knum) = array(1,knum) + cpu - array(2,knum)
            array(2,knum) = zero
         else
            if ( array(2,number)/=zero ) write (6,9020) carray(number)
            array(2,number) = cpu
         end if
         return
      else
!
         cptim  = cpu
         return
      end if
  300 continue
      return
!
 9020 format ('0******** Error- time in did not equal zero when subroutine ',a8,'entered subroutine timseg ******')
 9030 format ('0******** Error- time in was equal to zero when subroutine ',a8,'ended in subroutine timseg *****')
 9040 format ('1',1a80,' --  NRC APPROVED VERSION       ',1a8,2x,'Page',i5)
 9050 format (' LIGHT WATER REACTOR TRANSIENT ANALYSIS PACKAGE -- ','LICENSED FOR USE BY ',1a32,10x,1a8) 
 9060 format (1x,1a80,32x,'CPU TIME =',f9.3)
 9070 format (80x,'********** ERROR, JOB BEING ABORTED **********')
 9080 format (                                                                                                                     &
     &' * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *',&
     &' * * * * *')
 9090 format ('0 TIMING EDIT    ACCUMULATED TIME =',f9.3,' SECONDS'/)
 9100 format ('  SUBROUTINE',11x,'CPU TIME (SEC)',15x,'PERCENT',5x,                                                                &
             &'   NUMBER OF TIMING CALLS'/'  **********           **************            *************',                        &
             &'     **********************')
 9110 format (4x,a8,10x,f9.3,20x,f7.2,11x,i7)      
      end subroutine time_edit
!      
!  ===============================================================================================================================
!
      subroutine cputim (cptim)
!      
!  returns elapsed cpu time
!     
      implicit none
!    
      real(kind=rkd) :: cptim  
      intent(inout) cptim
!            
      kgo    = 1
      call time_edit
      end subroutine cputim
!
!  ===============================================================================================================================
!
      subroutine ztedit (number)
!      
!  timing array edit routine
!
      implicit none
!           
      integer(kind=ikd) :: number
      intent(in) number
!      
      igon =  1
      kgo  = -1
      call time_edit
!
      end subroutine ztedit
!
!  ===============================================================================================================================      
!
      subroutine subpg (idum)
!
!  pick up cpu time and print heading
!
      use kind_specs
      use system
      implicit none
!            
      integer(kind=ikd) :: idum
      igon    = 0
      kgo     = -1
      call time_edit
      end subroutine subpg
!      
!  ===============================================================================================================================      

      end module time_data
