!$Header$
!$Copyright$
!  ===============================================================================================================================
!  Specifications and space allocations for time-step related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_time_steps
!
!  ===============================================================================================================================
!
      module time_steps
!
      use kind_specs
      use problem_dimensions
      use restart_block_list 
!
      integer(kind=ikd), save :: &
          num_tstep, &  !  number of time-step sets
         siz_rtstep, &  !  size of the array allocated for the real tstep sets pointer arrays
         siz_itstep, &  !  size of the array allocated for the integer tstep sets pointer arrays
         siz_ltstep     !  size of the array allocated for the logical tstep sets pointer arrays
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_tstep)
!  Space is allocated and pointers associations are made in subroutine allok8_tstep
!
      real(kind=rkd), target, allocatable, save :: r_tstep(:)
      real(kind=rkd), pointer, save :: &
        condt(:), &  !  coefficient in constant*dt time-step algorithm   
         cour(:), &  !  iterative numerics courant t.s. control coeff.   
        deltm(:), &  !  maximum time step size                           
        dtmin(:), &  !  minimum allowable time step size                 
       epmdot(:), &  !  epsi multiplier when mdot present                
         epsi(:), &  !  iterative numerics energy convergence criteria   
        epsiu(:), &  !  flow reversal energy convergence coefficient     
       epsmax(:), &  !  currently not used                               
       epsmin(:), &  !  currently not used                               
      fnpower(:), &  !  iterative numerics power change factor
       pratef(:), &  !  iterative numerics pressure rate change factor
        tlast(:)     !  end of interval                                  
!
      integer(kind=ikd), target, allocatable, save :: i_tstep(:)
      integer(kind=ikd), pointer, save :: &
       ievery(:), &  !  detailed edit control                            
         nchk(:), &  !  time step control option flag                    
         ndmp(:), &  !  number of time steps per timing edit
         nmaj(:), &  !  number of time steps per major edit             
         nmin(:), &  !  number of time steps per minor edit
         nrst(:), &  !  number of time steps per restart record
        nshap(:), &  !  space time kinetics shape function interval      
         nvbc(:)     !  number of time steps per VIPRE boundary condition file record
!
      logical(kind=lkd), target, allocatable, save :: l_tstep(:)
      logical(kind=ikd), pointer, save :: &
         lvbc(:)     !  flag indicating VIPRE BC file will be written 
!       
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for time step control algorithm related arrays
!
      integer(kind=ikd), save :: &
          num_tspc, &  !  always = 1
         siz_rtspc, &  !  size of the array allocated for the real tstep control algorithm pointer arrays
         siz_itspc, &  !  size of the array allocated for the integer tstep control algorithm pointer arrays
         siz_ltspc, &  !  size of the array allocated for the logical tstep control algorithm pointer arrays           
            siz_xy     !  size of the arrays use to save minimum error curve fit data
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_tspc)
!  Space is allocated and pointers associations are made in subroutine allok8_time_steps
!
      real(kind=rkd), target, allocatable, save :: r_tspc(:)
      real(kind=rkd), pointer, save :: &
        dtavg(:), &  !  average time step size between minor edits       
       dt_bnd(:), &  !  time increment to next edit boundary
       dtedit(:), &  !  last time-step size - used by errlog for edit
        dtmax(:), &  !  maximum allowable time step length criteria (5) option reserved for future use                   
       dtnext(:), &  !  decremented time used to give even time time step boundaries
       dtolds(:), &  !  previous time step upon return to tstp, current dt until return                                  
          fc3(:), &  !  fc3 ... fc9 = time step constant factors         
          fc4(:), &  !                                                   
          fc5(:), &  !                                                   
          fc6(:), &  !                                                   
          fc7(:), &  !                                                   
          fc8(:), &  !                                                   
          fc9(:), &  !                                                   
       plinmx(:), &  !  maximum error in the linearized pressure 
       timlst(:), &  !  time last edit occurred
       timsav(:)     !  time last timing edit (written from etscon) was written
!
      integer(kind=ikd), target, allocatable, save :: i_tspc(:)
      integer(kind=ikd), pointer, save :: &
        iclst(:), &  !  case flag for last time-step selection algorithm
        icomp(:), &  !  flag used to reset time step control algorithm counters on time step reset                      
         idmp(:), &  !  number of complete edits since last timing edit  
       idmpsv(:), &  !  used to save previous value of idmp - used when time step is reset
         imaj(:), &  !  number of brief edits since last complete edit   
       imajsv(:), &  !  used to save previous value of imaj - used when time step is reset
         imin(:), &  !  number of time steps since last brief edit       
       iminsv(:), &  !  used to save previous value of imin - used when time step is reset
         iplt(:), &  !  number of time steps since last plot save        
       ipltsv(:), &  !  used to save previous value of iplt - used when time step is reset
        itspl(:), &  !  array used to save the count for volumes and junctions controlling time-step size
                     !  selection for each algorithm.  The array is itsmax = (6*nvol + 4*njun + ntrip) words long.
         ivbc(:), &  !  counter used to select vbc file write            
       ivbcsv(:), &  !  used to save previous value of ivbc - used when time step is reset            
       ivtstp(:), &  !  index for volume with max lin pressure error     
        kntdt(:), &  !  array used to save the total number of times a particular algorithm controlled the 
                     !  time-step size.  The array is kntmax = 13 words long.
       kntlst(:), &  !  pointer to last entry in kntdt array             
         nset(:), &  !  current interval index                           
        nstep(:), &  !  current time-step number                         
       nstpsv(:), &  !  prior value of nstep
        ntset(:), &  !  number of time-step sets
       ntssav(:), &  !  actual time-step count when last timing edit was made (written from etscon)                         
        nttsp(:)     !  total number of actual time steps used                  
!
      logical(kind=lkd), target, allocatable, save :: l_tspc(:)
      logical(kind=lkd), pointer, save :: &
       endstp(:), &  !  .true. indicates last time step interval reduced 
       impcvg(:), &  !  iterative numerics convergence flag (logical), .true. if iter num converged
       impskp(:), &  !  logical flag used by the iter numrcs to skip calcs if failure has been detected
        reduc(:)     !  .true. indicates previous time step reduced      
!
!  The following real pointer arrays are Rank (1:siz_xy). They are associated with the r_tspc block after the time-step 
!  control pointer arrays.  Space is allocated and pointers associations are made in subroutine allok8_time_steps.
!
      real(kind=rkd), pointer, save :: &
       x_meff(:), &  !  x array used to store minimum effort time step information
       y_meff(:)     !  y array used to store minimum effort time step information 
      contains
!
!  ===============================================================================================================================
      subroutine allok8_time_steps
!
!  Subroutine to allocate space and set pointers for time step related arrays
!     Note: num_tstep must be defined before allok8_time_steps is called
!
!     allok8_time_steps is called from subroutine intstp
! 
      use kind_specs
      use restart_block_list
! 
!  Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!            
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i2, l, len, num_itspl, num_kntdt, status
      logical(kind=lkd) allocatd, ll
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_tstep .le. 0) then
         write (6,110) num_tstep
         call fail
         return
      endif
!
      if (allocated (r_tstep)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      num_tspc = 1
      siz_xy   = 10   
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_tstep, i2,   condt, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,    cour, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,   deltm, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,   dtmin, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,  epmdot, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,    epsi, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,   epsiu, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,  epsmax, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,  epsmin, r_tstep)
         call r_pntr (allocatd, num_tstep, i2, fnpower, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,  pratef, r_tstep)
         call r_pntr (allocatd, num_tstep, i2,   tlast, r_tstep)
!
         len    = i2
         i2     = 0
         call r_pntr (allocatd, num_tspc, i2,  dtavg, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, dt_bnd, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, dtedit, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,  dtmax, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, dtnext, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, dtolds, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc3, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc4, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc5, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc6, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc7, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc8, r_tspc)
         call r_pntr (allocatd, num_tspc, i2,    fc9, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, plinmx, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, timlst, r_tspc)
         call r_pntr (allocatd, num_tspc, i2, timsav, r_tspc)
!         
         call r_pntr (allocatd,   siz_xy, i2, x_meff, r_tspc)
         call r_pntr (allocatd,   siz_xy, i2, y_meff, r_tspc)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rtstep   = len
!
            allocate (r_tstep(siz_rtstep), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtstep, status
               call fail
            endif
!
            siz_rtspc =  i2
            allocate ( r_tspc(siz_rtspc), stat = status)            
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtspc, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_tstep
            block  = 'r_tstep'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtstep, 1, 0)
!
            rdum   => r_tspc
            block  = 'r_tspc'
!
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtspc, 1, 2)            
!            
!     initialize real arrays to 0.0d0
!
         else
            r_tstep      = 0.0d0
            r_tspc       = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_tstep, i2, ievery, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   nchk, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   ndmp, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   nmaj, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   nmin, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   nrst, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,  nshap, i_tstep)
         call i_pntr (allocatd, num_tstep, i2,   nvbc, i_tstep)
!
         len    = i2
         i2     = 0
         call i_pntr (allocatd, num_tspc, i2,  iclst, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,  icomp, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   idmp, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, idmpsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   imaj, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, imajsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   imin, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, iminsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   iplt, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, ipltsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   ivbc, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, ivbcsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, ivtstp, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, kntlst, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,   nset, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,  nstep, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, nstpsv, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,  ntset, i_tspc)
         call i_pntr (allocatd, num_tspc, i2, ntssav, i_tspc)
         call i_pntr (allocatd, num_tspc, i2,  nttsp, i_tspc)
!
         num_itspl = 6*nvol(1) + 4*njun(1) + ntrp(1)
         num_kntdt = 13
!         
         call i_pntr (allocatd, num_itspl, i2,  itspl, i_tspc)         
         call i_pntr (allocatd, num_kntdt, i2,  kntdt, i_tspc)
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_itstep   = len
!
            allocate (i_tstep(siz_itstep), stat=status)
!           
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itstep, status
               call fail
            endif
!
            siz_itspc    = i2            
            allocate (i_tspc(siz_itspc), stat = status)
!            
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itspc, status
               call fail
            endif            
!
!        Add block to list of blocks included in restart file
!
            idum   => i_tstep
            block  = 'i_tstep'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_itstep, 2, 0)
!
            idum   => i_tspc
            block  = 'i_tspc'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_itspc, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_tstep      = 0
            i_tspc       = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_tstep, i2,   lvbc, l_tstep)
!        
         len  =  i2
         i2   =  0
!          
         call l_pntr (allocatd, num_tspc, i2, endstp, l_tspc)
         call l_pntr (allocatd, num_tspc, i2, impcvg, l_tspc)
         call l_pntr (allocatd, num_tspc, i2, impskp, l_tspc)
         call l_pntr (allocatd, num_tspc, i2,  reduc, l_tspc)
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_ltstep   = len
!
            allocate (l_tstep(siz_ltstep), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_ltstep, status
               call fail
            endif
!
            siz_ltspc  = i2
!            
            allocate (l_tspc(siz_ltspc), stat = status)
!             
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_ltspc, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_tstep
            block  = 'l_tstep'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ltstep, 3, 0)
!
            ldum   => l_tspc
            block  = 'l_tspc'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ltspc, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_tstep = .false.
            l_tspc  = .false.
         endif
      enddo
!
  100 format (/' ******** Error allocating space for ',a8,' time-step arrays (allok8_time_steps).'/ &
           10x,'Length requested = ',i8/ &
           10x,'Status flag returned from allocate = ',i3)
  110 format (/' ******** Error allocating space for time-step arrays (allok8_time_steps). '/ &
           10x,'Variable num_  = ',i3, ', must be > 0.')
!          
      end subroutine allok8_time_steps
!
      end module time_steps
