!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for trip and trip id related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_trips
!     allok8_tripids
!     allok8_rst_trips
!
!  ===============================================================================================================================
!
      module trips
!
      use kind_specs
!
      integer(kind=ikd), save :: &
           num_trip, &  !  number of trips
          siz_rtrip, &  !  size of the array allocated for the real trip pointer arrays
          siz_itrip, &  !  size of the array allocated for the integer trip pointer arrays
          siz_ltrip     !  size of the array allocated for the logical trip pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_trip(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_trip)
!  Space is allocated and pointers associations are made in subroutine allok8_trips
!
      real(kind=rkd), pointer, save :: &
          abx(:), &  !  previous time step value of monitored signal     
        delay(:), &  !  trip delay after setpoint is passed              
         oabx(:), &  !  old time step value of abx                       
        otsig(:), &  !  old time step value of tsig                      
        setpt(:), &  !  signal setpoint type according to idsig 
       trptl1(:), &  !  trip label field 1                               
       trptl2(:), &  !  trip label field 2                               
       trptl3(:), &  !  trip label field 3                               
       trptl4(:), &  !  trip label field 4                                                
         tsig(:)     !  time to reach setpoint                           
!
      integer(kind=ikd), target, allocatable, save :: i_trip(:)
      integer(kind=ikd), pointer, save :: &
        idsig(:), &  !  input signal index                               
        idtrp(:), &  !  index for trip type                              
          ix1(:), &  !  signal 1 index (volume or junction number)       
          ix2(:)     !  signal 2 index (nonzero for differences  )       
!
      logical(kind=lkd), target, allocatable, save :: l_trip(:)
      logical(kind=lkd), pointer, save :: &
         flog(:), &  !  true means setpoint reached                      
        oflog(:)     !  old time step value of flog                      
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!
      integer(kind=ikd), save :: &
          num_tid, &  !  number of unique trip ids
         siz_rtid, &  !  size of the array allocated for the real tid pointer arrays
         siz_itid, &  !  size of the array allocated for the integer tid pointer arrays
         siz_ltid     !  size of the array allocated for the logical tid pointer arrays
!
!  Specifications for trip id arrays
!
!  All of the following integer pointer arrays are Rank (1:num_tid)
!  Space is allocated and pointers associations are made in the first call to subroutine 
!  allok8_tripids
!
      integer(kind=ikd), target, allocatable, save :: i_tid(:)
      integer(kind=ikd), pointer, save :: &
       oldtrp(:), &  !  old trip id                                      
       tonoff(:)     !  variable edited on minor edit request for trip status (1 = on;  0 = off)                               
                     
!
!  All of the following real and integer pointer arrays are Rank (1:num_tid,1:2).  Space is allocated 
!  and pointers associations are made in subroutine allok8_tripid. For all of the following 2D arrays 
!  the second subscript is used to define the "foreward" (=1) trips and the reset (=2) trips for trip id
!  variable, unless otherwise noted.
!
      integer(kind=ikd), save :: &
         siz_r2tid, &  !  size of the array allocated for the real 2D tripid pointer arrays
         siz_i2tid, &  !  size of the array allocated for the integer 2D tripid pointer arrays
         siz_l2tid     !  size of the array allocated for the logical 2D tripid pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r2_tid(:,:)  !  real target array for trip id 2D pointer arrays
      real(kind=rkd), pointer, save :: &
        otrpt(:,:), &  !  time to reach setpoint + delay time, old time step for second dimension = 1 or                                              
                       !  time to reach reset setpoint + delay time, old time step for second dimension = 2                                        
         trpt(:,:)     !  time to reach setpoint + delay time for second dimension = 1 or              
                       !  time to reach reset setpoint + delay time for second dimension = 2             
!
      integer(kind=ikd), target, allocatable, save :: i2_tid(:,:)  !  integer target array for trip id 2D pointer arrays
      integer(kind=ikd), pointer, save :: &
         idxt(:,:), &  !  index of array elements containing forward trip & reset trip numbers with 
                       !  common id. if second dimension is 1 then it is forward trip numbers and if it is 2 
                       !  then it is reset trip. if there is only one trip with this id, 
                       !  idxt or idxtr is the trip number and no array used
         nfid(:,:)     !  number of forward trips or reset trips with common id. second dimension = 1 for forward trip
                       !  and second dimension = 2 for reset trips           
!       
      logical(kind=lkd), target, allocatable, save :: l2_tid(:,:)  !  integer target array for trip id 2D pointer arrays
      logical(kind=lkd), pointer, save :: &
         flag(:,:), &  !  true means setpoint reached and delay time has expired if second dimension = 1 or                                         
                       !  true means reset setpoint reached and delay time has expired if second dimension = 2                                     
        oflag(:,:)     !  true means setpoint reached and delay time has expired, old time step if second dimension = 1 or                           
                       !  true means reset setpoint reached and delay time has expired, old time step if second dimension = 2                            
!       
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for trip id array containing trip numbers associated with given id
!
!  The integer pointer array is Rank (1:num_nids) where num_nids is the sum of all 
!  nfid and nrid.  Indices pointing to the list of forward and reverse trip ids are 
!  ixdt and idxtr respectively.
!  Space is allocated in the second call to subroutine allok8_tripids
!
      integer(kind=ikd), save :: &
          num_nids     !  number of trip ids (forward and reverse)          
!
      integer(kind=ikd), target, allocatable, save :: &
          trip_list(:) !  list of trips associated with a given trip id (nfid or nrid > 1)
                       !  (idxt and idxtr are indicies for the start of the lists for each trip id)
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for trip variables used during restart run
!  
      integer(kind=ikd), save :: &
          siz_rst_rtrip, &  !  size of the array allocated for the real trip pointer arrays
          siz_rst_itrip     !  size of the array allocated for the integer trip pointer arrays
!          
!  The pointers array are Ranked (1:num_trip)
!
      integer(kind=ikd), target, allocatable, save :: i_rst_trip(:)
      integer(kind=ikd), save, pointer :: &
        rst_idsig(:), &  !  input signal index                               
        rst_idtrp(:), &  !  index for trip type                              
          rst_ix1(:), &  !  signal 1 index (volume or junction number)       
          rst_ix2(:)     !  signal 2 index (nonzero for differences  )       
!
      real(kind=rkd), target, allocatable, save :: r_rst_trip(:)
      real(kind=rkd), save, pointer :: &         
        rst_setpt(:), &  !  set point
        rst_delay(:)     !  trip delay after setpoint is passed
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_trips
!
!  Subroutine to allocate space and set pointers for trip related arrays
!     Note: num_trip must be defined before allok8_trip is called
!
!     allok8_trips is called from subroutine intrip
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=12) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_trip .le. 0) then
         write (6,110) num_trip
         call fail
         return
      endif
!
      if (allocated (r_trip)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_trip, i2,    abx, r_trip)
         call r_pntr (allocatd, num_trip, i2,  delay, r_trip)
         call r_pntr (allocatd, num_trip, i2,   oabx, r_trip)
         call r_pntr (allocatd, num_trip, i2,  otsig, r_trip)
         call r_pntr (allocatd, num_trip, i2,  setpt, r_trip)
         call r_pntr (allocatd, num_trip, i2, trptl1, r_trip)
         call r_pntr (allocatd, num_trip, i2, trptl2, r_trip)
         call r_pntr (allocatd, num_trip, i2, trptl3, r_trip)
         call r_pntr (allocatd, num_trip, i2, trptl4, r_trip)         
         call r_pntr (allocatd, num_trip, i2,   tsig, r_trip)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rtrip   = i2
!
            allocate (r_trip(siz_rtrip), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtrip, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_trip
            block  = 'r_trip'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtrip, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_trip      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_trip, i2,  idsig, i_trip)
         call i_pntr (allocatd, num_trip, i2,  idtrp, i_trip)
         call i_pntr (allocatd, num_trip, i2,    ix1, i_trip)
         call i_pntr (allocatd, num_trip, i2,    ix2, i_trip)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_itrip   = i2
!
            allocate (i_trip(siz_itrip), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itrip, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_trip
            block  = 'i_trip'
            call build_restart_block_list (block, rdum, idum, ldum, siz_itrip, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_trip      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_trip, i2,   flog, l_trip)
         call l_pntr (allocatd, num_trip, i2,  oflog, l_trip)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ltrip   = i2
!
            allocate (l_trip(siz_ltrip), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_ltrip, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_trip
            block  = 'l_trip'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ltrip, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_trip      = .false.
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' trip arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for trip arrays. '/ &
          10x,'Variable num_trip = ',i3, ', must be > 0.')
      end subroutine allok8_trips
!
!  ===============================================================================================================================
      subroutine allok8_tripids
!
!  Subroutine to allocate space and set pointers for tid related arrays
!     Note: num_tid must be defined before allok8_tripids is called the first time.
!           Prior to the second call to allok8_tripids, the nfid and nrid arrays 
!           must be initialized.
!
!     allok8_tripids is called from subroutine intrip
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
      use s_r2_pntr
      use s_i2_pntr
      use s_l2_pntr      
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, n2d, status
      logical(kind=lkd) allocatd
!      
      parameter (n2d    = 2)            
!
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)  !  dummy logical pointer
!
      if (num_tid .le. 0) then
         write (6,110) num_tid
         call fail
         return
      endif
!
      if (allocated (l2_tid)) go to 10
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call i_pntr (allocatd, num_tid, i2, oldtrp, i_tid)
         call i_pntr (allocatd, num_tid, i2, tonoff, i_tid)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_itid   = i2
!
            allocate (i_tid(siz_itid), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itid, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_tid
            block  = 'i_tid'
            call build_restart_block_list (block, rdum, idum, ldum, siz_itid, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_tid      = 0
         end if
!
         i2       = 0
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!         
         call i2_pntr (allocatd, num_tid, n2d,  i2,   idxt, i2_tid)
         call i2_pntr (allocatd, num_tid, n2d,  i2,   nfid, i2_tid)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_i2tid   = i2
!
            allocate (i2_tid(siz_i2tid,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_i2tid, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            i2dum  => i2_tid
            block  = 'i2_tid'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_i2tid, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i2_tid      = 0
         end if
!         
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------     
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         call r2_pntr (allocatd, num_tid, n2d, i2,  otrpt, r2_tid)
         call r2_pntr (allocatd, num_tid, n2d, i2,   trpt, r2_tid)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_r2tid   = i2
!
            allocate (r2_tid(siz_r2tid,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtid, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            r2dum  => r2_tid
            block  = 'r2_tid'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_r2tid, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r2_tid      = 0.0d0
         endif
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         call l2_pntr (allocatd, num_tid, n2d, i2,   flag, l2_tid)
         call l2_pntr (allocatd, num_tid, n2d, i2,  oflag, l2_tid)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_l2tid   = i2
!
            allocate (l2_tid(siz_l2tid,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_l2tid, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            l2dum   => l2_tid
            block   = 'l2_tid'
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_l2tid, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l2_tid      = .false.
         endif
      enddo
!
      return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Allocate array to store forward and reverse trip numbers associated with a 
!  given trip id
!
   10 continue
      if (allocated (trip_list)) return
! 
      num_nids = 0
!
      do l = 1, num_tid
         if (nfid(l,1) .gt. 1) then
            num_nids = num_nids + nfid(l,1)
         endif
         if (nfid(l,2) .gt. 1) then
            num_nids = num_nids + nfid(l,2)
         endif
      enddo
!
      if (num_nids .gt. 0) then
!
         allocate (trip_list(num_nids), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, num_nids, status
               call fail
               return
            endif
      endif
!
!        Add block to list of blocks included in restart file
!
            idum   => trip_list
            block  = 'trip_list'
            call build_restart_block_list (block, rdum, idum, ldum, num_nids, 2, 2)
!     
  100 format (' ******** Error allocating space for ',a8,' trip id arrays (allok8_tripids).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for trip id arrays (allok8_tripids). '/ &
          10x,'Variable num_tid = ',i3, ', must be > 0.')
      end subroutine allok8_tripids
!
!   ======================================================================================================================
!
      subroutine allok8_rst_trips
!
!  Subroutine to allocate space and set pointers for trip_arrays for restart run
!
!     allok8_rst_trips is called from subroutine intrip
!
      use kind_specs 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
      use s_r2_pntr
      use s_i2_pntr
      use s_l2_pntr      
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, n2d, status
      logical(kind=lkd) allocatd
!      
      parameter (n2d    = 2)            
!
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)  !  dummy logical pointer
!
      if (num_trip .le. 0) then
         write (6,110) num_trip
         call fail
         return
      endif
!
      if (allocated (r_rst_trip)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call i_pntr (allocatd, num_trip, i2, rst_idsig, i_rst_trip)
         call i_pntr (allocatd, num_trip, i2, rst_idtrp, i_rst_trip)
         call i_pntr (allocatd, num_trip, i2,   rst_ix1, i_rst_trip)
         call i_pntr (allocatd, num_trip, i2,   rst_ix2, i_rst_trip)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_rst_itrip   = i2
!
            allocate (i_rst_trip(siz_rst_itrip), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_rst_itrip, status
               call fail
               return
            endif
!
!     initialize integer arrays to 0
!
         else
            i_rst_trip      = 0
         end if
!
         i2       = 0
!        
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------     
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_trip, i2, rst_delay, r_rst_trip)
         call r_pntr (allocatd, num_trip, i2, rst_setpt, r_rst_trip)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rst_rtrip   = i2
!
            allocate (r_rst_trip(siz_rst_rtrip), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rst_rtrip, status
               call fail
            endif
!
!     initialize real arrays to 0.0d0
!
         else
            r_rst_trip      = 0.0d0
         endif
!         
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' restart trip arrays (allok8_rst_trips).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for restart trip arrays (allok8_rst_trips). '/ &
          10x,'Variable num_tid = ',i3, ', must be > 0.')
      end subroutine allok8_rst_trips
!
      end module trips
