!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for two-region nonequilibrium volume related arrays
!
!  This module also includes the following subroutine(s)
!
!     allok8_two_reg_neq
!     allok8_two_reg_jun_der
! 
!  ===============================================================================================================================
!
      module two_reg_neq
!
      use kind_specs 
      use restart_block_list 
! 
      integer(kind=ikd), save :: & 
          num_neq,  &  !  number of two-region nonequilibrium volumes
         siz_rneq,  &  !  size of the array allocated for the real neq pointer arrays
         siz_ineq      !  size of the array allocated for the integer neq pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_neq(:)
!
!  The following real and integer pointer arrays have bounds (1:num_neq)
!  Space is allocated and pointers associations are made in subroutine allok8_two_reg_neq
!
      real(kind=rkd), pointer, save :: &
         cone(:), &  !  energy transfer associated with conm             
         conm(:), &  !  mass rained from vapor region to liquid          
        delm1(:), &  !  change in region 1 mass                          
        delm2(:), &  !  change in region 2 mass                          
        delu1(:), &  !  change in region 1 int energy                    
        delu2(:), &  !  change in region 2 int energy                    
        dh1dm(:), &  !  partial of reg 1 enth wrt total mass             
       dh1dm2(:), &  !  partial of reg 1 enth wrt reg 2 mass             
        dh1du(:), &  !  partial of reg 1 enth wrt total int energy       
       dh1du2(:), &  !  partial of reg 1 enth wrt reg 2 int energy       
        dh2dm(:), &  !  partial of reg 2 enth wrt total mass             
       dh2dm2(:), &  !  partial of reg 2 enth wrt reg 2 mass             
        dh2du(:), &  !  partial of reg 2 enth wrt total int energy       
       dh2du2(:), &  !  partial of reg 2 enth wrt reg 2 int energy       
       dm2dm1(:), &  !  partial of reg 2 mass tran wrt mass of reg 1     
       dm2dm2(:), &  !  partial of reg 2 mass tran wrt mass of reg 2     
        dm2dt(:), &  !  dm/dt for region 2 (vapor)                       
       dm2du1(:), &  !  partial of reg 2 mass tran wrt int ener of reg 1 
       dm2du2(:), &  !  partial of reg 2 mass tran wrt int ener of reg 2 
        dpdm2(:)     !  partial of pressure wrt vap reg mass                     
!
      real(kind=rkd), pointer, save :: &
        dpdu2(:), &  !  partial of pressure wrt vap reg int energy             
        dt1dm(:), &  !  partial of reg 1 temperature wrt total mass      
       dt1dm2(:), &  !  partial of reg 1 temperature wrt reg 2 mass      
        dt1du(:), &  !  partial of reg 1 temperature wrt total int ener  
       dt1du2(:), &  !  partial of reg 1 temperature wrt reg 2 int ener  
        dt2dm(:), &  !  partial of reg 2 temperature wrt total mass      
       dt2dm2(:), &  !  partial of reg 2 temperature wrt reg 2 mass      
        dt2du(:), &  !  partial of reg 2 temperature wrt total int ener  
       dt2du2(:), &  !  partial of reg 2 temperature wrt reg 2 int ener  
       du1dm1(:), &  !  partial of reg 1 int energy wrt total mass       
       du1dm2(:), &  !  partial of reg 1 int energy wrt total int ener   
       du1du1(:), &  !  partial of reg 1 int energy wrt reg 2 mass       
       du1du2(:), &  !  partial of reg 1 int energy wrt reg 2 int ener   
       du2dm1(:), &  !  partial of reg 2 ener tran wrt mass of reg 1     
       du2dm2(:), &  !  partial of reg 2 ener tran wrt mass of reg 2     
        du2dt(:), &  !  du/dt for region 2 (vapor)                       
       du2du1(:), &  !  partial of reg 2 ener tran wrt int ener of reg 1 
       du2du2(:), &  !  partial of reg 2 ener tran wrt int ener of reg 2 
        dv1dm(:), &  !  partial of reg 1 spec vol wrt total mass         
       dv1dm2(:)     !  partial of reg 1 spec vol wrt reg 2 mass         
!
      real(kind=rkd), pointer, save :: &
        dv1du(:), &  !  partial of reg 1 spec vol wrt total int energy         
       dv1du2(:), &  !  partial of reg 1 spec vol wrt reg 2 int energy   
        dv2dm(:), &  !  partial of reg 2 spec vol wrt total mass         
       dv2dm2(:), &  !  partial of reg 2 spec vol wrt reg 2 mass         
        dv2du(:), &  !  partial of reg 2 spec vol wrt total int energy   
       dv2du2(:), &  !  partial of reg 2 spec vol wrt reg 2 int energy   
       dvldm1(:), &  !  partial of liq reg vol (reg 1) wrt reg 1 mass    
       dvldm2(:), &  !  partial of liq reg vol (reg 1) wrt reg 2 mass    
       dvldu1(:), &  !  partial of liq reg vol (reg 1) wrt reg 1 int enr 
       dvldu2(:), &  !  partial of liq reg vol (reg 1) wrt reg 2 int enr 
        dx1dm(:), &  !  partial of reg 1 quality wrt total mass          
       dx1dm2(:), &  !  partial of reg 1 quality wrt reg 2 mass          
        dx1du(:), &  !  partial of reg 1 quality wrt total int ener      
       dx1du2(:), &  !  partial of reg 1 quality wrt reg 2 int ener      
        dx2dm(:), &  !  partial of reg 2 quality wrt total mass          
       dx2dm2(:), &  !  partial of reg 2 quality wrt reg 2 mass          
        dx2du(:), &  !  partial of reg 2 quality wrt total int ener      
       dx2du2(:), &  !  partial of reg 2 quality wrt reg 2 int ener      
         flae(:), &  !  energy added to vapor region from flashing       
         flam(:)     !  liquid mass flashed from liquid to vapor region         
!
      real(kind=rkd), pointer, save :: &
         nele(:), &  !  non-eq. liquid region total energy               
        neleo(:), &  !  non-eq. liquid region total energy (previous time step)                             
         nelh(:), &  !  non-eq. liquid region average enthalpy           
         nelm(:), &  !  non-eq. liquid region total mass                 
        nelmo(:), &  !  non-eq. liquid region mass (previous time step)  
         nelt(:), &  !  non-eq. liquid region temperature                
         nelv(:), &  !  non-eq. liquid region specific volume            
       nelvol(:), &  !  non-eq. liquid region volume                     
         nelx(:), &  !  non-eq. liquid region quality                    
         neve(:), &  !  non-eq. vapor region total energy                
        neveo(:), &  !  non-eq. vapor region total energy (previous time step)                             
         nevh(:), &  !  non-eq. vapor region average enthalpy            
         nevm(:), &  !  non-eq. vapor region mass                        
        nevmo(:), &  !  non-eq. vapor region mass (previous time step)   
         nevt(:), &  !  non-eq. vapor region temperature                 
         nevv(:), &  !  non-eq. vapor region specific volume             
         nevx(:), &  !  non-eq. vapor region quality                     
       qliqid(:), &  !  liquid region heating rate                       
       qvapor(:), &  !  vapor region heating rate                        
        qvlht(:)     !  inter region energy exchange                              
!
      real(kind=rkd), pointer, save :: &
        vlhtc(:), &  !  inter region heat transfer coefficient           
       volold(:), &  !  old liquid region volume                         
        vrain(:), &  !  rain out velocity                                
       wspray(:)                                                    
!       
      integer(kind=ikd), target, allocatable, save :: i_neq(:)
      integer(kind=ikd), pointer, save :: &
       iphsel(:), &  !  phase index for liquid region                    
       iphsev(:), &  !  phase index for vapor region                     
       ipvbub(:), &  !  non-equilibrium bubble rise vel option flag      
       nprvol(:), &  !  volume                          
       isepfg(:), &  !  non-equilibrium separator flag                   
       idvhtc(:), &  !                                               
       nevlno(:)     !  volume number of nonequilibrium volume
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for junction derivatives related to two-region nonequilibrium volumes
! 
      integer(kind=ikd), save :: & 
            num_jder,  &  !  number of two-region nonequilibrium volumes
           siz_rjder       !  size of the array allocated for the real neq pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_jder(:)
!
!  The following real pointer arrays have bounds (1:num_jder)
!  Space is allocated and pointers associations are made in subroutine allok8_two-reg_jun_der
!
      real(kind=rkd), pointer, save :: &
       dfdwsp(:), &  !  partial of vap reg mass eq wrt spray jun flow    
        wcstm(:)     !  condensed steam flow rate       
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_jder,1:2)
!  Space is allocated and pointers associations are made in subroutine allok8_two-reg_jun_der. For 
!  all of the following 2D arrays the second subscript is used to define the "from" side (=1)
!  and the "to" side (=2) for a junction variable, unless otherwise noted.
!
      integer(kind=ikd), save :: & 
           siz_r2jder       !  size of the array allocated for the real neq pointer arrays
!           
      real(kind=rkd), target, allocatable, save :: r2_jder(:,:)
      real(kind=rkd), pointer, save :: &
          cx1(:,:), &  !  fraction of jun flow entering or leaving, from or to vol 
        dcdm1(:,:), &  !  partial of cx1(:,i) wrt mass in, from or to vol              
        dcdm2(:,:), &  !  partial of cx1(:,i) wrt mass in, from or to vol              
        dcdu1(:,:), &  !  partial of cx1(:,i) wrt energy in, from or to vol            
        dcdu2(:,:), &  !  partial of cx1(:,i) wrt energy in, from or to vol            
        dfsdm(:,:), &  !  partial of spray energy conv wrt mass in, from or to vol 
        dfsdu(:,:), &  !  partial of spray energy conv wrt enrg in, from or to vol 
        dfsm2(:,:), &  !  partial of spray energy conv wrt reg 2 mass in, from or to vol                                          
        dfsu2(:,:), &  !  partial of spray energy conv wrt reg 2 enrg in, from or tovol                                          
        dwdm1(:,:), &  !  deriv of reg 1 enrg eq conv wrt mass in, from or to vol        
        dwdm2(:,:), &  !  deriv of reg 2 enrg eq conv wrt mass in, from or to vol
        dwdu1(:,:), &  !  deriv of reg 1 enrg eq conv wrt enrg in, from or to vol
        dwdu2(:,:), &  !  deriv of reg 2 enrg eq conv wrt enrg in, from or to vol
        dwhm2(:,:), &  !  deriv of mix enrg eq conv wrt reg 2 m in, from or to vol
        dwhu2(:,:), &  !  deriv of mix enrg eq conv wrt reg 2 e in, from or to vol     
        h2cnv(:,:)     !  enthalpy leaving vap reg of, from or to vol
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_two_reg_neq 
!
!  Subroutine to allocate space and set pointers for neq related arrays
!     Note: num_neq must be defined before allok8_two-reg_neq is called
!
!     allok8_two_reg_nonequilibrium is called from subroutine inneq
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i2, status, l
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_neq .le. 0) then
         write (6,110) num_neq
         call fail
         return
      endif
!
      if (allocated (r_neq)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_neq, i2,   cone, r_neq)
         call r_pntr (allocatd, num_neq, i2,   conm, r_neq)
         call r_pntr (allocatd, num_neq, i2,  delm1, r_neq)
         call r_pntr (allocatd, num_neq, i2,  delm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  delu1, r_neq)
         call r_pntr (allocatd, num_neq, i2,  delu2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dh1dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dh1dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dh1du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dh1du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dh2dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dh2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dh2du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dh2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2, dm2dm1, r_neq)
         call r_pntr (allocatd, num_neq, i2, dm2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dm2dt, r_neq)
         call r_pntr (allocatd, num_neq, i2, dm2du1, r_neq)
         call r_pntr (allocatd, num_neq, i2, dm2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dpdm2, r_neq)
!         
         call r_pntr (allocatd, num_neq, i2,  dpdu2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dt1dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dt1dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dt1du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dt1du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dt2dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dt2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dt2du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dt2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2, du1dm1, r_neq)
         call r_pntr (allocatd, num_neq, i2, du1dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2, du1du1, r_neq)
         call r_pntr (allocatd, num_neq, i2, du1du2, r_neq)
         call r_pntr (allocatd, num_neq, i2, du2dm1, r_neq)
         call r_pntr (allocatd, num_neq, i2, du2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  du2dt, r_neq)
         call r_pntr (allocatd, num_neq, i2, du2du1, r_neq)
         call r_pntr (allocatd, num_neq, i2, du2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dv1dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dv1dm2, r_neq)
!         
         call r_pntr (allocatd, num_neq, i2,  dv1du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dv1du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dv2dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dv2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dv2du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dv2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2, dvldm1, r_neq)
         call r_pntr (allocatd, num_neq, i2, dvldm2, r_neq)
         call r_pntr (allocatd, num_neq, i2, dvldu1, r_neq)
         call r_pntr (allocatd, num_neq, i2, dvldu2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dx1dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dx1dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dx1du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dx1du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dx2dm, r_neq)
         call r_pntr (allocatd, num_neq, i2, dx2dm2, r_neq)
         call r_pntr (allocatd, num_neq, i2,  dx2du, r_neq)
         call r_pntr (allocatd, num_neq, i2, dx2du2, r_neq)
         call r_pntr (allocatd, num_neq, i2,   flae, r_neq)
         call r_pntr (allocatd, num_neq, i2,   flam, r_neq)
!         
         call r_pntr (allocatd, num_neq, i2,   nele, r_neq)
         call r_pntr (allocatd, num_neq, i2,  neleo, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nelh, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nelm, r_neq)
         call r_pntr (allocatd, num_neq, i2,  nelmo, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nelt, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nelv, r_neq)
         call r_pntr (allocatd, num_neq, i2, nelvol, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nelx, r_neq)
         call r_pntr (allocatd, num_neq, i2,   neve, r_neq)
         call r_pntr (allocatd, num_neq, i2,  neveo, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nevh, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nevm, r_neq)
         call r_pntr (allocatd, num_neq, i2,  nevmo, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nevt, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nevv, r_neq)
         call r_pntr (allocatd, num_neq, i2,   nevx, r_neq)
         call r_pntr (allocatd, num_neq, i2, qliqid, r_neq)
         call r_pntr (allocatd, num_neq, i2, qvapor, r_neq)
         call r_pntr (allocatd, num_neq, i2,  qvlht, r_neq)
!         
         call r_pntr (allocatd, num_neq, i2,  vlhtc, r_neq)
         call r_pntr (allocatd, num_neq, i2, volold, r_neq)
         call r_pntr (allocatd, num_neq, i2,  vrain, r_neq)
         call r_pntr (allocatd, num_neq, i2, wspray, r_neq)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rneq   = i2
!
            allocate (r_neq(siz_rneq), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rneq, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_neq
            block  = 'r_neq'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rneq, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_neq      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_neq, i2, idvhtc, i_neq)
         call i_pntr (allocatd, num_neq, i2, iphsel, i_neq)
         call i_pntr (allocatd, num_neq, i2, iphsev, i_neq)
         call i_pntr (allocatd, num_neq, i2, ipvbub, i_neq)
         call i_pntr (allocatd, num_neq, i2, isepfg, i_neq)
         call i_pntr (allocatd, num_neq, i2, nevlno, i_neq)
         call i_pntr (allocatd, num_neq, i2, nprvol, i_neq)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ineq   = i2
!
            allocate (i_neq(siz_ineq), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ineq, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_neq
            block  = 'i_neq'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ineq, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_neq      = 0
         endif
      enddo
      return
!
  100 format (' ******** Error allocating space for ',a8,' two-region nonequilibrium volume arrays ' &
              '(allok8_two_reg_nonequilibrium).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for  two-region nonequilibrium volume arrays ' &
              '(allok8_two_reg_nonequilibrium).'/ &
           10x,'Variable num_neq = ',i3, ', must be > 0.')
!           
      end subroutine allok8_two_reg_neq
!
!  ===============================================================================================================================
      subroutine allok8_two_reg_jun_der
!
!  Subroutine to allocate space and set pointers for junction derivatives associated with the two-region nonequilibrium model
!     Note: num_jder must be defined before allok8_two-reg_jun_der is called
!
!     allok8_two_reg_jun_der is called from subroutine inneq
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      use s_r2_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i2, status, l, n2d
      logical(kind=lkd) allocatd
!
      parameter (n2d   = 2)      
!      
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)  !  dummy logical pointer
!
      if (num_jder .le. 0) then
         write (6,110) num_jder
         call fail
         return
      endif
!
      if (allocated (r_jder)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_jder, i2, dfdwsp, r_jder)
         call r_pntr (allocatd, num_jder, i2,  wcstm, r_jder)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rjder   = i2
!
            allocate (r_jder(siz_rjder), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rjder, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_jder
            block  = 'r_jder'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rjder, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_jder      = 0.0d0
         endif
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         i2       = 0
!
         call r2_pntr (allocatd, num_jder, n2d, i2,   cx1, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dcdm1, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dcdm2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dcdu1, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dcdu2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dfsdm, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dfsdu, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dfsm2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dfsu2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dwdm1, r2_jder)        
         call r2_pntr (allocatd, num_jder, n2d, i2, dwdm2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dwdu1, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dwdu2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dwhm2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, dwhu2, r2_jder)
         call r2_pntr (allocatd, num_jder, n2d, i2, h2cnv, r2_jder)
!        
!      allocate space
!
         if (.not.allocatd) then
            siz_r2jder   = i2
!
            allocate (r2_jder(siz_r2jder,1:n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_r2jder, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            r2dum  => r2_jder
            block  = 'r2_jder'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_r2jder, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r2_jder      = 0.0d0
         end if 
         
      end do
!      
      return
!
  100 format (' ******** Error allocating space for ',a8,' junction derivative arrays for two-reg noneq vol   ' &
              '(allok8_two_reg_jun_derivaties).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for junction derivative arrays for two-reg noneq vol ' &
              '(allok8_two_reg_jun_derivaties).'/ &
           10x,'Variable num_jder = ',i3, ', must be > 0.')
!           
      end subroutine allok8_two_reg_jun_der
!
      end module two_reg_neq
