!$Header$
!$Copyright$
!===========================================================================================================
!  Module units contains the general units conversions used in RETRAN-3D engineering calculations.  It also
!  contains a units conversion array that contains conversion constants that are used to convert output 
!  edits to si units.  It also contains siunt logical that is used to indicate that output should be si.
!
!  Module unit also contains the following function
!
!     cftok
!
!===========================================================================================================
      module units 
!      
!  Commonly used global constants
! 
      use kind_specs 
! 
      real(kind=rkd), parameter ::  &
          btumwh = 3.413d6           ,  &  !
          cfmgps = .2228009d-2       ,  &  !
          foot   = 12.d0             ,  &  !
          fpbtu  = 778.169262d0      ,  &  !
          gravty = 32.174d0          ,  &  !
          g120pi = 307.2192d0        ,  &  !
          pi     = 3.14159265359d0   ,  &  !
          roteng = 0.13460123d-3     ,  &  !
          sechr  = 3600.d0           ,  &  !
          sqinch = 144.d0            ,  &  !
          zero   = 0.0d0             ,  &  !
          half   = 0.5d0             ,  &  !
          one    = 1.0d0             ,  &  !
          two    = 2.0d0             ,  &  !
          three  = 3.0d0             ,  &  !
          btufp  = one/fpbtu         ,  &  !
          hke    = btufp/(two*gravty),  &  !
          hrsec  = one/sechr         ,  &  !
          ccc    = sqinch/fpbtu      ,  &  !
          t0     = 459.67d0                !
!
      integer(kind=ikd) nunt
      parameter (nunt   = 33)
      real(kind=rkd), dimension(nunt)               ::  conv 
      character(kind=ckd, len=8), dimension(nunt,2) ::  units_label
!      
!  The conv array defines the unit conversion factors for conversion from english to si as follows.
!  Note: (si value) = conv(i)*(english value)
!
!                      conversion          english         si        
!
      data conv(01) / 1.00000d+0 /,  &  !  (deg f)         (k)       
           conv(02) / 5.67824d+0 /,  &  !  b/h-f-f2      (w/m2-k)             
           conv(03) / 4.18680d+0 /,  &  !  btu/lb-f      kj/kg-k              
           conv(04) / 6.89475d+0 /,  &  !   (psia)        (kpa)               
           conv(05) / 4.88242d+0 /,  &  !  lb/hr-f2      kg/hr-m2             
           conv(06) / 6.24302d-2 /,  &  !  (ft3/lb)      (m3/kg)              
           conv(07) / 2.32444d+0 /,  &  !  (btu/lb)      (kj/kg)              
           conv(08) / 4.53592d-1 /,  &  !    (lb)          (kg)               
           conv(09) / 1.60179d+1 /,  &  !  (lb/ft3)      (kg/m3)              
           conv(10) / 1.05448d+0 /,  &  !   (btu)          (kj)               
           conv(11) / 4.53593d-1 /,  &  !  (lb/sec)      (kg/sec)             
           conv(12) / 2.92826d-1 /,  &  !  (btu/hr)      (watts)              
           conv(13) / 1.05417d+0 /,  &  !  (btu/s)         (kw)               
           conv(14) / 3.04801d-1 /,  &  !  (ft/sec)      (m/sec)              
           conv(15) / 3.04801d-1 /,  &  !   (feet)       meters               
           conv(16) / 2.83171d-2 /,  &  !   (ft3)          (m3)               
           conv(17) / 9.29034d-2 /,  &  !   (ft2)          (m2)               
           conv(18) / 1.00000d+0 /,  &  !   (sec)         (sec)               
           conv(19) / 1.00000d+0 /,  &  !   (mw)           (mw)               
           conv(20) / 1.00000d+0 /,  &  !   (rpm)         (rpm)               
           conv(21) / 3.15438d+0 /,  &  !  b/hr-f2       watts/m2             
           conv(22) / 1.00000d+0 /,  &  !    ($)           ($)                
           conv(23) / 1.00000d+0 /,  &  !  (1/sec)       (1/sec)              
           conv(24) / 3.28083d+0 /,  &  !  (kw/ft)        (kw/m)              
           conv(25) / 1.00000d+0 /,  &  !   (ppm)         (ppm)               
           conv(26) / 1.00000d+0 /,  &  !  (k-1)/k       (k-1)/k              
           conv(27) / 1.00000d+0 /,  &  !                                     
           conv(28) / 3.04801d-1 /,  &  !  (ft/s2)        (m/s2)              
           conv(29) / 5.55556d-1 /,  &  !  (deg f)       (deg k)              
           conv(30) / 1.00000d+0 /,  &  !                                     
           conv(31) / 3.28083d+0 /,  &  !  (1/ft)         (1/m)               
           conv(32) / 1.00000d+0 /,  &  !  (rps)          (rps)               
           conv(33) / 1.00000d+0 /      ! (cal/gm)       (cal/gm)               
!
!  The unit_label array defines the unit labels for the corresponding conversion factors given above.  The second
!  subscript is as follows -- 1=english and 2=si.
!
      data units_label(01,1) /  '(DEG F) ' /,  &
           units_label(02,1) /  'B/H-F-F2' /,  &
           units_label(03,1) /  'BTU/LB-F' /,  &
           units_label(04,1) /  ' (PSIA) ' /,  &
           units_label(05,1) /  'LB/HR-F2' /,  &
           units_label(06,1) /  '(FT3/LB)' /,  &
           units_label(07,1) /  '(BTU/LB)' /,  &
           units_label(08,1) /  '  (LB)  ' /,  &
           units_label(09,1) /  '(LB/FT3)' /,  &
           units_label(10,1) /  ' (BTU)  ' /,  &
           units_label(11,1) /  '(LB/SEC)' /,  &
           units_label(12,1) /  '(BTU/HR)' /,  &
           units_label(13,1) /  '(BTU/S) ' /,  &
           units_label(14,1) /  '(FT/SEC)' /,  &
           units_label(15,1) /  ' (FEET) ' /,  &
           units_label(16,1) /  ' (FT3)  ' /,  &
           units_label(17,1) /  ' (FT2)  ' /,  &
           units_label(18,1) /  ' (SEC)  ' /,  &
           units_label(19,1) /  ' (MW)   ' /,  &
           units_label(20,1) /  ' (RPM)  ' /,  &
           units_label(21,1) /  'B/HR-F2 ' /,  &
           units_label(22,1) /  '  ($)   ' /,  &
           units_label(23,1) /  '(1/SEC) ' /,  &
           units_label(24,1) /  '(KW/FT) ' /,  &
           units_label(25,1) /  ' (PPM)  ' /,  &
           units_label(26,1) /  '(K-1)/K ' /,  &
           units_label(27,1) /  '        ' /,  &
           units_label(28,1) /  '(FT/S2) ' /,  &
           units_label(29,1) /  '(DEG F) ' /,  &
           units_label(30,1) /  '(F**1/2)' /,  &
           units_label(31,1) /  ' (1/FT) ' /,  &
           units_label(32,1) /  ' (RPS)  ' /,  &
           units_label(33,1) /  '(CAL/GM)' /
      data units_label(01,2) /  '(DEG K) ' /,  &
           units_label(02,2) /  '(W/M2-K)' /,  &
           units_label(03,2) /  'KJ/KG-K ' /,  &
           units_label(04,2) /  ' (KPA)  ' /,  &
           units_label(05,2) /  'KG/HR-M2' /,  &
           units_label(06,2) /  '(M3/KG) ' /,  &
           units_label(07,2) /  '(KJ/KG) ' /,  &
           units_label(08,2) /  '  (KG)  ' /,  &
           units_label(09,2) /  '(KG/M3) ' /,  &
           units_label(10,2) /  '  (KJ)  ' /,  &
           units_label(11,2) /  '(KG/SEC)' /,  &
           units_label(12,2) /  '(WATTS) ' /,  &
           units_label(13,2) /  '  (KW)  ' /,  &
           units_label(14,2) /  '(M/SEC) ' /,  &
           units_label(15,2) /  'METERS  ' /,  &
           units_label(16,2) /  '  (M3)  ' /,  &
           units_label(17,2) /  '  (M2)  ' /,  &
           units_label(18,2) /  ' (SEC)  ' /,  &
           units_label(19,2) /  '  (MW)  ' /,  &
           units_label(20,2) /  ' (RPM)  ' /,  &
           units_label(21,2) /  'WATTS/M2' /,  &
           units_label(22,2) /  '  ($)   ' /,  &
           units_label(23,2) /  '(1/SEC) ' /,  &
           units_label(24,2) /  ' (KW/M) ' /,  &
           units_label(25,2) /  ' (PPM)  ' /,  &
           units_label(26,2) /  '(K-1)/K ' /,  &
           units_label(27,2) /  '        ' /,  &
           units_label(28,2) /  ' (M/S2) ' /,  &
           units_label(29,2) /  '(DEG K) ' /,  &
           units_label(30,2) /  '(K**1/2)' /,  &
           units_label(31,2) /  ' (1/M)  ' /,  &
           units_label(32,2) /  ' (RPS)  ' /,  &
           units_label(33,2) /  '(CAL/GM)' / 
!
      logical(kind=lkd) ::  &
           siunit   !  logical to request si output (when .true.)
!
      data siunit / .false. /
!
      contains      
!
!  ===============================================================================================================================
      function cftok (arg)
!
!  function cftok converts arg from deg f to k
!
!     arg    = temperature in deg F                                                         (input)
!
!  cftok is called by edit, edit_pressurizer, edit_five_eq, edit_conductors and edit_core
!
      use kind_specs
!
      implicit none
!
!  Dummy arguments
!
      real(kind=rkd) :: arg
!
      intent (in) arg
!
! Local variables
!
      real(kind=rkd) :: cftok
!
      cftok = conv(29)*(arg - 32.0d0) + 273.15d0
!
      end function cftok
      end module units
