!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for valve related arrays
!
!  This module also includes the following subroutine(s)
!
!     allok8_valves
!     allok8_ttime
! 
!  ===============================================================================================================================
!
      module valves
!
      use kind_specs
!
      integer(kind=ikd), save :: &
           num_vlv, &  !  number of valves
          siz_rvlv, &  !  size of the array allocated for the real vlv pointer arrays
          siz_ivlv, &  !  size of the array allocated for the integer vlv pointer arrays
          siz_lvlv     !  size of the array allocated for the logical vlv pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_vlv(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_vlv)
!  Space is allocated and pointers associations are made in subroutine allok8_valves
!
      real(kind=rkd), pointer, save :: &
!
          cv1(:), &  !  forward flow friction coefficient (if itcv = 0 or 1)
!                       area*moment arm (if itcv = -1)
          cv2(:), &  !  reverse flow friction coefficient (valve open) (if itcv = 0 or 1)
!                       moment of inertia (if itcv = -1)
          cv3(:), &  !  reverse flow friction coefficient (valve closed) (if itcv = 0 or 1)
!                       damping constant (if itcv = -1)
          pcv(:), &  !  back pres to close check valve (if itcv = 0 or 1)
!                       not used for this valve type (if itcv = -1)
        thdot(:), &  !  not used for this valve type (if itcv = 0 or 1)
!                       door angle velocity (if itcv = -1)
        theta(:), &  !  not used for this valve type (if itcv = 0 or 1)
!                       door angle (if itcv = -1)
         tinc(:), &  !  time, in time-versus-area table, corresponding to area of valve at trip action                     
        tvlst(:), &  !  last time step value of time used in area-versus-time data tables                                 
         vano(:)     !  last time step value of normalized area          
        
!
      integer(kind=ikd), target, allocatable, save :: i_vlv(:)
      integer(kind=ikd), pointer, save :: &
         iacv(:), &  !  control system idc for area-versus-time from control system                                   
        iacv2(:), &  !  index of time-versus-area data table for valve reset                                            
        idxti(:), &  !  index pointing to subfile containing area-versus-time data tables                                 
         itcv(:), &  !  type of valve if itcv is equal to -1;                                       
        itime(:), &  !  memory index pointing to last value of time used in area-versus-time data tables                  
        ntime(:)     !  number of data points in area-versus-time tables 
!
      logical(kind=lkd), target, allocatable, save :: l_vlv(:)
      logical(kind=lkd), pointer, save :: &
         open(:), &  !  .true. means valve in open position              
       opning(:)     !  .true. means opening or open -- .false. means closing or closed                  
!
!  ===============================================================================================================================
!
!  subfile ttime ( time v/s area information for inverted closing curves)
!    
      integer(kind=ikd), save :: &
         num_ttime, &  !  number of time v/s area data pairs * 2 for closing curves
        siz_rttime     !  size of the array allocated for the real ttime pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_ttime(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_ttime)
!  index into ttime array for each valve having iacv2(:) > 0 will be stored in idxti(:)
!  Space is allocated and pointers associations are made in subroutine allok8_ttime
!
      real(kind=rkd), pointer, save :: &
        ttime(:)     !   time v/s area tables in pairs 1st item = time, 2nd item = normalized area
!        
      contains
!
!  ===============================================================================================================================
      subroutine allok8_valves
!
!  Subroutine to allocate space and set pointers for vlv related arrays
!     Note: num_vlv must be defined before allok8_vlv is called
!
!     allok8_vlv is called from subroutine inckv
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_vlv .le. 0) then
         write (6,110) num_vlv
         call fail
         return
      endif
!
      if (allocated (r_vlv)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_vlv, i2,    cv1, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,    cv2, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,    cv3, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,    pcv, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,  thdot, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,  theta, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,   tinc, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,  tvlst, r_vlv)
         call r_pntr (allocatd, num_vlv, i2,   vano, r_vlv)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rvlv   = i2
!
            allocate (r_vlv(siz_rvlv), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rvlv, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_vlv
            block  = 'r_vlv'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rvlv, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_vlv      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_vlv, i2,   iacv, i_vlv)
         call i_pntr (allocatd, num_vlv, i2,  iacv2, i_vlv)
         call i_pntr (allocatd, num_vlv, i2,  idxti, i_vlv)
         call i_pntr (allocatd, num_vlv, i2,   itcv, i_vlv)
         call i_pntr (allocatd, num_vlv, i2,  itime, i_vlv)
         call i_pntr (allocatd, num_vlv, i2,  ntime, i_vlv)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ivlv   = i2
!
            allocate (i_vlv(siz_ivlv), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ivlv, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_vlv
            block  = 'i_vlv'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ivlv, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_vlv      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_vlv, i2,   open, l_vlv)
         call l_pntr (allocatd, num_vlv, i2, opning, l_vlv)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_lvlv   = i2
!
            allocate (l_vlv(siz_lvlv), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_lvlv, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_vlv
            block  = 'l_vlv'
            call build_restart_block_list (block, rdum, idum, ldum, siz_lvlv, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_vlv      = .false.
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' valve arrays. (allok8_valves)'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for valve arrays. (allok8_valves)'/ &
          10x,'Variable num_vlv = ',i3, ', must be > 0.')
!          
      end subroutine allok8_valves
!
!  ===============================================================================================================================
!
      subroutine allok8_ttime
!
!  Subroutine to allocate space and set pointers for ttime related arrays
!     Note: num_ttime must be defined before allok8_ttime is called
!
!     allok8_ttime is called from subroutine inckv
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_vlv .le. 0) then
         write (6,110) num_vlv
         call fail
         return
      endif
!
      if (allocated (r_ttime)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_ttime, i2,  ttime, r_ttime)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rttime   = i2
!
            allocate (r_ttime(siz_rttime), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rttime, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_ttime
            block  = 'r_ttime'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rttime, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_ttime      = 0.0d0
         endif
      enddo
!
      return
!      
  100 format (' ******** Error allocating space for ',a8,' valve ttime arrays. (allok8_ttime)'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for valve ttime arrays. (allok8_ttime)'/ &
          10x,'Variable num_ttime = ',i3, ', must be > 0.')
!          
      end subroutine allok8_ttime
!      
!  ===============================================================================================================================
!
      end module valves
!
!  ===============================================================================================================================
      
