!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for volume related arrays
!
!  This module also includes the following subroutine(s)
!
!     allok8_volumes
!     allok8_ttrn
!     allok8_ttmesh
! 
!  ===============================================================================================================================
!
      module volumes
!
      use kind_specs
!
      integer(kind=ikd), save :: &
          num_vol, &  !  number of volumes
         siz_rvol, &  !  size of the array allocated for the real vol pointer arrays
         siz_ivol, &  !  size of the array allocated for the integer vol pointer arrays
         siz_lvol     !  size of the array allocated for the logical vol pointer arrays
!
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_vol)
!  Space is allocated and pointers associations are made in subroutine allok8_volumes
!
      real(kind=rkd), target, allocatable, save :: r_vol(:)
      real(kind=rkd), pointer, save :: &
        afdai(:), &  !  normal to interfacial area density ratio         
       areaif(:), &  !  interfacial area density ( ft**2 / ft**3 )       
       armaso(:), &  !  previous time step noncondensible mass           
       armass(:), &  !  noncondensible mass                              
         aved(:), &  !  average density                                  
         avex(:), &  !  average mixture quality = gas mass / total mass  
       csbpgm(:), &  !  gaseous mixture specific heat                    
       csbpnc(:), &  !  noncondensible heat capacity                     
        csubp(:), &  !  mixture heat capacity                            
       csubpf(:), &  !  saturated liquid heat capacity                   
       csubpg(:), &  !  saturated vapor heat capacity                    
        delma(:), &  !  most recent change in noncon mass                
       delmal(:), &  !  change in noncon mass from matrix inversion      
       delmao(:), &  !  previous time step change in noncon mass         
        delmf(:), &  !  most recent change in fluid mass                 
       delmfo(:), &  !  previous time step change in fluid mass          
        delmv(:), &  !  most recent change in vapor mass                 
       delmvl(:), &  !  change in vapor mass from matrix inversion       
       delmvo(:), &  !  previous time step change in vapor mass          
         delu(:)     !  most recent change in internal energy            
!
      real(kind=rkd), pointer, save :: &
        delul(:), &  !  most recent change in liquid phase int energy 
        deluo(:), &  !  previous time step change in internal energy     
       dhfsdp(:), &  !  derivative of sat liq enth wrt pressure          
        dhgdm(:), &  !  partial of gas enth wrt fluid mass               
       dhgdmn(:), &  !  partial of gas enth wrt noncondensible mass      
       dhgdmv(:), &  !  partial of gas enth wrt vapor mass               
        dhgdu(:), &  !  partial of gas enth wrt internal energy          
       dhgdul(:), &  !  partial of vap enth wrt liq int energy           
       dhgsdp(:), &  !  derivative of sat vap enth wrt pressure          
       dhlaol(:), &  !  previous time step dhldmn                        
        dhldm(:), &  !  partial of liq enth wrt fluid mass               
       dhldmn(:), &  !  partial of liquid enth wrt noncon mass           
       dhldmv(:), &  !  partial of liq enth wrt vapor mass               
        dhldu(:), &  !  partial of liq enth wrt internal energy          
       dhldul(:), &  !  partial of liq enth wrt liq int energy           
       dhlmol(:), &  !  previous time step dhldm                         
       dhluol(:), &  !  previous time step dhldu                         
       dhlvol(:), &  !  previous time step dhldmv                        
       dhmdmf(:), &  !  partial of mixture enth wrt total mass           
       dhmdmn(:)     !  partial of mixture enth wrt noncon mass          
!
      real(kind=rkd), pointer, save :: &
       dhmdmv(:), &  !  partial of mixture enth wrt vapor mass           
       dhmdum(:), &  !  partial of mixture enth wrt energy               
        dhndm(:), &  !  partial of noncon enth wrt total mass            
       dhndmn(:), &  !  partial of noncon enth wrt noncon mass           
       dhndmv(:), &  !  partial of noncon enth wrt vapor mass            
        dhndu(:), &  !  partial of noncon enth wrt energy                
       dhvaol(:), &  !  previous time step dhvdmn                        
        dhvdm(:), &  !  partial of vapor enth wrt total mass             
       dhvdmg(:), &  !  partial of vapor enth wrt vapor mass             
       dhvdmn(:), &  !  partial of vapor enth wrt noncondensible mass    
        dhvdu(:), &  !  partial of vapor enth wrt energy                 
       dhvmol(:), &  !  previous time step dhvdm                         
       dhvuol(:), &  !  previous time step dhvdu                         
        diamv(:), &  !  equivalent diameter of flow area                 
        dmldm(:), &  !  partial of net mass tran wrt fluid mass          
       dmldmn(:), &  !  partial of net mass transfer wrt noncon mass     
       dmldmv(:), &  !  partial of net mass tran wrt vapor mass          
        dmldu(:), &  !  partial of net mass tran wrt ind energy          
       dmldul(:), &  !  partial of net mass tran wrt liq int energy      
       dpdaol(:)     !  previous time step dpdma                         
!
      real(kind=rkd), pointer, save :: &
        dpdma(:), &  !  partial of p with respect to ma at const u & mf  
        dpdmf(:), &  !  partial of p with respect to mf at const u & ma  
       dpdmol(:), &  !  previous time step dpdmf                         
        dpdmv(:), &  !  partial of pressure wrt vapor mass               
         dpdu(:), &  !  partial of p with respect to u at const mf & ma  
        dpdul(:), &  !  partial of pressure wrt liquid phase energy      
       dpduol(:), &  !  previous time step dpdu                          
       dpdvol(:), &  !  previous time step dpdmv                         
        dprdt(:), &  !  time rate of change of pressure                  
       dprdto(:), &  !  previous time step dprdt                         
       dpvaol(:), &  !  previous time step dpvdma                        
        dpvdm(:), &  !  partial of vapor pressure wrt total mass         
       dpvdma(:), &  !  partial of vapor pressure wrt noncon mass        
       dpvdmv(:), &  !  partial of vapor pressure wrt vapor mass         
        dpvdu(:), &  !  partial of vapor pressure wrt energy             
       dpvmol(:), &  !  previous time step dpvdm                         
       dpvuol(:), &  !  previous time step dpvdu                         
        dqldm(:), &  !  partial of liq htng rate wrt fluid mass          
       dqldmv(:), &  !  partial of liq htng rate wrt vapor mass          
        dqldu(:)     !  partial of liq htng rate wrt ind energy          
!
      real(kind=rkd), pointer, save :: &
       dqldul(:), &  !  partial of liq htng rate wrt liq int energy      
        dtfdm(:), &  !  partial of temperature wrt fluid mass            
       dtfdmg(:), &  !  partial of temperature wrt vapor mass            
       dtfdmn(:), &  !  partial of temperature wrt noncon mass           
        dtfdu(:), &  !  partial of temperature wrt internal energy       
       dvfsdp(:), &  !  derivative of sat liq spec vol wrt pressure      
       dvgsdp(:), &  !  derivative of sat vap spec vol wrt pressure      
        dvldm(:), &  !  partial of liq spec vol wrt fluid mass           
       dvldmv(:), &  !  partial of liq spec vol wrt vapor mass           
        dvldu(:), &  !  partial of liq spec vol wrt int energy           
       dvldul(:), &  !  partial of liq spec vol wrt liq int energy       
       dvvdpv(:), &  !  partial of vaporv wrt vapor pressure (const t)   
       dxmdmn(:), &  !  partial of mixture quality wrt noncon mass       
       dxmdmu(:), &  !  partial of mix quality wrt total fluid mass      
       dxmdmv(:), &  !  partial of quality with respect to vapor mass    
       dxmdum(:), &  !  partial of mix quality wrt internal energy       
       dxncao(:), &  !  previous time step dxncnc                        
       dxncdm(:), &  !  partial of noncon quality wrt total mass         
       dxncdu(:), &  !  partial of noncon quality wrt energy             
       dxncmo(:)     !  previous time step dxncdm                        
!
      real(kind=rkd), pointer, save :: &
       dxncmv(:), &  !  partial of noncon quality wrt vapor mass         
       dxncnc(:), &  !  partial of noncon quality wrt noncon mass        
       dxncuo(:), &  !  previous time step dxncdu                        
       dxvaol(:), &  !  previous time step dxvdmn                        
        dxvdm(:), &  !  partial of xvapor wrt total mass                 
       dxvdmn(:), &  !  partial of xvapor wrt noncon mass                
        dxvdu(:), &  !  partial of xvapor wrt energy                     
       dxvmol(:), &  !  previous time step dxvdm                         
       dxvuol(:), &  !  previous time step dxvdu                         
         elev(:), &  !  elevation at the bottom of the volume            
         elod(:), &  !  (l/d)/(144*gc*a**2)                              
           ew(:), &  !  relative surface roughness                       
        flowa(:), &  !  volume flow area                                 
        flowl(:), &  !  volume flow length                               
       flowra(:), &  !  1/(144*gc*flowa**2)                              
           fm(:), &  !  dm/dt = mass equation                            
        fmass(:), &  !  total mass                                       
        fmold(:), &  !  previous time step dm/dt                         
          fmv(:), &  !  dmv/dt = vapor mass equation                     
       fmvold(:)     !  previous time step dmv/dt                        
!
      real(kind=rkd), pointer, save :: &
          fnc(:), &  !  d(mnc)/dt = noncondensible mass equation         
         fnco(:), &  !  previous time step d(mnc)/dt                     
           fu(:), &  !  du/dt = energy equation                          
          ful(:), &  !  dul/dt = liquid energy equation                  
        fuold(:), &  !  previous time step du/dt                         
         gash(:), &  !  gas specific enthalpy                            
       gashol(:), &  !  previous time step gas enthalpy                  
         gasm(:), &  !  gas mass (vapor+noncondensible)                  
        gasmo(:), &  !  previous time step gas mass                      
         gint(:), &  !  geometric intensity                              
       hliqol(:), &  !  previous time step liquid enthalpy               
        hncol(:), &  !  previous time step nonch                         
        htcif(:), &  !  interfacial heat transfer coeff (btu/hr-ft2-f)   
          hum(:), &  !  volume humidity                                  
           hw(:), &  !  volume specific enthalpy                         
         liqh(:), &  !  liquid specific enthalpy                         
       liqmas(:), &  !  liquid mass                                      
       liqvol(:), &  !  liquid volume                                    
       mdotlg(:), &  !  liquid-to-vapor mass transfer rate parameters for full nonequilibrium 
        mdotx(:)     !  not used                                         
!
      real(kind=rkd), pointer, save :: &
       mdotxo(:), &  !  not used                                         
        mdtdt(:), &  !  mass transfer due to phasic temperature diff     
        nonch(:), &  !  noncondensible specific enthalpy                 
       odmldm(:), &  !  previous time step dmldm                         
       odmldn(:), &  !  previous time step dmldmn                        
       odmldu(:), &  !  previous time step dmldug                        
       odmldv(:), &  !  previous time step dmldmv                        
         oldp(:), &  !  previous time step pressure                      
       oldtem(:), &  !  previous time step temperature                   
       oldvoi(:), &  !  previous time step void fraction                 
       omdtlg(:), &  !  previous time step mdotlg                        
            p(:), &  !  volume pressure                                  
        pncol(:), &  !  previous time step presnc                        
       presnc(:), &  !  noncondensible partial pressure                  
        psatl(:), &  !  not used                                         
       psatpv(:), &  !  not used                                         
        psatv(:), &  !  not used                                         
         qspl(:), &  !  energy equation source term (btu/sec)            
        qspll(:), &  !  net liquid heating rate                          
          rnc(:)     !  noncondensible ideal gas constant                
!
      real(kind=rkd), pointer, save :: &
        sathf(:), &  !  specific enthalpy of saturated liquid            
        sathg(:), &  !  specific enthalpy of saturated gas               
         satp(:), &  !  saturation pressure                              
         satt(:), &  !  saturation temperature                           
        satvf(:), &  !  saturation specific volume of liquid             
        satvg(:), &  !  saturation specific volume of gas                
         spvz(:), &  !  volume average specific volume (ft3/lbm)         
        sumha(:), &  !  sum of heat trans coef*area for cond connected   
       surftn(:), &  !  surface tension                                  
         temp(:), &  !  volume temperature                               
        templ(:), &  !  liquid temperature                               
        tempv(:), &  !  gas temperature                                  
       thcong(:), &  !  gas thermal conductivity                         
       thconl(:), &  !  liquid thermal conductivity                      
            u(:), &  !  total internal energy                            
         uliq(:), &  !  liquid phase internal energy                     
           uw(:), &  !  internal specific volume                         
            v(:), &  !  volume (ft3)                                     
       vaphol(:), &  !  previous time step vaporh                        
       vaporh(:)     !  vapor specific enthalpy                          
!
      real(kind=rkd), pointer, save :: &
       vaporv(:), &  !  vapor specific volume                            
        vgasv(:), &  !  volume average gas velocity                      
        viscg(:), &  !  gas viscosity                                    
        viscl(:), &  !  liquid viscosity                                 
           vl(:), &  !  liquid specific volume                           
        vnonc(:), &  !  noncondensible specific volume                   
        voidn(:), &  !  neutron void = voidv unless profile fit is used  
        voidv(:), &  !  volume void fraction                             
           vs(:), &  !  gas specific volume                              
        vslpv(:), &  !  volume average slip velocity                     
          wgv(:), &  !  volume gas flow                                  
          wlv(:), &  !  volume liquid flow                               
           wq(:), &  !  heat addition to fluid (btu/hr)                  
        wvbar(:), &  !  average volume flow (lbm/sec)                    
         xmfo(:), &  !  previous time step fluid mass                    
          xnc(:), &  !  noncondensible mass / gas mass = noncon quality  
         xnco(:), &  !  previous time step xnc                           
       xnctot(:)     !  noncon mass / total mass                         
!
      real(kind=rkd), pointer, save :: &
         xulo(:), &  !  previous time step liq phase internal energy     
          xuo(:), &  !  previous time step internal energy               
       xvapol(:), &  !  previous time step xvapor                        
       xvapor(:), &  !  vapor mass / water mass                          
       zlqlev(:), &  !  equivalent collapsed liquid level                
         zvol(:)     !  volume height                                    
! 
!  All of the following real pointer arrays are Rank (1:num_vol+1,1:2)
!  Space is allocated and pointers associations are made in subroutine allok8_volumes. For 
!  all of the following 2D arrays the second subscript is used to define the "up" side (=1)
!  and the "down" side (=2) for a volume variable, unless otherwise noted.
!
      integer(kind=ikd), save :: &
         siz_r2vol   !  size of the array allocated for the 2d real vol pointer arrays
 !         
      real(kind=rkd), target, allocatable, save :: r2_vol(:,:)
      real(kind=rkd), pointer, save :: &
       wjsum(:,:)    !  sum of absolute value of all junction flows for upstream side(1) and downstream side (2) of volume                               
!
      integer(kind=ikd), target, allocatable, save :: i_vol(:)
      integer(kind=ikd), pointer, save :: &
         ibub(:), &  !  bubble data index                                
       icndns(:), &  !  mode 20 recondensation flag                      
       idxetr(:), &  !  index to subfile to 2-reg enth tran model
       idxlst(:), &  !  index for connection list (w shs model)
       iequat(:), &  !  equation set flag - model option                 
          ifr(:), &  !  flow regime index                                
        ifric(:), &  !  laminar wall friction correlation index          
         ifrv(:), &  !  volume flow regime index                         
        ihtlg(:), &  !  interphase heat and mass transfer option flag    
         imap(:), &  !  volume orientation flag                          
         ineq(:), &  !  non-equilibrium pressurizer flag                 
        inout(:), &  !  index into kwf array for termininal and initial junction index lists (jun in and out) 
         iptn(:), &  !  piping transport delay time flag                 
       ipumpi(:), &  !  index to pump associated with this volume        
         iqin(:), &  !  heat exchanger calculation index (non-cond)      
        iread(:), &  !  volume data retrieval index                      
        isepr(:), &  !  seperator flag                                   
        ivfit(:), &  !  index for profile fit parameters                 
        ixset(:), &  !  not used                                         
       olvoln(:)     !  input volume number                                      
!
      integer(kind=ikd), pointer, save :: &

        phase(:), &  !  volume phase index                               
        phaso(:)     !  previous time step phase index                   
!
      logical(kind=lkd), target, allocatable, save :: l_vol(:)
      logical(kind=lkd), pointer, save :: &
        lliqs(:), &  !  liquid present or appearing flag (yes if .true.) 
         lncs(:), &  !  noncon present or appearing flag (yes if .true.) 
        lvaps(:), &  !  vapor present or appearing flag (yes if .true.)  
        optv1(:), &  !  energy eqn. solve flag (solve if .false.)        
        optv2(:), &  !  vapor mass eqn. solve flag (solve if .false.)    
        optv3(:)     !  noncon mass eqn solve flag (solve if .false.)    
!
!  ===============================================================================================================================
!  Specifications and space allocations for ttrn related arrays
!  ===============================================================================================================================
!
! 
      integer(kind=ikd), save :: & 
        num_ttrn,  &  !  number of ttrn
       siz_rttrn,  &  !  size of the array allocated for the real ttrn pointer arrays
       siz_ittrn,  &  !  size of the array allocated for the integer ttrn pointer arrays
       siz_lttrn      !  size of the array allocated for the logical ttrn pointer arrays
!
!  The following real and integer pointer arrays have bounds (1:num_ttrn)
!  Space is allocated and pointers associations are made in subroutine allok8_ttrn
!
      real(kind=rkd), target, allocatable, save :: r_ttrn(:) 
      real(kind=rkd), pointer, save :: &
          ein(:), &  !  integrated energy into the volume over time step 
         eino(:), &  !  value of ein at previous converged time step     
         eout(:), &  !  integrated energy out of volume over time step   
        eouto(:), &  !  value of eout at previous converged time step    
        hmino(:), &  !  old inlet integrated product of mass & enthalpy  
       hmouto(:), &  !  old outlet integrated product of mass & enthalpy 
         min1(:), &  !  integrated mass into the volume over time step   
         mino(:), &  !  value of min at previous converged time step     
         mout(:), &  !  integrated mass out of the volume over time step 
        mouto(:), &  !  value of mout at previous converged time step    
        shfts(:), &  !  current mesh displacement                        
       shftso(:), &  !  value of shfts at previous converged time step   
        summi(:), &  !  mass convected into transport volume during current time step                                
        summo(:), &  !  mass convected out of transport volume during current time step                                
        sumui(:), &  !  energy convected into transport volume during current time step                                
        sumuo(:), &  !  energy convected out of transport volume during current time step                                
        sumwv(:)     !  mesh displacement rate of change for current time step                                        
!
      integer(kind=ikd), target, allocatable, save :: i_ttrn(:)
      integer(kind=ikd), pointer, save :: &
       idxmsh(:), &  !  index of the subfile containing mesh enthalpies  
        islug(:), &  !  memory for direction of mesh displacement at beginning of timestep                            
       islugo(:), &  !  value of islug at previous converged time step   
       itvold(:), &  !  old volume number for minor edit region checks   
         mesh(:)     !  number of mesh points in the volume              
!
!  ===============================================================================================================================
!  Specifications and space allocations for mesh related arrays
!  ===============================================================================================================================

      integer(kind=ikd), save :: &
       siz_rttmesh, &  !  size of real array allocated for mesh related arrays
          num_mesh     !  maximum number of mesh points in any volume
!
      real(kind=rkd), target, allocatable, save :: r_ttmesh(:)  !  real target array for mesh related pointer arrays
!
      real(kind=rkd), pointer, save :: &
             hmesh(:), &  !  interval mesh point value                        
            hmesho(:), &  !  old values of hmesh                              
            hmeshs(:)     !                                                   
!            
      contains
!      
!  ===============================================================================================================================
      subroutine allok8_volumes
!
!  Subroutine to allocate space and set pointers for vol related arrays
!     Note: num_vol must be defined before allok8_vol is called
!
!     allok8_volumes is called from subroutine invol
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!      
      use s_r2_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, n2d, status
      logical(kind=lkd) allocatd
!
      parameter (n2d   = 2)
!      
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)  !  dummy real pointers
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)  !  dummy integer pointers
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)  !  dummy logical pointer
!
      if (num_vol .le. 0) then
         write (6,110) num_vol
         call fail
         return
      endif
!
      if (allocated (l_vol)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  One extra volume is added as dummy for property calculations particularly required for genopt
!  care must be taken when using num_vol as counter to move explicitly through allocatable space i_vol/r_vol/l_vol
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_vol+1, i2,  afdai, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, areaif, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, armaso, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, armass, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   aved, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   avex, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, csbpgm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, csbpnc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  csubp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, csubpf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, csubpg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  delma, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, delmal, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, delmao, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  delmf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, delmfo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  delmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, delmvl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, delmvo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   delu, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2,  delul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  deluo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhfsdp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhgdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhgdmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhgdmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhgdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhgdul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhgsdp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhlaol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhldm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhldmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhldmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhldu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhldul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhlmol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhluol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhlvol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhmdmf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhmdmn, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2, dhmdmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhmdum, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhndm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhndmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhndmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhndu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhvaol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhvdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhvdmg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhvdmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dhvdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhvmol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dhvuol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  diamv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dmldm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dmldmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dmldmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dmldu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dmldul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpdaol, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2,  dpdma, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dpdmf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpdmol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dpdmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   dpdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dpdul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpduol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpdvol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dprdt, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dprdto, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpvaol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dpvdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpvdma, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpvdmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dpvdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpvmol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dpvuol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dqldm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dqldmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dqldu, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2, dqldul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dtfdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dtfdmg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dtfdmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dtfdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dvfsdp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dvgsdp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dvldm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dvldmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dvldu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dvldul, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dvvdpv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxmdmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxmdmu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxmdmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxmdum, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncao, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncmo, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2, dxncmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncnc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxncuo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxvaol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dxvdm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxvdmn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  dxvdu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxvmol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, dxvuol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   elev, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   elod, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     ew, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  flowa, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  flowl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, flowra, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     fm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  fmass, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  fmold, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    fmv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, fmvold, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2,    fnc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   fnco, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     fu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    ful, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  fuold, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   gash, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, gashol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   gasm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  gasmo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   gint, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, hliqol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  hncol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  htcif, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    hum, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     hw, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   liqh, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, liqmas, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, liqvol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, mdotlg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  mdotx, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2, mdotxo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  mdtdt, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  nonch, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, odmldm, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, odmldn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, odmldu, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, odmldv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   oldp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, oldtem, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, oldvoi, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, omdtlg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,      p, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  pncol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, presnc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  psatl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, psatpv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  psatv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   qspl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  qspll, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    rnc, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2,  sathf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  sathg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   satp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   satt, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  satvf, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  satvg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   spvz, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  sumha, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, surftn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   temp, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  templ, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  tempv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, thcong, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, thconl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,      u, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   uliq, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     uw, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,      v, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, vaphol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, vaporh, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2, vaporv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  vgasv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  viscg, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  viscl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     vl, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  vnonc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  voidn, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  voidv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     vs, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  vslpv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    wgv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    wlv, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,     wq, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,  wvbar, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   xmfo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    xnc, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   xnco, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, xnctot, r_vol)
!         
         call r_pntr (allocatd, num_vol+1, i2,   xulo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,    xuo, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, xvapol, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, xvapor, r_vol)
         call r_pntr (allocatd, num_vol+1, i2, zlqlev, r_vol)
         call r_pntr (allocatd, num_vol+1, i2,   zvol, r_vol)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rvol   = i2
!
            allocate (r_vol(siz_rvol), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rvol, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_vol
            block  = 'r_vol'
!            
            if (num_vol /= 1 ) call build_restart_block_list (block, rdum, idum, ldum, siz_rvol, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_vol      = 0.0d0            
         end if
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         i2       = 0
!
         call r2_pntr (allocatd, num_vol+1, n2d, i2, wjsum, r2_vol)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_r2vol   = i2
!
            allocate (r2_vol(siz_r2vol,1:n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_r2vol, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            r2dum  => r2_vol
            block  = 'r2_vol'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_r2vol, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r2_vol      = 0.0d0
         end if
      end do      
!                  
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_vol+1, i2,   ibub, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, icndns, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, idxetr, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, idxlst, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, iequat, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,    ifr, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  ifric, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,   ifrv, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  ihtlg, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,   imap, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,   ineq, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  inout, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,   iptn, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, ipumpi, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,   iqin, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  iread, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  isepr, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  ivfit, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  ixset, i_vol)
         call i_pntr (allocatd, num_vol+1, i2, olvoln, i_vol)
!         
         call i_pntr (allocatd, num_vol+1, i2,  phase, i_vol)
         call i_pntr (allocatd, num_vol+1, i2,  phaso, i_vol)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ivol   = i2
!
            allocate (i_vol(siz_ivol), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ivol, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_vol
            block  = 'i_vol'
!            
            if (num_vol /= 1 ) call build_restart_block_list (block, rdum, idum, ldum, siz_ivol, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_vol      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_vol+1, i2,  lliqs, l_vol)
         call l_pntr (allocatd, num_vol+1, i2,   lncs, l_vol)
         call l_pntr (allocatd, num_vol+1, i2,  lvaps, l_vol)
         call l_pntr (allocatd, num_vol+1, i2,  optv1, l_vol)
         call l_pntr (allocatd, num_vol+1, i2,  optv2, l_vol)
         call l_pntr (allocatd, num_vol+1, i2,  optv3, l_vol)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_lvol   = i2
!
            allocate (l_vol(siz_lvol), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_lvol, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_vol
            block  = 'l_vol'
!            
            if (num_vol /= 1 ) call build_restart_block_list (block, rdum, idum, ldum, siz_lvol, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_vol      = .false.
         endif
      enddo
!
      return                
!      
  100 format (' ******** Error allocating space for ',a8,' volume arrays (allok8_volumes).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for volume arrays (allok8_volumes). '/ &
          10x,'Variable num_vol = ',i3, ', must be > 0.')
!
      end subroutine allok8_volumes
!
!  ===============================================================================================================================
!
      subroutine allok8_ttrn  
!
!  Subroutine to allocate space and set pointers for temperature transport delay related arrays
!     allok8_ttrn is called from subroutine XXXXX 
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i2, l, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_ttrn)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_ttrn, i2,    ein, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,   eino, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,   eout, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  eouto, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  hmino, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2, hmouto, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,   min1, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,   mino, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,   mout, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  mouto, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  shfts, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2, shftso, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  summi, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  summo, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  sumui, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  sumuo, r_ttrn)
         call r_pntr (allocatd, num_ttrn, i2,  sumwv, r_ttrn)         
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rttrn   = i2
!
            allocate (r_ttrn(siz_rttrn), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rttrn, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_ttrn
            block  = 'r_ttrn'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rttrn, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_ttrn      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_ttrn, i2, idxmsh, i_ttrn)
         call i_pntr (allocatd, num_ttrn, i2,  islug, i_ttrn)
         call i_pntr (allocatd, num_ttrn, i2, islugo, i_ttrn)
         call i_pntr (allocatd, num_ttrn, i2, itvold, i_ttrn)
         call i_pntr (allocatd, num_ttrn, i2,   mesh, i_ttrn)
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_ittrn   = i2
!
            allocate (i_ttrn(siz_ittrn), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ittrn, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_ttrn
            block  = 'i_ttrn'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ittrn, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_ttrn      = 0
         endif
      enddo
!
  100 format (' ******** Error allocating space for ',a8,' ttrn arrays (allok8_ttrn).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_ttrn
!     
!  ===============================================================================================================================
!
      subroutine allok8_ttmesh
!
!  Subroutine to allocate space and set pointers for mesh related arrays
!     allok8_ttmesh is called from subroutine invol
! 
      use kind_specs 
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, i1, i2, l, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_ttmesh)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (num_mesh .le.0) then
         return
      endif
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_mesh, i2,    hmesh, r_ttmesh)
         call r_pntr (allocatd, num_mesh, i2,   hmesho, r_ttmesh)
         call r_pntr (allocatd, num_mesh, i2,   hmeshs, r_ttmesh)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rttmesh   = i2
!
            allocate (r_ttmesh(siz_rttmesh), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rttmesh, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_ttmesh
            block  = 'r_ttmesh'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rttmesh, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_ttmesh      = 0.0d0
         endif
      enddo     
!       
!  initialize idxmsh for all temperature transport volumes
!
      i1     = 1
      do i = 1, num_ttrn
         idxmsh(i)  = i1
         i1         = i1 + mesh(i)
      enddo
!
  100 format (' ******** Error allocating space for ',a8,' ttmesh arrays (allok8_ttmesh).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!          
      end subroutine allok8_ttmesh
!
      end module volumes
